#!/bin/sh

#--------------------------------------------------------------------#
#Create Makefile for cpmd.x program.                                 #
#--------------------------------------------------------------------#
# Options for CPPFLAG                                                #
#  -DLAPACK  Use LAPACK routine, you must have this option and       #
#            the LAPACK library                                      #
#  -DFFT_DEFAULT Use default FFT routine in the code (from Goedecker)#
#  -DFFT_ESSL    Use FFT ESSL (need ESSL library)                    #
#  -DFFT_HP      Use FFT HP (Hewlet-Packard)                         #
#  -DFFT_YMP     Use CRAY YMP, C94, T90 FFT routines                 #
#  -DFFT_T3D     Use CRAY T3D or T3E FFT routines                    #
#  -DFFT_X1D     Use CRAY X1 FFT routines                            #
#  -DPARALLEL    For PARALLEL computers, you have to indicate        #
#                the parallel library with :                         #
#  -DMP_LIBRARY=__MPI   Message Passing Interface library            #
#  -D__VECTOR    For vectorial computers                             #
#  -DPOINTER8    If the addresses are coded in INTEGER*8             #
#                (for LOC and MALLOC)                                #
#  -DMALLOC8     If the argument of MALLOC is INTEGER*8              #
#  -D__NOINT8    If the compiler does not use INTEGER*8 or           #
#                if the function BTEST and IBTEST do not like it     #
#--------------------------------------------------------------------#
#Create Makefile for cpmd.x program.

#Display all configurations
Message () {
cat << END >&2

./Configure.nocpp  [options] Name_of_Configuration
where the name of the configuration is:
**** Hint **** :
This is the configure script for not using "cpp",
but to have the Fortran compiler use the *.F files directly by 
passing the CPPFLAGS directly to the FORTRAN compiler
OBJECTS_GROMOS and MM_Interface setup not changed yet
****************************************+
END
\ls CONFIGURE >&2
cat << END >&2

To display additional information about a configuration 
type: ./Configure.nocpp -info SUN
Use Configure.nocpp in the directory where the SOURCE FILES are.
Ex: ./Configure.nocpp SUN > Makefile
See the description of further options with './Configure.nocpp -help'

To create a new configuration, add a new file in the CONFIGURE directory.

END
}


#Help
Help () {
Message
cat << END >&2

Description of options:
   -debug    (-d) Give special options for debugging
   -help     (-h) Print this message
   -info     (-i) Print additional information about a machine configuration
   -makefile (-m) Create the file Makefile in DEST directory
                  instead of using standard output
   -verbose  (-v) Display the name of files for the dependencies
   -qmmm          Generates a makefile for QMMM 
   -VAR=value     Use the value of VAR in the makefile
            Ex: -DEST=destdir specifies where the compilation will be
   You can use:
     FFLAGS   Fortran flags
     LFLAGS   Link editor flags
     CFLAGS   C compiler flags
     CPP      Preprocessor program
     CPPFLAGS CPP flags
     FC       Fortran compiler
     LD       Link editor
     AR       Archive library maintainer
     RANLIB   Converts archives to random libraries
     SRC      Source files directory (default=.)
     DEST     Object files directory (default=.)
              Use to write irat.inc in the right directory
     BIN      Use to put the cpmd.x executable (default=.)
Note: If you want to compile cpmd.x in a different directory
      than the source files, use:
         Configure.nocpp -DEST=destdir SUN > destdir/Makefile
      or Configure.nocpp -makefile -DEST=destdir SUN
      and then cd destdir; make
      In this case, SRC is equal to source_dir_pathname
      except if you specify SRC.
END
}

#No argument: Error
if [ $# -eq 0 ]; then
  Message
 exit 2
fi

#Is it help option or debug option
info=0
opt=0
i=1
while [ $i -le $# ];
do
  #Select the i-th argument
  eval option=\$$i
  case $option in
    -qmmm|-q)
      qmmm=1
      ;;
    -debug|-d)
      debug=1
      ;;
    -help|-h)
      Help
      exit 0
      ;;
    -makefile|-m)
      makefile=1
      ;;
    -verbose|-v)
      verbose=1
      ;;
    -info|-i)
      info=1
      ;;
    -DEST=*)
      opt=1
      DEST=`echo $option | cut -c2- | cut -d= -f 2`
      ;;
    # hidden option for 'make reconfig'
    -reconfig|-r)
      reconfig=1
      ;;
    -*=*)
      opt=1
      ;;
    -*)
      echo "Unknow option: $option" >&2
      Message
      exit 1
      ;;
    *)
      Configuration=$option
      ;;
  esac
  i=`expr $i + 1`
done

#No configuration given
if [ -z "${Configuration}" ]; then
  echo "Configure.nocpp: No configuration name" >&2
  Message
  exit 2
fi

#Check if the Configuration does exist.
if [ -f CONFIGURE/${Configuration} ]; then
   . CONFIGURE/${Configuration}
else
   echo "Configure.nocpp: Unknown configuration '${Configuration}'" >&2
   Message
   exit 2
fi

#Print info message and exit
if [ $info -eq 1 ]
then
   echo
   echo "Additional information for configuration: ${Configuration}"
   echo
   grep '^#INFO#' CONFIGURE/${Configuration} | cut -c7-
   exit 0
fi

#--------------------------------------------------------------------#
#Configurations                                                      #
#--------------------------------------------------------------------#

CC='cc'
CPPFLAGS_GROMOS='-DEWALD -DEWATCUT -DHAT_SHAPE -UNPACKED_GRID'

#By default, we try to use nawk
if [ -x /usr/bin/nawk ]; then
   AWK='/usr/bin/nawk'
else
   AWK='awk'
fi

#Check if ${AWK} does exist.

haveawk=0
echo 1 | ${AWK} '{ }' > /dev/null 2>&1 && haveawk=1 || haveawk=0
if [ ${haveawk} = 0 ]; then
  echo "The command ${AWK} does not exist!" >&2
  exit 3
fi

#--------------------------------------------------------------------#
#End of Configurations                                               #
#--------------------------------------------------------------------#

# File descriptor usage:
# 0 standard input
# 1 file creation (standard output or makefile)
# 2 errors and warnings
# 3 Makefile if $makefile or &1

echo "Default configuration for $Configuration." >&2
if [ $debug ]; then
   echo "Debug option is used." >&2
fi

#Default DEST is .
DEST=${DEST:-'.'}

#Check if DEST directory exists.
if [ ! -d ${DEST} ]; then
   echo "The directory ${DEST} does not exist" >&2
  exit 1
fi
if [ ! -w ${DEST} ]; then
  echo "The directory ${DEST} has no write permission" >&2
  exit 1
fi

# save old makefile for reconfig
if [ $reconfig ]; then
   if [ ! -e ${DEST}/Makefile ]; then 
      echo "There is no old Makefile in ${DEST}. Reconfig not possible." >&2
      exit 1
   fi
   mv ${DEST}/Makefile ${DEST}/Makefile.bak
fi
#Create Makefile if output
if [ $makefile ]; then
  echo "Creation of Makefile: ${DEST}/Makefile" >&2
  exec 3>${DEST}/Makefile
else
  echo "Use standard output for the Makefile." >&2
  exec 3>&1
fi

# for make reconfig, we take the header section of the old makefile
# instead of the default and only recreate the second part from scratch.
if [ $reconfig ]; then
   echo '#----------------------------------------------------------------------------'    >&3
   sed -n -e '/Makefile for cpmd.x/,/End of Personal Configuration/p' ${DEST}/Makefile.bak >&3
   echo '#----------------------------------------------------------------------------'    >&3
else
cat << END >&3
#----------------------------------------------------------------------------
# Makefile for cpmd.x (plane wave electronic calculation)
# Configuration: ${Configuration}
# Creation of Makefile: `date '+%b %e %Y'`
# on `uname -a`
# Author: `who am i | cut -d' ' -f 1`
#----------------------------------------------------------------------------
#
SHELL = /bin/sh
#
#--------------- Default Configuration for $Configuration ---------------
SRC  = .
DEST = .
BIN  = .
END

if [ $qmmm ]; then
  if [ -f QMMM_OBJECTS ]; then
    QMMM_FLAGS='-D__GROMOS'
  else
    printf "\nThe file QMMM_OBJECTS does not exist.\n" >&2
    echo "The file QMMM_OBJECTS does not exist" >&2
    exit 1
  fi
else
  QMMM_FLAGS=' '
  CPPFLAGS_GROMOS=' '
  FFLAGS_GROMOS=' '
fi

cat << END >&3
#QMMM_FLAGS = -D__QMECHCOUPL
#QMMM_LIBS  = -L. -lmm
CPPFLAGS = ${CPPFLAGS} ${QMMM_FLAGS}
MYFFLAGS = ${MYFFLAGS} \$(CPPFLAGS)
IPAFLAGS = ${IPAFLAGS} 
FFLAGS = ${FFLAGS} \$(CPPFLAGS)
LFLAGS = ${LFLAGS} \$(QMMM_LIBS)
CFLAGS = ${CFLAGS} \$(CPPFLAGS)
CPP = ${CPP}
NOOPT_FLAG = ${NOOPT_FLAG}
END

if [ $qmmm ]; then
  cat << END >&3
CPPFLAGS_GROMOS = ${CPPFLAGS_GROMOS}
FFLAGS_GROMOS = ${FFLAGS_GROMOS}
END
fi

cat << END >&3
CC = ${CC}
FC = ${FC}
LD = ${LD}
AR = ${AR}
#----------------------------------------------------------------------------
END

#There is Personal Configuration.
if [ $opt -eq 1 ]; then
  printf "Personal configurations..." >&2
  
  #Check the options and personal variables 
  cat << END >&3
  
#----------------------------------------------------------------------------
# Personal Configuration
#----------------------------------------------------------------------------
END

  i=1
  while [ $i -le $# ];
  do
    eval option=\$$i
    case $option in
      -DEST=*)
	       #Special case for DEST (only used for makefile)
	       ;;
      -*=*)
	    var=`echo $option | cut -c2- | cut -d= -f 1`
	    val=`echo $option | cut -c2- | cut -d= -f 2`
	    echo "$var = $val" >&3
	    eval $var='$val'
	    ;;
      *)
	 ;;
    esac
    i=`expr $i + 1`
  done
  
  #Default personal option
  if [ $DEST != "." ]; then
    if [ -z "$SRC" ]; then
      SRC=`pwd`
      echo "SRC = $SRC" >&3
    fi
  fi
  
  if [ -n "$SRC" ]; then
    echo "FC = $FC -I. -I\$(SRC)" >&3
  fi
  cat << END >&3
#----------------------------------------------------------------------------
# End of Personal Configuration
#----------------------------------------------------------------------------
END
  echo "done." >&2
  if [ -n "$BIN" ]; then
    echo "cpmd.x executable will be in: $BIN" >&2
  fi
  
fi #End of Personal Configuration.

fi #end of if [ ! $reconfig ]

#Default SRC is .
SRC=${SRC:-'.'}
# record how we called configure for make reconfig
echo "CFGDEST = $DEST" >&3
echo "CFGMACH = $Configuration" >&3 

echo "The source directory is: $SRC" >&2
echo "The object directory is: $DEST" >&2
  

#Now we change irat if necessary
if [ -f ${DEST}/irat.inc ]; then
  iratfile="${DEST}/irat.inc"
else
  iratfile="irat.inc"
fi
irat=`${AWK} -F= '/PARAMETER/ { print substr($2,1,1) }' ${iratfile}`
if [ ${irat} != ${IRAT} -o ! -f ${DEST}/irat.inc ]; then
  echo "We change the file $iratfile " \
   "[old irat=${irat}, new irat=${IRAT}]" >&2
  if [ -w /dev/null ]; then
    exec 4> /dev/null
  else
    exec 4>&2
  fi
  irw=1; if [ ! -w ${iratfile} ]; then chmod u+w ${iratfile}; irw=0; fi
  sed -e "s/IRAT=${irat}/IRAT=${IRAT}/g" ${iratfile} > ${DEST}/irat.temp
  mv ${DEST}/irat.temp ${DEST}/irat.inc
  iratchk=`awk -F= '/PARAMETER/ { print substr($2,1,1) }' ${DEST}/irat.inc`
    if [ $iratchk -eq $IRAT ]; then
      echo "The file ${DEST}/irat.inc has been correctly changed." >&2
    else
      echo "ERROR: could not change file \"irat.inc\"" >&2
      exit 1
    fi
    if [ $irw -eq 0 ]; then chmod u-w ${iratfile}; fi
else
  echo "The file ${DEST}/irat.inc is correct." >&2
fi

#We change gromos.h if necessary
if [ -f ${DEST}/gromos.h ]; then
  gromosfile="${DEST}/gromos.h"
else
  gromosfile="gromos.h"
fi
if [ ! -f ${gromosfile} ]; then
  echo "The file ${gromosfile} does not exist." >&2
  exit 1
fi
comm=`awk 'BEGIN{comm=0} 
     {if (index(toupper($0),"INCLUDE") != 0) {if (toupper($1)=="c" || $1=="!") comm=1}} 
     END{ print comm}' ${gromosfile}`   
if [ $qmmm ]; then
  if [ $comm -eq 1 -o ! -f ${DEST}/gromos.h ]; then
    #if gromos not writable give the good right.
    if [ -f ${DEST}/gromos.h -a ! -w ${DEST}/gromos.h ]; then
      echo "+ chmod u+w ${DEST}/gromos.h" >&2
      chmod u+w ${DEST}/gromos.h
    fi
    cat << END > ${DEST}/gromos.h
      include 'Gromos/toposz.h'
      include 'Gromos/topoar.h'
      include 'Gromos/coordsz.h'
      include 'Gromos/box.h'     
END
    echo "The file ${DEST}/gromos.h is changed." >&2
  else
    echo "The file ${DEST}/gromos.h is consistent with qmmm." >&2
  fi
else
  if [ $comm -eq 0 -o ! -f ${DEST}/gromos.h ]; then
    #if gromos not writable give the good right.
    if [ -f ${DEST}/gromos.h -a ! -w ${DEST}/gromos.h ]; then
      echo "+ chmod u+w ${DEST}/gromos.h" >&2
      chmod u+w ${DEST}/gromos.h
    fi
    cat << END > ${DEST}/gromos.h
!     include 'Gromos/toposz.h'
!     include 'Gromos/topoar.h'
!     include  'Gromos/coordsz.h'
!     include  'Gromos/box.h'     
END
    echo "The file ${DEST}/gromos.h is changed." >&2
  else
    echo "The file ${DEST}/gromos.h is consistent with normal qm." >&2
  fi
fi

#Now DEST is always ./
DEST='.'

#Default SRC
SRC=${SRC:-'.'}

if [[ -n $MODOBJECTS ]]; then
  printf "Add MODOBJECTS (object files)..." >&2
  printf "#  LIST OF MODULE FILES\n#\n" >&3
  printf "MODOBJECTS = ${MODOBJECTS}\n\n" >&3
  echo "done."  >&2
fi

#Print OBJECTS
if [ -f OBJECTS ]; then
  printf "Add OBJECTS (object files)..." >&2
  cat OBJECTS >&3
  echo "done."  >&2
else
  echo "The file OBJECTS does not exist" >&2
  exit 1
fi

#QMMM OBJECTS
if [ $qmmm ]; then
  printf "Add QMMM_OBJECTS (object files)..." >&2
  cat QMMM_OBJECTS >&3
  echo "done."  >&2
fi

#Include files
printf "Add INCFILES (include files)..." >&2
IncludeFile=`ls *.inc`
printf "INCFILES = " >&3
i=0
for name in ${IncludeFile}
do
  if [ $i -eq 6 ]; then
    printf "\\\\\n           " >&3
    i=0
  fi
  i=`expr $i + 1`
  printf "%s " ${name} >&3
done
printf "\n\n" >&3
echo "done." >&2

#QMMM Include files
if [ $qmmm ]; then
  printf "Add  Gromos INCFILES (include files)..." >&2
  IncludeFile=`ls Gromos/*.h`
  printf "QMMM_INCFILES = " >&3
  i=0
  for name in ${IncludeFile}
  do
    if [ $i -eq 6 ]; then
      printf "\\\\\n           " >&3
      i=0
    fi
    i=`expr $i + 1`
    printf "%s " ${name} >&3
  done
  printf "\n\n" >&3
  echo "done." >&2  
  printf "INCFILES = \$(INCFILES) \$(QMMM_INCFILES) \n" >&3 
fi
printf "Add explicit rules..." >&2
if [ $qmmm ]; then
  OBJ_GRM='$(OBJECTS_GROMOS)'
else
  OBJ_GRM=' '
fi
cat << END >&3
#----------------------------------------------------------------------------
# Compile cpmd.x
#----------------------------------------------------------------------------
cpmd.x : \$(MODOBJECTS) \$(OBJECTS) ${OBJ_GRM} \$(OBJ_CC)
	 rm -f timetag.f
	 \$(CPP) \$(CPPFLAGS) \$(SRC)/timetag.F \$(DEST)/timetag.f
	 \$(FC) -c \$(FFLAGS) \$(DEST)/timetag.f
	 rm -f cpmd.x
	 if [ "\$(BIN)" != "." ]; then ln -s \$(BIN)/cpmd.x cpmd.x; fi
	 \$(LD) -o \$(BIN)/cpmd.x timetag.o \$(MODOBJECTS) \$(OBJECTS) \$(OBJ_CC) \$(LFLAGS) ${OBJ_GRM}

#----------------------------------------------------------------------------
# Generate library libcpmd.a
#----------------------------------------------------------------------------
lib : \$(OBJ_LIB)
	 rm -f timetag.f
	 \$(CPP) \$(CPPFLAGS) \$(SRC)/timetag.F \$(DEST)/timetag.f
	 \$(FC) \$(FFLAGS) \$(DEST)/timetag.f
	 \$(AR) libcpmd.a timetag.o \$(OBJ_LIB)
	 \$(RANLIB) libcpmd.a

#----------------------------------------------------------------------------
# Generate the manual
#----------------------------------------------------------------------------
doc : manual.ps

docpdf: manual.pdf

manual.ps: manual.dvi
	dvips -o \$@ manual.dvi

manual.dvi: \$(SRC)/manual.tex
	rm -f manual.aux manual.ilg manual.log manual.idx \
		manual.ind  manual.out  manual.toc
	latex \$(SRC)/manual.tex
	latex \$(SRC)/manual.tex
	makeindex manual
	latex \$(SRC)/manual.tex
	latex \$(SRC)/manual.tex

manual.pdf: \$(SRC)/manual.tex
	rm -f manual.aux manual.ilg manual.log manual.idx \
		manual.ind  manual.out  manual.toc
	pdflatex \$(SRC)/manual.tex
	pdflatex \$(SRC)/manual.tex
	makeindex manual
	pdflatex \$(SRC)/manual.tex
	pdflatex \$(SRC)/manual.tex

#----------------------------------------------------------------------------
# Create a new makefile with new dependencies but retain the configuration.
#----------------------------------------------------------------------------

reconfig:
	( cd \$(SRC) ; ./Configure.nocpp -r -m -SRC=\$(SRC) -DEST=\$(CFGDEST) \$(CFGMACH) ; cd \$(CFGDEST) )

#----------------------------------------------------------------------------
# Remove all *.o and *.f
#----------------------------------------------------------------------------
clean : 
	 rm -f \$(OBJECTS) \$(OBJ_CC) 
	 rm -f *~ .*~ *.o *.f *.L
	 rm -f *.aux *.brf *.dvi *.idx *.ilg *.ind *.log
	 rm -f *.out *.ps *.pdf *.toc 
	 rm -f ./Gromos/*.f ./Gromos/*.o

#----------------------------------------------------------------------------
# Explicit rules
#----------------------------------------------------------------------------
.SUFFIXES:
.SUFFIXES: .o .F

\$(MODOBJECTS):
	\$(FC) \$(MYFFLAGS) -Omodinline \$(DEST)/\$*.F -c -o \$(DEST)/\$*.o

\$(OBJECTS):\$(MODOBJECTS)
	\$(FC) \$(FFLAGS) \$(DEST)/\$< -c -o  \$(DEST)/\$@

\$(OBJ_CC) :
	\$(CC) \$(CPPFLAGS) \$(CFLAGS) -c \$(SRC)/\$(@:.o=.c)
END

# append rules for objects with disabled optimization.
if [ -n "$NOOPT_OBJS" ] 
then
    for o in $NOOPT_OBJS
    do \
        echo "$o : " >&3
        echo "	\$(FC)  \$(NOOPT_FLAG) \$(DEST)/\$< -o  \$(DEST)/\$@" >&3
    done
fi

if [ $qmmm ]; then
cat << END >&3
\$(OBJECTS_GROMOS:.o=.f) : 
	rm -f \$@
	\$(CPP) \$(CPPFLAGS_GROMOS) \$(SRC)/\$(@:.f=.F) \$(DEST)/\$@
\$(OBJECTS_GROMOS) :
	\$(FC) \$(FFLAGS_GROMOS) \$(DEST)/\$< -o \$(DEST)/\$@
END
fi

#----------------------------------------------------------------------------

cat << END >&3
#----------------------------------------------------------------------------
# Dependencies
#----------------------------------------------------------------------------
END
echo "done." >&2

printf "Create dependencies..." >&2
FortranFiles=`ls *.F  *.c `

for name in ${FortranFiles}
do
  if [ $verbose ]; then
    printf "[%s]" $name
  fi
  ${AWK} -v qmmm=${qmmm} 'NR==1 { Ninclude = 2;
               OldInclude["mpif.h"] = 0;
               MaxLength=60;
               ll = length(FILENAME);
               prefix = substr(FILENAME,1,ll-2);
               suffix = substr(FILENAME,ll-1,2);
               src    = "$(SRC)";
               dest = "$(DEST)";
               #To have the quote.
               quote = sprintf("%c",39);
#              if (suffix == ".F") {
#                line = sprintf("%s.f:", prefix);
#                printf "%-16s%s/%s%s\n", line,src,prefix,suffix;
#                suffix = ".f";
#              }
               line = sprintf("%s.o:", prefix);
               if (suffix == ".c") {
                 line = sprintf("%-16s%s/%s%s",line,src,prefix,suffix);
               } else {
                 line = sprintf("%-16s%s%s",line,prefix,suffix);
               }
             }
       /^[ ]*INCLUDE/ || /^[ ]*include/ {
               split($0,a,quote);
               Name = a[2];
               no = 0;
               if (Ninclude != 0) {
                 for ( Include in OldInclude ) {
                   if (Include == Name ) {
                     no = 1;
                     break;
                   }
                 }
               }
               if((Name == "gromos.h") && (qmmm == 1)) 
                 Name=sprintf("Gromos/toposz.h $(SRC)/Gromos/topoar.h $(SRC)/Gromos/coordsz.h $(SRC)/Gromos/box.h")
               if (no != 1) {
                 Ninclude=Ninclude+1;
                 OldInclude[Name] = 0;
                 lline = length(line);
                 if (lline+2 >= MaxLength) {
                   printf "%s \\\n", line;
                   line = sprintf("%-15s"," ");
                 }
                 if (Name == "irat.inc") {
                   line=sprintf("%s %s/%s",line,dest,Name)
                 } else {
                   line=sprintf("%s %s/%s",line,src,Name)
                 }
               }
       }
       END   { printf "%s\n\n", line }' ${name} >&3
done                               



echo "done dep main."  >&2

if [ $qmmm ]; then
  printf "create gromos dependencies \n" >&2 
  FortranFiles=`ls Gromos/*.F `

  for name in ${FortranFiles}
  do
    if [ $verbose ]; then
      printf "[%s]" $name
    fi
    awk 'NR==1 { Ninclude = 2;
               OldInclude["mpif.h"] = 0;
               MaxLength=60;
               ll = length(FILENAME);
               prefix = substr(FILENAME,1,ll-2);
               suffix = substr(FILENAME,ll-1,2);
               src    = "$(SRC)/Gromos";
               dest = "$(DEST)";
               #To have the quote.
               quote = sprintf("%c",39);
               if (suffix == ".F") {
                 src    = "$(SRC)";
#                 line = sprintf("%s.f:", prefix);
#                 printf "%-16s%s/%s%s\n", line,src,prefix,suffix;
#                 suffix = ".f";
                 src    = "$(SRC)/Gromos";
               }
               line = sprintf("%s.o:", prefix);
               if (suffix == ".c") {
                 line = sprintf("%-16s%s/%s%s",line,src,prefix,suffix);
               } else {
                 line = sprintf("%-16s%s%s",line,prefix,suffix);
               }
             }
       /^[ ]*INCLUDE/ || /^[ ]*include/ {
               split($0,a,quote);
               Name = a[2];
               no = 0;
               if(Name=="") no=1;
               if(Name==" ") no=1;
               if (Ninclude != 0) {
                 for ( Include in OldInclude ) {
                   if (Include == Name ) { 
                     no = 1;
                     break;
                   }
                 }
               }
               if (no != 1) {
                 Ninclude=Ninclude+1;
                 OldInclude[Name] = 0;
                 lline = length(line);
                 if (lline+2 >= MaxLength) {
                   printf "%s \\\n", line;
                   line = sprintf("%-15s"," ");
                 }
                 if (Name == "irat.inc") {
                   line=sprintf("%s %s/%s",line,dest,Name)
                 } else {
                   line=sprintf("%s %s/%s",line,src,Name)
                 }
               }
       }
       END   { printf "%s\n\n", line }' ${name} >&3
  done


  FortranFiles=`ls MM_Interface/*.F`
  printf "create mm_interface dependencies \n" >&2 

  for name in ${FortranFiles}
  do
    if [ $verbose ]; then
      printf "[%s]" $name
    fi
    awk 'NR==1 { Ninclude = 2;
               OldInclude["mpif.h"] = 0;
               MaxLength=60;
               ll = length(FILENAME);
               prefix = substr(FILENAME,1,ll-2);
               suffix = substr(FILENAME,ll-1,2);
               src    = "$(SRC)/MM_Interface";
               dest = "$(DEST)";
               #To have the quote.
               quote = sprintf("%c",39);
               if (suffix == ".F") {
                 src    = "$(SRC)";
#                 line = sprintf("%s.f:", prefix);
#                 printf "%-16s%s/%s%s\n", line,src,prefix,suffix;
#                 suffix = ".f";
                 src    = "$(SRC)/MM_Interface";
               }
               line = sprintf("%s.o:", prefix);
               if (suffix == ".c") {
                 line = sprintf("%-16s%s/%s%s",line,src,prefix,suffix);
               } else {
                 line = sprintf("%-16s%s%s",line,prefix,suffix);
               }
             }
       /^[ ]*INCLUDE/ || /^[ ]*include/ {
               split($0,a,quote);
               Name = a[2];
               no = 0;
               if (Ninclude != 0) {
                 for ( Include in OldInclude ) {
                   if (Include == Name ) { 
                     no = 1;
                     break;
                   }
                 }
               }
               if (no != 1) {
                 Ninclude=Ninclude+1;
                 OldInclude[Name] = 0;
                 lline = length(line);
                 if (lline+2 >= MaxLength) {
                   printf "%s \\\n", line;
                   line = sprintf("%-15s"," ");
                 }
                 if (Name == "irat.inc") {
                   line=sprintf("%s %s/%s",line,dest,Name)
                 } else {
                   line=sprintf("%s %s/%s",line,src,Name)
                 }
               }
       }
       END   { printf "%s\n\n", line }' ${name} >&3
  done
cat << END >&3

tar   :
	tar -cvf code.tar Configure.nocpp *OBJ* */mm_diff */QMMMHELP.html *F *inc *c *h */*F */*h  
	echo 'code.tar created'  
END
fi

echo "done."  >&2
echo "O.K." >&2
exit 0


