C     ==================================================================
      SUBROUTINE ADJMU(NSTATE,NKPOINT,NEL,BETAEL,WE,WK,TSPIN,AMU,TINFO)
C     ==--------------------------------------------------------------==
C     == ADJUST THE CHEMICAL POTENTIAL (AMU)                          ==
C     == WITH THE BISECTION METHOD                                    ==
C     == WE DOESN''T NEED BE SORTED                                   ==
C     ==--------------------------------------------------------------==
C     == INPUT:                                                       ==
C     ==    NSTATE number of states                                   ==
C     ==    NKPOINT number of k points                                ==
C     ==    NEL number of electrons                                   ==
C     ==    BETAEL 1/kT                                               ==
C     ==    WE(NSTATE,NKPOINT) eigenvalues (energies for each state)  ==
C     ==    WK(NKPOINT) weight of k points                            ==
C     ==    TSPIN .TRUE. if  local spin polarization                  ==
C     == OUTPUT:                                                      ==
C     ==    AMU chemical potential                                    ==
C     == INPUT/OUTPUT:                                                ==
C     ==    TINFO if .TRUE. return .TRUE. if ADJMU finds AMU          ==
C     ==          if .FALSE. stops if ADJMU fails                     ==
C     ==--------------------------------------------------------------==
C     == PRECISION GIVEN DELTA = 1.D-6 EV  (0.1Kelvin)                ==
C     ==--------------------------------------------------------------==
      IMPLICIT NONE
      INCLUDE 'system.h'        !PARENT
C     Arguments
      INTEGER   NSTATE,NKPOINT
      REAL*8    BETAEL,WE(NSTATE,NKPOINT),WK(NKPOINT),AMU,NEL
      LOGICAL   TSPIN,TINFO
C     Variables
      INTEGER   NIT_MAX
      REAL*8    PRECPAR,RY,DELTA
      PARAMETER (NIT_MAX=200,
     &           PRECPAR=1.D-10,RY=13.605804D0,DELTA=1.D-6/(2.D0*RY))
      INTEGER   IK,I,NIT
      LOGICAL   NOT_CONVERGED
      REAL*8    AMU1,AMU2,DAMU,RHINT,RHINT1,RHINT2,RHOINT,PRECISION
C     ==--------------------------------------------------------------==
C     == STARTING CHEMICAL POTENTIAL AS THE AVERAGE BETWEEN           ==
C     == LAST OCCUPIED STATE AND THE FIRST EMPTY STATE                ==
C     ==--------------------------------------------------------------==
C     AMU1(lowest)...AMU...AMU2(biggest)...
      PRECISION=PRECPAR*DLOG(NEL+1.D0)
      AMU1=WE(1,1)
      AMU2=WE(1,1)
      DO IK=1,NKPOINT
        DO I=1,NSTATE
          IF(AMU1.GT.WE(I,IK)) THEN
             AMU1=WE(I,IK)
          ENDIF
          IF(AMU2.LT.WE(I,IK)) THEN
             AMU2=WE(I,IK)
          ENDIF
        ENDDO
      ENDDO
      RHINT1=1.D0
      DO WHILE(RHINT1.GT.0.D0)
        RHINT1=RHOINT(WE,WK,AMU1,BETAEL,NSTATE,NKPOINT,NEL,TSPIN)
        IF (RHINT1.GT.0.D0) AMU1=-2.D0*DABS(AMU1)
      ENDDO
      RHINT2=RHOINT(WE,WK,AMU2,BETAEL,NSTATE,NKPOINT,NEL,TSPIN)
      IF(RHINT2.LT.0.D0) THEN
        DAMU=AMU2-AMU1
        RHINT=RHINT1
        NOT_CONVERGED=.FALSE.
        GOTO 1000
      ENDIF
      DAMU=AMU2-AMU1
      AMU=AMU1
C     ==--------------------------------------------------------------==
      DO NIT=1,NIT_MAX
        AMU=AMU1+0.5D0*DAMU
        RHINT=RHOINT(WE,WK,AMU,BETAEL,NSTATE,NKPOINT,NEL,TSPIN)
        IF(DAMU.LT.DELTA) THEN
          IF(DABS(RHINT).LT.PRECISION) THEN
            RETURN
          ELSEIF(DABS(RHINT1).LT.PRECISION) THEN
            AMU=AMU1
            RETURN
          ELSEIF(DABS(RHINT2).LT.PRECISION) THEN
            AMU=AMU2
            RETURN
          ENDIF
        ENDIF
C       LT in order to have the lowest possible AMU (if gap).
        IF(RHINT.LT.0.D0) THEN
          AMU1=AMU
          RHINT1=RHINT
        ELSE
          AMU2=AMU
          RHINT2=RHINT
        ENDIF
        DAMU=AMU2-AMU1
      ENDDO
      NOT_CONVERGED=.TRUE.
 1000 CONTINUE
      IF(TINFO) THEN
        TINFO=.FALSE.
        RETURN
      ELSE
        IF(PARENT) THEN
          WRITE(6,*) ' ADJMU|    IK     I    EIGENVALUES:'
          DO IK=1,NKPOINT
            DO I=1,NSTATE
              WRITE(6,'(7x,2i6,2x,f14.8)') IK,I,WE(I,IK)
            ENDDO
          ENDDO
          WRITE(6,*) '        AMU1=',AMU1,'       RHINT1=',RHINT1
          WRITE(6,*) '        AMU2=',AMU2,'       RHINT2=',RHINT2
          WRITE(6,*) '        DAMU=',DAMU,'       RHINT =',RHINT
          WRITE(6,*) '        RHINT-NEL=',RHINT
          IF(NOT_CONVERGED) THEN
            CALL STOPGM('ADJMU','BISECTION DID NOT CONVERGE')
          ELSE
            WRITE(6,*) ' ADJMU! THE NUMBER OF STATES [',
     &           NSTATE,'] IS TOO SMALL'
            CALL STOPGM('ADJMU','BISECTION COULD NOT CONVERGE')
          ENDIF
        ENDIF
      ENDIF
C     ==--------------------------------------------------------------==
      RETURN
      END
C     ==================================================================
      FUNCTION RHOINT(WE,WK,AMU,BETAEL,NSTATE,NKPOINT,NEL,TLSD)
C     ==--------------------------------------------------------------==
C     == CALCULATES THE NUMBER OF ELECTRONS FOR THE GIVEN POTENTIAL   ==
C     == WORKS WITH T=0K (BETAEL almost infinite) and with the last   ==
C     == state degenerated                                            ==
C     ==--------------------------------------------------------------==
C     == WE(NSTATE,NKPOINT) eigenvalues                               ==
C     == WK(NKPOINT) weight of k points                               ==
C     == AMU the given chemical potential                             ==
C     == BETAEL 1/kT                                                  ==
C     == NSTATE number of states                                      ==
C     == NKPOINT number of kpoints                                    ==
C     == NEL number of electrons                                      ==
C     == TLSD if false 2 electrons per state                          ==
C     ==--------------------------------------------------------------==
C     == PRECISION GIVEN DELTA = 1.D-6 EV  (0.1Kelvin)                ==
C     ==--------------------------------------------------------------==
      IMPLICIT NONE
C     Arguments
      INTEGER   NSTATE,NKPOINT
      REAL*8    WE(NSTATE,NKPOINT),WK(NKPOINT),AMU,BETAEL,NEL
      LOGICAL   TLSD
      REAL*8    RHOINT
C     Variables
      REAL*8    TOLLPAR,RY,DELTA
      PARAMETER (TOLLPAR=1.D-13,RY=13.606D0,DELTA=1.D-6/(2.D0*RY))
      REAL*8    ARGMIN,ARGMAX,ARG,XLM,REMAIN,WBIGWE,
     &          DRWE,DROUND,DRAMU
      INTEGER   IK,I
      LOGICAL   TAMU
C     ==--------------------------------------------------------------==
      ARGMIN=DLOG(TOLLPAR)
      ARGMAX=-ARGMIN
      DRAMU=DROUND(AMU,DELTA)
C     If last occupied states are degenerated, there should be some pb.
      DO IK=1,NKPOINT
        DO I=1,NSTATE
          DRWE=DROUND(WE(I,IK),DELTA)
          IF(DABS(DRWE-DRAMU).LT.DELTA) THEN
             TAMU=.TRUE.
          ENDIF
        ENDDO
      ENDDO
C     Find the degeneracy of last occupied state.
      WBIGWE=0.D0
      RHOINT=0.D0
      DO IK=1,NKPOINT
        DO I=1,NSTATE
          ARG=-BETAEL*(WE(I,IK)-AMU)
          IF(ARG.GT.ARGMAX) THEN
            XLM=1.D0
          ELSE IF(ARG.LT.ARGMIN) THEN
            XLM=0.D0
          ELSE
            XLM=1.D0/(DEXP(-ARG)+1.D0)
          ENDIF
          DRWE=DROUND(WE(I,IK),DELTA)
          IF(TAMU.AND.DABS(DRWE-DRAMU).LT.DELTA) THEN
            WBIGWE=WBIGWE+WK(IK)
          ELSEIF(XLM.GT.0.D0) THEN
            RHOINT=RHOINT+WK(IK)*XLM
          ENDIF
        ENDDO
      ENDDO
C     Solve some small problem.
      IF(.NOT.TLSD) THEN
        RHOINT=2.D0*RHOINT
      ENDIF
      REMAIN=NEL-RHOINT
      IF(REMAIN.LE.0.D0) THEN
        IF(TLSD) THEN
          REMAIN=1.D0
        ELSE
          REMAIN=2.D0
        ENDIF
      ENDIF
      IF(WBIGWE.GT.0.D0) THEN
        REMAIN=REMAIN/WBIGWE
        IF(TLSD) THEN
          REMAIN=MIN(REMAIN,1.D0)
        ELSE
          REMAIN=MIN(REMAIN,2.D0)
        ENDIF
        DO IK=1,NKPOINT
          DO I=1,NSTATE
            DRWE=DROUND(WE(I,IK),DELTA)
            IF(TAMU.AND.DABS(DRWE-DRAMU).LT.DELTA) THEN
              RHOINT=RHOINT+WK(IK)*REMAIN
            ENDIF
          ENDDO
        ENDDO
      ENDIF
C     ==--------------------------------------------------------------==
      RHOINT=RHOINT-NEL
C     ==--------------------------------------------------------------==
      RETURN
      END
C     ==================================================================
      SUBROUTINE OCCUP(WE,WK,AMU,BETAEL,FOCC,NSTATE,NKPOINT,NEL,TLSD)
C     ==--------------------------------------------------------------==
C     ==                        COMPUTES                              ==
C     ==                                                              ==
C     ==  THE OCCUPATION NUMBERS: F = 2/(1+LAMBDA^P*EXP(BETA MU))     ==
C     ==  WORKS WITH T=0K AND THE LAST STATE DEGENERATED              ==
C     ==--------------------------------------------------------------==
C     == PRECISION GIVEN DELTA = 1.D-6 EV  (0.1Kelvin)                ==
C     ==--------------------------------------------------------------==
      IMPLICIT NONE
C     Arguments
      INTEGER   NSTATE,NKPOINT
      REAL*8    WE(NSTATE,NKPOINT),WK(NKPOINT),AMU,BETAEL,NEL,
     &          FOCC(NSTATE,NKPOINT)
      LOGICAL   TLSD
C     Variables
      REAL*8    TOLLPAR,RY,DELTA
      PARAMETER (TOLLPAR=1.D-13,RY=13.606D0,DELTA=1.D-6/(2.D0*RY))
      REAL*8    ARGMIN,ARGMAX,ARG,XLM,RHOINT,WBIGWE,
     &          REMAIN,DRWE,DRAMU,
     &          DROUND
      INTEGER   IK,I
      LOGICAL   TAMU
C     ==--------------------------------------------------------------==
      ARGMIN=DLOG(TOLLPAR)
      ARGMAX=-ARGMIN
      DRAMU=DROUND(AMU,DELTA)
C     If last occupied states are degenerated, there should be some pb.
      DO IK=1,NKPOINT
        DO I=1,NSTATE
          DRWE=DROUND(WE(I,IK),DELTA)
          IF(DABS(DRWE-DRAMU).LT.DELTA) THEN
             TAMU=.TRUE.
          ENDIF
        ENDDO
      ENDDO
C     Find the degeneracy of last occupied state.
      WBIGWE=0.D0
      RHOINT=0.D0
      IF(TLSD) THEN ! LSD
      DO IK=1,NKPOINT
        DO I=1,NSTATE
          ARG=-BETAEL*(WE(I,IK)-AMU)
          IF(ARG.GT.ARGMAX) THEN
            XLM=1.D0
          ELSE IF(ARG.LT.ARGMIN) THEN
            XLM=0.D0
          ELSE
            XLM=1.D0/(DEXP(-ARG)+1.D0)
          ENDIF
          DRWE=DROUND(WE(I,IK),DELTA)
          IF(TAMU.AND.DABS(DRWE-DRAMU).LT.DELTA) THEN
            WBIGWE=WBIGWE+WK(IK)
          ELSEIF(XLM.GT.0.D0) THEN
            RHOINT=RHOINT+WK(IK)*XLM
          ENDIF
          FOCC(I,IK)=XLM
        ENDDO
      ENDDO
      ELSE ! LDA
      DO IK=1,NKPOINT
        DO I=1,NSTATE
          ARG=-BETAEL*(WE(I,IK)-AMU)
          IF(ARG.GT.ARGMAX) THEN
            XLM=1.D0
          ELSE IF(ARG.LT.ARGMIN) THEN
            XLM=0.D0
          ELSE
            XLM=1.D0/(DEXP(-ARG)+1.D0)
          ENDIF
          DRWE=DROUND(WE(I,IK),DELTA)
          IF(TAMU.AND.DABS(DRWE-DRAMU).LT.DELTA) THEN
            WBIGWE=WBIGWE+WK(IK)
          ELSEIF(XLM.GT.0.D0) THEN
            RHOINT=RHOINT+WK(IK)*XLM
          ENDIF
          FOCC(I,IK)=2.D0*XLM
        ENDDO
      ENDDO
      ENDIF ! LSD/LDA select 

C     Solve some small problem.
      IF(.NOT.TLSD) THEN
        RHOINT=2.D0*RHOINT
      ENDIF
      REMAIN=NEL-RHOINT
      IF(REMAIN.LE.0.D0) THEN
        IF(TLSD) THEN
          REMAIN=1.D0
        ELSE
          REMAIN=2.D0
        ENDIF
      ENDIF
      IF(WBIGWE.GT.0.D0) THEN
        REMAIN=REMAIN/WBIGWE
        IF(TLSD) THEN
          REMAIN=MIN(REMAIN,1.D0)
        ELSE
          REMAIN=MIN(REMAIN,2.D0)
        ENDIF
        DO IK=1,NKPOINT
          DO I=1,NSTATE
            DRWE=DROUND(WE(I,IK),DELTA)
            IF( TAMU.AND.DABS(DRWE-DRAMU).LT.DELTA ) THEN
              FOCC(I,IK)=REMAIN
            ENDIF
          ENDDO
        ENDDO
      ENDIF
C     ==--------------------------------------------------------------==
      RETURN
      END
C     ==================================================================
      SUBROUTINE ENERT(WE,WK,F,AMU,BETAEL,NS,NSTATE,NKPOINT,NEL,TLSD,
     &                 FREE_ENERGY,BAND_ENERGY,MKT_ENTROPY)
C     ==--------------------------------------------------------------==
C     == COMPUTES:                                                    ==
C     == - FREE_ENERGY - THE ELECTRONIC FREE ENERGY                   ==
C     == (see A. Alavi, J. Kohanoff, M. Parrinello, D. Frenkel,       ==
C     ==  PRL 73 2599 (1994)),                                        ==
C     == - BAND_ENERGY - THE BAND ENERGY (sum of eigenvalues x occ. n)==
C     == - MKT_ENTROPY = - T*S                                        ==
C     ==--------------------------------------------------------------==
      IMPLICIT NONE
C     Arguments
      INTEGER   NS,NSTATE,NKPOINT
      REAL*8    WE(NSTATE,NKPOINT),WK(NKPOINT),F(NSTATE,NKPOINT),
     &          NEL,AMU,BETAEL
      LOGICAL   TLSD
      REAL*8    FREE_ENERGY,BAND_ENERGY,MKT_ENTROPY
C     Variables
      REAL*8    TOLLPAR
      PARAMETER (TOLLPAR=1.D-13)
      REAL*8    BETAELMAX
      PARAMETER (BETAELMAX=1.D33)
      INTEGER   IK,I
      REAL*8    ARGMIN,ARGMAX,GPBAND,ARG,GIBBS,SPIND,TEMP_EN,
     &          OCCUP,SPINDINV
C     ==--------------------------------------------------------------==
      ARGMIN=DLOG(TOLLPAR)
      ARGMAX=-ARGMIN
      GPBAND = 0.D0
      BAND_ENERGY = 0.D0
      MKT_ENTROPY = 0.D0
      IF(TLSD) THEN
        SPIND = 1.D0
      ELSE
        SPIND = 2.D0
      ENDIF
      SPINDINV=1.D0/SPIND
      DO IK=1,NKPOINT
        TEMP_EN = 0.D0
        DO I=1,NS
          ARG=-BETAEL*(WE(I,IK)-AMU)
          IF(ARG.GT.ARGMAX) THEN
            GPBAND=GPBAND+WK(IK)*SPIND*(WE(I,IK)-AMU)
          ELSEIF(ARG.LT.ARGMIN) THEN
C           Do nothing (avoid underflow).
          ELSE
            GPBAND=GPBAND-WK(IK)*SPIND*DLOG(DEXP(ARG)+1.D0)/BETAEL
          ENDIF
          BAND_ENERGY = BAND_ENERGY + WK(IK)*F(I,IK)*WE(I,IK)
          OCCUP=F(I,IK)*SPINDINV
          IF(OCCUP.GT.0.D0) THEN
            TEMP_EN = TEMP_EN + OCCUP*DLOG(OCCUP)
          ENDIF
          IF(OCCUP.LT.1.D0) THEN
            TEMP_EN = TEMP_EN + (1.D0-OCCUP)*DLOG(1.D0-OCCUP)
          ENDIF
        ENDDO
        MKT_ENTROPY = MKT_ENTROPY + WK(IK)*TEMP_EN
      ENDDO
      GIBBS=AMU*NEL 
      FREE_ENERGY = GPBAND + GIBBS
      IF(BETAEL.GE.BETAELMAX) THEN
        MKT_ENTROPY = 0.D0
      ELSE
        MKT_ENTROPY = 1.D0/BETAEL * SPIND * MKT_ENTROPY
      ENDIF
C     ==--------------------------------------------------------------==
      RETURN
      END
C     ==================================================================
      SUBROUTINE CONVFRIE(NSTATE,NCONV,NEL,WE,
     &                    TISLSD,AMU,FOCC,WEINT,TCONV)
C     ==--------------------------------------------------------------==
C     == TCONV=.TRUE. IF THE NUMBER OF STATES NSTATE IS SUFFICIENT    ==
C     == I.E. THE LAST STATE COULD BE OCCUPANCY NUMBER < TOLLPAR      ==
C     ==--------------------------------------------------------------==
      IMPLICIT NONE
      INCLUDE 'system.h'
      INCLUDE 'nlps.inc'
      INCLUDE 'fint.inc'
C     Arguments
      INTEGER   NSTATE,NCONV
      REAL*8    WE(NSTATE),AMU,FOCC(NSTATE),WEINT(NSTATE),NEL
      LOGICAL   TISLSD,TCONV
C     Variables
      REAL*8    TOLLPAR,WK(1)
      PARAMETER (TOLLPAR=1.D-6)
      INTEGER   I,NTOTAL
C     ==--------------------------------------------------------------==
      IF(TTROT) THEN
!$OMP parallel do private(I)
        DO I=1,NSTATE
          WEINT(I)=-DLOG(WE(I))/BETAP
        ENDDO
      ELSE
        CALL DCOPY(NSTATE,WE(1),1,WEINT(1),1)
        CALL DSCAL(NSTATE,-1.D0,WEINT(1),1)
      ENDIF
      WK(1)=1.D0
      CALL ADJMU(NSTATE,1,NEL,BETAEL,WEINT,WK,TISLSD,AMU,TCONV)
      IF(.NOT.TCONV) THEN
         RETURN
      ENDIF
C     Check if the last state has focc<toll.
      CALL OCCUP(WEINT,WK,AMU,BETAEL,FOCC,NSTATE,1,NEL,TISLSD)
      NTOTAL=0
      DO I=1,NSTATE
        IF(FOCC(I).GT.TOLLPAR) THEN
           NTOTAL=NTOTAL+1
        ENDIF
      ENDDO
      TCONV=(NTOTAL.LT.NCONV)
C     ==--------------------------------------------------------------==
      RETURN
      END
C     ==================================================================
      FUNCTION DROUND(A,DELTA)
C     ==--------------------------------------------------------------==
      IMPLICIT NONE
      REAL*8 A,DELTA,DROUND
C     ==--------------------------------------------------------------==
      DROUND=A-MOD(A,DELTA)
C     ==--------------------------------------------------------------==
      RETURN
      END
C     ==================================================================
