C Author: Patrik Mauer
C Revision: Ivano Tavernelli (Dec. 2010)
C     ==================================================================
      subroutine fm_fit_covalent(nframes, nr_qm, r_qm, fqm_ref)
C     ==--------------------------------------------------------------==
C     The main driver routine for fitting covalent parameters (force
C     constants, bond lengths, angles, etc.)
C     NB: We use atomic units!
C     ==================================================================
        implicit none
        include 'forcematch.inc'
C       Arguments
C       nr of frames, nr of QM atoms
        integer nframes, nr_qm
C       coordinates of QM atoms
        real*8  r_qm(3, nr_qm, nframes)
C       reference forces on QM atoms
        real*8  fqm_ref(3, nr_qm, nframes)
C       Variables
C       total nr. of parameters
        integer npar
C       parameters
        real*8, allocatable  :: par(:)

        write(6,*)
        write(6,'(A)') '  fm  ========================================'
        write(6,'(A)') '  fm  Now fitting covalent parameters'
        write(6,'(A)') '  fm  ========================================'

C       in case I want to exclude a subset of parameters from
C       the fitting I adjust the number of parameters here:

        if(.not. fm_fit_bonds) then
          fm_nbonopt=0
          write(6,*) ' fm  readjusting number of fitting parms'
          write(6,*) ' fm  nr of bonds to optimize: ',fm_nbonopt
        end if
        if(.not. fm_fit_angles) then
          fm_nangopt=0
          write(6,*) ' fm  readjusting number of fitting parms'
          write(6,*) ' fm  nr of angles to optimize: ',fm_nangopt
        end if
        if(.not. fm_fit_diheds) then
          fm_ndihopt=0
          write(6,*) ' fm  readjusting number of fitting parms'
          write(6,*) ' fm  nr of dihedrals to optimize: ',fm_ndihopt
        end if
        if(.not. fm_fit_improps) then
          fm_nimpopt=0
          write(6,*) ' fm  readjusting number of fitting parms'
          write(6,*) ' fm  nr of improper dihedrals to optimize: ',
     &         fm_nimpopt
        end if

C compute total nr. of parameters. For impropers and dihedrals, we
C always fit force constants only. For bonds and angles, we can also fit
C equilibrium values
C ordering of parameters is:
C - with only force constants:
C   kb(1..N), kt(1..N), kq(1..N), kp(1..N)
C - with equilibrium values:
C   kb(1),b0(1),...,kb(NB),b0(NB), kt(1),t0(1),...,kt(N), t0(N),
C   kq(1..N), kp(1..N)
        if (fm_fit_fc_only) then
          npar = fm_nbonopt + fm_nangopt + fm_nimpopt + fm_ndihopt
        else
          npar = 2*fm_nbonopt + 2*fm_nangopt + fm_nimpopt + fm_ndihopt
        endif
        write(6,'(A, i6)') '  fm  Total nr. of parameters: ', npar

        allocate(par(npar))
C We need to set some common variables to circumvent the optimizer
        fm_nframes = nframes
        fm_nrqm = nr_qm

        call fm_kfit_collect_optimized
        call fm_kfit_setup_mytypes

C_DEBUG[
        call fm_writeout_topo('beforefit.top')
C_DEBUG]
        if (fm_fit_fc_only) then
           write(6,*) ' fm  fitting force constants only'

C    TESTING ONLY FIT ANGLES
c           fm_nbonopt =0
c           fm_nangopt = 0
c           fm_nimpopt=0
C           fm_ndihopt=0
c           npar = fm_ndihopt
c           write(6,*) 'total nr. of parameters: ',npar
          call fm_optimize_fc (npar,par,nframes,nr_qm,r_qm,fqm_ref)
        else
          call fm_initialguess(npar, par)
          call fm_optimize_allcovalent(npar, par, nframes, nr_qm, r_qm, 
     $                                 fqm_ref)
        endif
        call fm_kfit_par_to_fmarray(npar, par)
        call fm_kfit_writeback(npar, par)

        deallocate(par)
        return
      end subroutine fm_fit_covalent
C
C     ==================================================================
      subroutine fm_optimize_fc (npar,par,nframes,nr_qm,r_qm,fqm_ref)
C     ==--------------------------------------------------------------==
C     Driver for the optimization of the force constants only in the 
C     covalent interactions (including bond/angles/torsion and improper 
C     torsions potentials). This is a linear set of equations and we use 
C     the routine dgels from LAPACK to solve it.
C     ==================================================================
      implicit none
      include 'forcematch.inc'
      include 'fm_cnst.inc'
C     nr of parameters, nr. of frames, nr. of QM atoms      
      integer npar,nframes,nr_qm
      integer i,j,k,l,iframe,lb,ub
C     variables for the loops      
      integer ipar0,ipar,ibon,bt,iang,at,iimp,it,idih,dt
c     dummy integer
      integer indx
C     parameters, the array that's going to be updated
      real*8  par(npar)
C       coordinates of QM atoms
      real*8 r_qm(3, nr_qm, nframes)
      real*8 ri(3),rj(3),rk(3),rl(3)
C     reference forces
      real*8 fqm_ref(3, nr_qm, nframes)
      real*8 fqm_ref_fc(3*nr_qm*nframes)
C     derivatives coming back from fm_forces_*** routines
      real*8 fi(3),fj(3),fk(3),fl(3)
C     the derivative matrix
c     number of rows: 3*noatoms*noframes
C     number of columns:  noof parameters
      real*8,target ::  fm_fc_derivs(3*nr_qm*nframes,npar)
      real*8, pointer :: derivs(:, :)
C debugging
      real*8 bla(3*nr_qm*nframes)
      real*8 blu(3*nr_qm*nframes,npar)
C     lapack related stuff:
      integer  no_rows,no_coloumns,nrhs
C     pass some information codes around
      integer info
C     work array, and size of work array
      integer lwoa
      real*8     wa(3*npar)
      lwoa=3*npar
C     nullify the derivs matrix
      do i=1,3*nr_qm*nframes
         do j=1,npar
          fm_fc_derivs(i,j)=0.0d0
         end do
      end do
C     I need a copy of the reference forces array
C     because it is going to be overwritten by the lapack routine
      indx=0
      do i=1,nframes
         do j=1,nr_qm
            do k=1,3
              indx=indx+1
              fqm_ref_fc(indx)=fqm_ref(k,j,i)
C              write(6,*) 'fqm_ref_fc ',fqm_ref_fc(indx)
            end do
         end do
      end do   
C    TESTING ONLY FIT BONDS
c       fill the derivative matrix
C Loop over all frames. In each frame, we use derivs to point to
C the appropriate "windows" of the  full matrix fm_fc_derivs,
C to facilitate the indexing business
C        write(6,*) fm_nframes,fm_nrqm,lb,ub,fm_nbon
        do iframe = 1, fm_nframes
          lb = (iframe-1) * 3*fm_nrqm + 1
          ub = lb + 3*fm_nrqm - 1
          derivs => fm_fc_derivs(lb:ub, :)
C          write(6,*) 'fm_fc_derivs() ',lb,' : ',ub
C         derivs contains now one frame for the matrix fm_fc_derivs          
C Bonds   (ipar0=0: bonds come first)
          ipar0 = 0
          do ibon = 1, fm_nbon
C             write(6,*) fm_ib(:,ibon)
C            write(6,*) 'grumpf',fm_ib(0, ibon)
            bt = fm_ib(0, ibon)
             write(6,*) 
     &       'ibon',ibon,' bt ',bt,' kb ',fm_kb(bt),
     &       ' gromos: ',kb_au2grm*fm_kb(bt)
C            write(6,*) 'grumpf',fm_ib(1, ibon)
            i = fm_ib(1, ibon)
c            write(6,*) i
C             write(6,*) 'grumpf',fm_ib(2, ibon)
            j = fm_ib(2, ibon)
c            write(6,*) j
C             write(6,*) 'kampf',r_qm(:, i, iframe)
            ri = r_qm(:, i, iframe)
C            write(6,*) 'ri ',ri
            rj = r_qm(:, j, iframe)
           write(6,*) 'i ',i,' j ',j,' ri ',ri,' rj ',rj

C           compute i,j as index into array derivs
            i = 3*(i-1) + 1
            j = 3*(j-1) + 1
            call fm_forces_bon(fi, fj, ri, rj, bt, 'dk')
c           the column index             
            ipar = ipar0 + bt
            derivs(i:i+2, ipar) = derivs(i:i+2, ipar) + fi
            derivs(j:j+2, ipar) = derivs(j:j+2, ipar) + fj
C            write(6,*) 'derivs i ',i,' : ',i+2
C            write(6,*) derivs(i:i+2, ipar)
C            write(6,*) 'derivs j ',j,' : ',j+2
C            write(6,*) derivs(j:j+2, ipar)
C            write(6,*) 'grimpf'
          enddo ! loop over all the bonds in the QM system
C Angles
          ipar0 = fm_nbonopt
          do iang = 1, fm_nang
            at = fm_it(0, iang)
            i = fm_it(1, iang)
            j = fm_it(2, iang)
            k = fm_it(3, iang)
            ri = r_qm(:, i, iframe)
            rj = r_qm(:, j, iframe)
            rk = r_qm(:, k, iframe)

            i = 3*(i-1) + 1
            j = 3*(j-1) + 1
            k = 3*(k-1) + 1
              
            ipar = ipar0 + at 
              call fm_forces_ang(fi, fj, fk, ri, rj, rk, at, 'dk')
              derivs(i:i+2, ipar) = derivs(i:i+2, ipar) + fi
              derivs(j:j+2, ipar) = derivs(j:j+2, ipar) + fj
              derivs(k:k+2, ipar) = derivs(k:k+2, ipar) + fk
          enddo ! loop over angle types

C Impropers
          ipar0 = ipar0 + fm_nangopt
          do iimp = 1, fm_nimp
            it = fm_iq(0, iimp)
            i = fm_iq(1, iimp)
            j = fm_iq(2, iimp)
            k = fm_iq(3, iimp)
            l = fm_iq(4, iimp)
            ri = r_qm(:, i, iframe)
            rj = r_qm(:, j, iframe)
            rk = r_qm(:, k, iframe)
            rl = r_qm(:, l, iframe)

            i = 3*(i-1) + 1
            j = 3*(j-1) + 1
            k = 3*(k-1) + 1
            l = 3*(l-1) + 1
              ipar = ipar0 + it
              call fm_forces_imp(fi,fj,fk,fl,ri,rj,rk,rl,it,'dk')
              derivs(i:i+2, ipar) = derivs(i:i+2, ipar) + fi
              derivs(j:j+2, ipar) = derivs(j:j+2, ipar) + fj
              derivs(k:k+2, ipar) = derivs(k:k+2, ipar) + fk
              derivs(l:l+2, ipar) = derivs(l:l+2, ipar) + fl
          enddo ! loop over all the improper dihedrals
C Dihedrals
          ipar0 = ipar0 + fm_nimpopt
          do idih = 1, fm_ndih
            dt = fm_ip(0, idih)
            i = fm_ip(1, idih)
            j = fm_ip(2, idih)
            k = fm_ip(3, idih)
            l = fm_ip(4, idih)
            ri = r_qm(:, i, iframe)
            rj = r_qm(:, j, iframe)
            rk = r_qm(:, k, iframe)
            rl = r_qm(:, l, iframe)

            i = 3*(i-1) + 1
            j = 3*(j-1) + 1
            k = 3*(k-1) + 1
            l = 3*(l-1) + 1
              ipar = ipar0 + dt
              call fm_forces_dih(fi,fj,fk,fl,ri,rj,rk,rl,dt,'dk')
              derivs(i:i+2, ipar) = derivs(i:i+2, ipar) + fi
              derivs(j:j+2, ipar) = derivs(j:j+2, ipar) + fj
              derivs(k:k+2, ipar) = derivs(k:k+2, ipar) + fk
              derivs(l:l+2, ipar) = derivs(l:l+2, ipar) + fl
          enddo !loop over all the dihedrals



        end do ! loop over no frames 
C       solve the matrix equation: fm_fc_derivs * X = fqm_ref_fc          
        
      do i=1,3*nr_qm*nframes
         do j=1,npar
          blu(i,j)=fm_fc_derivs(i,j)
         end do
         write(6,*) fm_fc_derivs(i,:)
      end do
c      write(6,*) 'parameters beginning'
c       do i=1,fm_nbonopt
c          par(i)=fm_kb(i)
c          write(6,*) par(i),par(i)*kb_au2grm
c       end do   
       
c        bla=matmul(blu,par)
c        write(6,*) 'first test'
c        indx=0
c        do i=1,nframes
c         do j=1,nr_qm
c            do k=1,3
c              indx=indx+1
c              write(6,*) bla(indx),fqm_ref(k,j,i)
c            end do
c         end do
c        end do   
c        write(6,*) 'end first test'
        bla(:)=0.0d0
        par(:)=0.0d0

        no_rows=3*nr_qm*nframes
        no_coloumns=npar
        nrhs=1
        call dgels('N',no_rows,no_coloumns,nrhs,fm_fc_derivs,
     &       no_rows,fqm_ref_fc,no_rows,wa,lwoa,info)     
C        call dgels('N',3*nr_qm*nframes,npar,1,fm_fc_derivs,
C     &       3*nr_qm*nframes,fqm_ref_fc,3*nr_qm*nframes,wa,lwoa,info)     
C      note that both matrices fm_fc_derivs and fqm_ref_fc have been 
C      overwritten
C      the npar first elements of fqm_ref_fc is the solution 
C        write(6,*) ' fm  after dgels, info=',info
        if (info .ne. 0) then
          write(6,'(A)') '  fm  Optimization FAILED:'
          call STOPGM('fm_optimize_fc', 'FAIL (see output)')
        else
          write(6,'(A)') '  fm  Optimization lyn sys successfull'
        end if  
C       write(6,*) ' fm  list of parameters after fitting: au, grm'
C        do i=1,npar
C          write(6,*) fqm_ref_fc(i),kb_au2grm* fqm_ref_fc(i)
C        end do 

C       write back to the par array
        do i=1,npar
            par(i)=fqm_ref_fc(i)
        end do    
C       I should get back the reference forces!
        write(6,*) ' '
        write(6,*) ' fm  test: forces after fit, ref forces'
        bla=matmul(blu,par)
        indx=0
        do i=1,nframes
         do j=1,nr_qm
            do k=1,3
              indx=indx+1
              write(6,*) bla(indx),fqm_ref(k,j,i)
            end do
         end do
        end do   
c        do i=1,3*nr_qm*nframes
c          write(6,*) bla(i),fqm_ref_fc(i)
c        end do

C        call STOPGM('fm_optimize_fc', 'TEST')

      return
      end subroutine fm_optimize_fc
C
C     ==================================================================
      subroutine fm_optimize_allcovalent(npar, par, nframes, nr_qm, 
     $                                   r_qm, fqm_ref)
C     ==--------------------------------------------------------------==
C     Driver for the optimization of ALL covalent interactions parameters
C     (including bond lengths/angles). This is a non-linear problem and 
C     we use the routine lmder1 from MINPACK to solve it.
C     ==================================================================
        implicit none
        include 'forcematch.inc'
C       Arguments
C       nr. of parameters, nr. of frames, nr. of QM atoms
        integer npar, nframes, nr_qm
C       parameters
        real*8  par(npar)
C       coordinates of QM atoms
        real*8, target  :: r_qm(3, nr_qm, nframes)
C       reference forces
        real*8, target  :: fqm_ref(3, nr_qm, nframes)
C       Variables
C       size of the target vector (=3*nr_qm*nframes)
        integer trget_size
C       vector with force differences 
        real*8  fdiff(3, nr_qm, nframes)
C       jacobian matrix (derivative of fdiff wrt. parameters)
        real*8  fjac(3*nr_qm*nframes, npar)
C       was the optimization successful?
        logical success
C       strings for error or success messages
        character*255   err_msg, succ_msg
C       minpack related stuff:
C       tolerance (convergence criterion)
        real*8  tol
C       pass some information codes around
        integer info
C       pivot array, work array, and size of work array
        integer ipvt(npar)
        integer lwa
C        real*8, allocatable    :: work_array(:)
        real*8 work_array(5*npar*3*nr_qm*nframes)

C       name of the function that evaluates fdiff and fjac
        external fm_fcn_allcovalent

C hardcode tol for now
C FIXME: make tol user-definable
        tol = 1.d-06

        fm_rqm => r_qm
        fm_fqm_ref => fqm_ref

        trget_size = 3*nr_qm*nframes

        lwa = 5 * npar * trget_size
C      allocate(wa(lwa))
C_DEBUG[
C        write(6,*) ' lwa is: ', lwa
C        write(6,*) ' allocated wa?', allocated(work_array)
C        write(6,*) ' allocating wa'
C_DEBUG]
C        allocate(work_array(lwa))
C_DEBUG[
C        write(6,*) ' allocated wa?', allocated(work_array)
C        write(6,*) ' size wa?', size(work_array)
C_DEBUG]
        success = .false.
#ifdef __MINPACK
        call lmder1(fm_fcn_allcovalent, trget_size, npar, par, fdiff,
     $              fjac, trget_size, tol, info, ipvt, work_array, lwa)
#else
        call STOPGM('fm_optimize_allcovalent', 'No MINPACK linked!')
#endif

C_DEBUG[
        write(6,*) " lmder1 returned info = ", info
C_DEBUG]
C check if we were successful
        if (info.lt.0) then
          err_msg = 'exceeded max. nr of iterations'
          success = .false.
        elseif (info.eq.0) then
          success = .false.
          err_msg = 'improper input parameters to lmder1'
        elseif (info.eq.1) then
          success = .true.
          succ_msg = 'algorithm estimates that the relative error in '
     $             //'the sum of squares is at most tol'
        elseif (info.eq.2) then
          success = .true.
          succ_msg = 'algorithm estimates that the relative error '
     $             //'between x and the solution is at most tol'
        elseif (info.eq.3) then
          success = .true.
          succ_msg = 'super converged!'
        elseif (info.eq.4) then
          success = .true. ! I guess???
          succ_msg = 'fvec is orthogonal to the columns of the jacobian'
     $             //' to machine precision'
        elseif (info.eq.5) then
          success = .false.
          err_msg = 'too many iterations needed'
        elseif (info.eq.6) then
          success = .true. ! sure?
          succ_msg = 'tol is too small. no further reduction in the '
     $             //'sum of squares is possible'
        elseif (info.eq.7) then
          success = .true. ! sure?
          succ_msg = 'tol is too small. no further improvement in the '
     $             //'approximate solution x is possible'
        else
          success = .false.
          err_msg = ' I do not know what this value of info means'
        endif

        if (.not.success) then
          write(6,'(A)') '  fm  Optimization FAILED:'
          write(6,'(A)') '  fm  ' // err_msg
          call STOPGM('fm_optimize_allcovalent', 'FAIL (see output)')
        endif

        write(6,'(A)') '  fm  Optimization successful:'
        write(6,'(A)') '  fm  ' // succ_msg

C_DEBUG[
C        write(6,*) 'deallocating wa. lwa is: ', lwa
C        write(6,*) ' allocated(wa)?:', allocated(work_array)
C        write(6,*) ' size(wa):', size(work_array)
C        write(6,*) ' first 3 elements:', work_array(1:3)
C_DEBUG]
C        deallocate(work_array)
C_DEBUG[
C        write(6,*) 'done deallocating wa. '
C        call my_flush(6)
C_DEBUG]
        return
      end subroutine fm_optimize_allcovalent
C
C     ==================================================================
      subroutine fm_kfit_collect_optimized
C     ==--------------------------------------------------------------==
C     collect all bonds (and angles, impropers, dihedrals) that are 
C     optimized.
C     ==================================================================
        implicit none
#if defined (__GROMOS)
        include 'system.h' ! only for NAX in mm_dim.inc
        include 'gromos.h'
        include 'mm_dim.inc'
        include 'forcematch.inc'
C       Variables
C       loop counters, temporaries, etc.
        integer i, j, ii

        write(6,'(A)') '  fm  collecting covalent interactions that are'
        write(6,'(A)') '  fm  affected by optimization'
C       bonds
C        allocate(fm_ib(0:2, MAXBNH+MAXBON))
        allocate(fm_ib(0:2, NBONH + NBON))
        ii = 0
        do i = 1, NBONH
          do j = 1, fm_nbonopt
C            if(ICBH(i).eq.j) then
            if(ICBH(i).eq.fm_bonopt(j)) then
              ii = ii + 1
              fm_ib(0, ii) = j
              fm_ib(1, ii) = NAT_cpmd(IBH(i))
              fm_ib(2, ii) = NAT_cpmd(JBH(i))
C_DEBUG[
C      write(6,*) " deb: bond i, ICBH(i), fm_bond i"
C      write(6,*) i, ICBH(i), ii
C      write(6,*) "      fm_ib(0-2, ii)"
C      write(6,*) fm_ib(:, ii)
C_DEBUG]
            endif
          enddo
        enddo
        do i = 1, NBON
          do j = 1, fm_nbonopt
C            if(ICB(i).eq.j) then
            if(ICB(i).eq.fm_bonopt(j)) then
              ii = ii + 1
              fm_ib(0, ii) = j
              fm_ib(1, ii) = NAT_cpmd(IB(i))
              fm_ib(2, ii) = NAT_cpmd(JB(i))
C_DEBUG[
C      write(6,*) " deb: bond i, ICB(i), fm_bond i"
C      write(6,*) i, ICB(i), ii
C      write(6,*) "fm_bonopt(j):", fm_bonopt(j)
C      write(6,*) "      fm_ib(0-2, ii)"
C      write(6,*) fm_ib(:, ii)
C_DEBUG]
            endif
          enddo
        enddo
        fm_nbon = ii
        write(6,'(A,i6)') '  fm  bonds:     ', fm_nbon
C angles
        allocate(fm_it(0:3, MXQHEH+MAXTHE))
        ii = 0
        do i = 1, NTHEH
          do j = 1, fm_nangopt
C            if(ICTH(i).eq.j) then
            if(ICTH(i).eq.fm_angopt(j)) then
              ii = ii + 1
              fm_it(0, ii) = j
              fm_it(1, ii) = NAT_cpmd(ITH(i))
              fm_it(2, ii) = NAT_cpmd(JTH(i))
              fm_it(3, ii) = NAT_cpmd(KTH(i))
            endif
          enddo
        enddo
        do i = 1, NTHE
          do j = 1, fm_nangopt
C            if(ICT(i).eq.j) then
            if(ICT(i).eq.fm_angopt(j)) then
              ii = ii + 1
              fm_it(0, ii) = j
              fm_it(1, ii) = NAT_cpmd(IT(i))
              fm_it(2, ii) = NAT_cpmd(JT(i))
              fm_it(3, ii) = NAT_cpmd(KT(i))
            endif
          enddo
        enddo
        fm_nang = ii
        write(6,'(A,i6)') '  fm  angles:    ', fm_nang
C impropers
        allocate(fm_iq(0:4, MAXHIH+MAXQHI))
        ii = 0
        do i = 1, NQHIH
          do j = 1, fm_nimpopt
C            if(ICQH(i).eq.j) then
            if(ICQH(i).eq.fm_impopt(j)) then
              ii = ii + 1
              fm_iq(0, ii) = j
              fm_iq(1, ii) = NAT_cpmd(IQH(i))
              fm_iq(2, ii) = NAT_cpmd(JQH(i))
              fm_iq(3, ii) = NAT_cpmd(KQH(i))
              fm_iq(4, ii) = NAT_cpmd(LQH(i))
            endif
          enddo
        enddo
        do i = 1, NQHI
          do j = 1, fm_nimpopt
C            if(ICQ(i).eq.j) then
            if(ICQ(i).eq.fm_impopt(j)) then
              ii = ii + 1
              fm_iq(0, ii) = j
              fm_iq(1, ii) = NAT_cpmd(IQ(i))
              fm_iq(2, ii) = NAT_cpmd(JQ(i))
              fm_iq(3, ii) = NAT_cpmd(KQ(i))
              fm_iq(4, ii) = NAT_cpmd(LQ(i))
            endif
          enddo
        enddo
        fm_nimp = ii
        write(6,'(A,i6)') '  fm  impropers: ', fm_nimp
C dihedrals
        allocate(fm_ip(0:4, MXPHIH+MAXPHI))
        ii = 0
        do i = 1, NPHIH
          do j = 1, fm_ndihopt
C            if(ICPH(i).eq.j) then
            if(ICPH(i).eq.fm_dihopt(j)) then
              ii = ii + 1
              fm_ip(0, ii) = j
              fm_ip(1, ii) = NAT_cpmd(IPH(i))
              fm_ip(2, ii) = NAT_cpmd(JPH(i))
              fm_ip(3, ii) = NAT_cpmd(KPH(i))
              fm_ip(4, ii) = NAT_cpmd(LPH(i))
            endif
          enddo
        enddo
        do i = 1, NPHI
          do j = 1, fm_ndihopt
C            if(ICP(i).eq.j) then
            if(ICP(i).eq.fm_dihopt(j)) then
              ii = ii + 1
              fm_ip(0, ii) = j
              fm_ip(1, ii) = NAT_cpmd(IP(i))
              fm_ip(2, ii) = NAT_cpmd(JP(i))
              fm_ip(3, ii) = NAT_cpmd(KP(i))
              fm_ip(4, ii) = NAT_cpmd(LP(i))
            endif
          enddo
        enddo
        fm_ndih = ii
        write(6,'(A,i6)') '  fm  dihedrals: ', fm_ndih

        return
#endif
      end subroutine fm_kfit_collect_optimized
C
C     ==================================================================
      subroutine fm_initialguess(npar, par)
C     ==--------------------------------------------------------------==
C     provide an initial guess from values in the original topology
C     ==================================================================
        implicit none
C       Arguments
C       total nr. of parameters
        integer npar
C       parameters
        real*8  par(npar)
#if defined (__GROMOS)
        include 'gromos.h'
        include 'mm_input.inc' !qmmm_amber
        include 'fm_cnst.inc'
        include 'forcematch.inc'
C       Variables
C       loop counters, temporaries, etc.
        integer iframe, iqm, ipar, jpar

        write(6,'(A)') '  fm  Getting initial guess'
        jpar = 0
        do ipar = 1, fm_nbonopt
          jpar = jpar + 1
          par(jpar) = fm_kb(ipar)
          jpar = jpar + 1
          par(jpar) = fm_b0(ipar)
        enddo
        do ipar = 1, fm_nangopt
          jpar = jpar + 1
          par(jpar) = fm_kt(ipar)
          jpar = jpar + 1
          par(jpar) = fm_t0(ipar)
        enddo
        do ipar = 1, fm_nimpopt
          jpar = jpar + 1
          par(jpar) = fm_kq(ipar)
        enddo
        do ipar = 1, fm_ndihopt
          jpar = jpar + 1
          par(jpar) = fm_kp(ipar)
        enddo
        if (jpar .ne. npar) call STOPGM('fm_initialguess', 
     $                                  'jpar .ne. npar')

        return
#endif
      end subroutine fm_initialguess
C
C     ==================================================================
      subroutine fm_kfit_setup_mytypes
C     ==--------------------------------------------------------------==
        implicit none
#if defined (__GROMOS)
        include 'gromos.h'
        include 'mm_input.inc' ! qmmm_amber
        include 'forcematch.inc'
        include 'fm_cnst.inc'
C Variables
C       loop counters
        integer i, j

        write(6,'(A)') '  fm  Setting up'
C copy qmmm_amber
        fm_amber = qmmm_amber

C Bonds
        allocate(fm_kb(fm_nbonopt))
        allocate(fm_b0(fm_nbonopt))
        do i = 1, fm_nbonopt
C         put back saved value
          CB(fm_bonopt(i)) = fm_kbon(i)
C         convert parameters to a.u.
          fm_b0(i) = NM_BOHR * B0(fm_bonopt(i))
          fm_kb(i) = kb_grm2au * CB(fm_bonopt(i))
          if (fm_amber) then
            fm_kb(i) = 2.d0 * fm_b0(i)**2 * fm_kb(i) 
          endif
        enddo

C Angles
        allocate(fm_kt(fm_nangopt))
        allocate(fm_t0(fm_nangopt)) 
        do i = 1, fm_nangopt
C         put back saved value
          CT(fm_angopt(i)) = fm_kang(i)
C         convert parameters to a.u.
          fm_t0(i) = T0(fm_angopt(i))
          if (fm_amber) then
C We use the simplified conversion formula given in the GROMACS manual
C (as opposed to the temperature-dependent one in the GROMOS manual)
C according to the GROMACS manual, the error at 300K is ~0.2 %
            fm_kt(i) = KJM_AU * dsin(dacos(T0(fm_angopt(i))))**2 
     $               * CT(fm_angopt(i))
          else
            fm_kt(i) = KJM_AU * CT(fm_angopt(i))
          endif
        enddo

C Impropers
        allocate(fm_kq(fm_nimpopt))
        allocate(fm_q0(fm_nimpopt)) 
        do i = 1, fm_nimpopt
C         put back saved value
          CQ(fm_impopt(i)) = fm_kimp(i)
C         convert parameters to a.u.
          fm_q0(i) = Q0(fm_impopt(i))
          fm_kq(i) = KJM_AU * CQ(fm_impopt(i))
        enddo

C Dihedrals
        allocate(fm_kp(fm_ndihopt))
        allocate(fm_p0(fm_ndihopt)) 
        allocate(fm_pn(fm_ndihopt))
        do i = 1, fm_ndihopt
C         put back saved value
          CP(fm_dihopt(i)) = fm_kdih(i)
C         convert parameters to a.u.
          fm_p0(i) = PD(fm_dihopt(i))
          fm_pn(i) = dble(NP(fm_dihopt(i)))
          fm_kp(i) = KJM_AU * CP(fm_dihopt(i))
        enddo

        return
#endif
      end subroutine fm_kfit_setup_mytypes
C
C     ==================================================================
      subroutine fm_setup_topology(nrpt, nsolv, is_qm)
C     ==--------------------------------------------------------------==
C     setup topology information for force-matching, .i.e. create new 
C     types, etc.
C     ==================================================================
        implicit none
C       Arguments
C       nr of solute atoms, solvent atoms
        integer nrpt, nsolv
C       are atoms QM?
        logical is_qm(nrpt+nsolv)
#if defined (__GROMOS)
        include 'gromos.h'
        include 'forcematch.inc'
        include 'mm_input.inc'
C       Variables
C       function that checks if two interactions are between equivalent
C       atoms
        logical are_equivalent
C       we need a new type
        logical need_new
C       array to flag type to be optimized
        logical, allocatable    :: optimize(:)
C       keep some old values
        integer old_type, old_ntypes
C       iterators
        integer i, j, k
C
        write(6,'(A)') '  fm '
        write(6,'(A)') '  fm  Setting up topology for force-matching '
     $                 // 'and QM/MM'
C       Initialize
        fm_nbonopt = 0
        fm_nangopt = 0
        fm_nimpopt = 0
        fm_ndihopt = 0
        allocate(fm_bonopt(MAXNBT))
        allocate(fm_angopt(MAXTTY))
        allocate(fm_impopt(MAXQTY))
        allocate(fm_dihopt(MAXPTY))
        allocate(optimize(MAX(MAXNBT,MAXTTY,MAXQTY,MAXPTY)))
C create new types for interactions that are between QM atoms, but also
C appear between QM and MM or MM and MM atoms
C First come bonds involving hydrogens. How we do it: Loop over all
C bonds involving hydrogens (HBOND), and check if both atoms are QM. 
C If so, loop over all HBOND of same type and see if there is one that
C involves at least one MM atom. If that is the case, we need a new bond
C type for the QM only bond, so we create it as NBTY+1. Then loop again
C over the remaining HBOND of same (old) type to find all-QM bonds that
C are equivalent to the one just created, and change their bond type.
C        
C This procedure is then repeated for bonds not involving hydrogens, and
C analogously for angles, impropers, and dihedrals, each with and
C without H...
C That's what happens in the following ~300 lines of code...
C
C TODO: For dihedrals (and maybe impropers), we should probably not
C create a new type for any non-equivalent quadruple of QM atoms.
C Instead, if a type involves both QM-QM and QM-MM/MM-MM interactions,
C simply duplicate it and fit to ONE type regardless of inequivalent QM
C atoms
C FIXME: If there is the same type between inequivalent QM atoms, but
C but type is not used by QM-MM or MM-MM interactions, then NO new type 
C is created, i.e. inequivalent QM atoms will interact via the same old
C type (for dihedrals, this is probably what we want anyway..)
C DONE: The above is fixed for bonds
C
C-------bonds involving hydrogens
        optimize = .false.
        old_ntypes = NBTY
        do i = 1, NBONH
          need_new = .false.
          if (is_qm(IBH(i)) .and. is_qm(JBH(i))) then
C           this bond is QM only. Now loop over other bonds of same type
C           to see if this type occurs also as MM-QM or MM-MM
            do j = 1, NBONH
              if (j.eq.i) cycle
              if (ICBH(i).eq.ICBH(j)) then
                if (.not.is_qm(IBH(j)) .or. .not.is_qm(JBH(j))) then
                  need_new = .true.
                  exit
                endif
              endif
            enddo
C           now make sure that we do not have a earlier QM-QM bond of
C           this type with NON-equivalent atoms
            if (.not.need_new .and. optimize(ICBH(i))) then
              do j = 1, i-1
                if (ICBH(i).eq.ICBH(j)) then
                  if (.not.are_equivalent('hbond',i,j)) then
                    need_new = .true.
                    exit
                  endif
                endif
              enddo
            endif
            if (.not.need_new) then
              optimize(ICBH(i)) = .true.
              cycle
            endif
            old_type = ICBH(i)
            NBTY = NBTY + 1
            if (NBTY.gt.MAXNBT) then
              call STOPGM('fm_setup_topology','NBTY.gt.MAXNBT')
            endif
            ICBH(i) = NBTY
            CB(NBTY) = CB(old_type)
            B0(NBTY) = B0(old_type)
            ! check for equivalencies
            do j = i+1, NBONH
              if (ICBH(j) .eq. old_type) then
                if (is_qm(IBH(j)).and.is_qm(JBH(j))) then
                  if(are_equivalent('hbond', i, j)) ICBH(j) = NBTY
                endif
              endif
            enddo
            optimize(ICBH(i)) = .true.
          endif ! both atoms are QM
        enddo
        write(6, '(A, i6)') '  fm  New bond types for bonds with H: ', 
     $                      NBTY - old_ntypes
C-------bonds not involving hydrogens
        old_ntypes = NBTY
        do i = 1, NBON
          need_new = .false.
          if (is_qm(IB(i)) .and. is_qm(JB(i))) then
C           this bond is QM only. Now loop over other bonds of same type
C           to see if this type occurs also as MM-QM or MM-MM
            do j = 1, NBON
              if (j.eq.i) cycle
              if (ICB(i).eq.ICB(j)) then
                if (.not.is_qm(IB(j)) .or. .not.is_qm(JB(j))) then
                  need_new = .true.
                  exit
                endif
              endif
            enddo
C           now make sure that we do not have a earlier QM-QM bond of
C           this type with NON-equivalent atoms
            if (.not.need_new .and. optimize(ICB(i))) then
              do j = 1, i-1
                if (ICB(i).eq.ICB(j)) then
                  if (.not.are_equivalent('bond',i,j)) then
                    need_new = .true.
                    exit
                  endif
                endif
              enddo
            endif
            if (.not.need_new) then
              optimize(ICB(i)) = .true.
              cycle
            endif
            old_type = ICB(i)
            NBTY = NBTY + 1
            if (NBTY.gt.MAXNBT) then
              call STOPGM('fm_setup_topology','NBTY.gt.MAXNBT')
            endif
            ICB(i) = NBTY
            CB(NBTY) = CB(old_type)
            B0(NBTY) = B0(old_type)
C           check for equivalencies
            do j = i+1, NBON
              if (ICB(j) .eq. old_type) then
                if (is_qm(IB(j)).and.is_qm(JB(j))) then
                  if(are_equivalent('bond', i, j)) ICB(j) = NBTY
                endif
              endif
            enddo
            optimize(ICB(i)) = .true.
          endif
        enddo
        write(6, '(A, i6)') '  fm  New bond types for other bonds:  ', 
     $                      NBTY - old_ntypes
        fm_nbonopt = 0
        do i = 1, NBTY
          if (optimize(i)) then
            fm_nbonopt = fm_nbonopt + 1
            fm_bonopt(fm_nbonopt) = i
          endif
        enddo
C---------------------------------
C-------angles involving hydrogens
        optimize = .false.
        old_ntypes = NTTY
        do i = 1, NTHEH
          need_new = .false.
          if (is_qm(ITH(i)).and.is_qm(JTH(i)).and.is_qm(KTH(i))) then
C           this angle is QM only. Now loop over other angles of same type
C           to see if this type occurs also as MM-QM or MM-MM
            do j = 1, NTHEH
              if (j.eq.i) cycle
              if (ICTH(i).eq.ICTH(j)) then
                if (.not.is_qm(ITH(j)) .or. .not.is_qm(JTH(j)) .or.
     $              .not.is_qm(KTH(j))) then
                  need_new = .true.
                  exit
                endif
              endif
            enddo
C           now make sure that we do not have a earlier QM-QM angle of
C           this type with NON-equivalent atoms
            if (.not.need_new .and. optimize(ICTH(i))) then
              do j = 1, i-1
                if (ICTH(i).eq.ICTH(j)) then
                  if (.not.are_equivalent('hangl',i,j)) then
                    need_new = .true.
                    exit
                  endif
                endif
              enddo
            endif
            if (.not.need_new) then
              optimize(ICTH(i)) = .true.
              cycle
            endif
            old_type = ICTH(i)
            NTTY = NTTY + 1
            if (NTTY.gt.MAXTTY) then
              call STOPGM('fm_setup_topology','NTTY.gt.MAXTTY')
            endif
            ICTH(i) = NTTY
            CT(NTTY) = CT(old_type)
            T0(NTTY) = T0(old_type)
            ! check for equivalencies
            do j = i+1, NTHEH
              if (ICTH(j) .eq. old_type) then
                if (is_qm(ITH(j)) .and. is_qm(JTH(j)) .and. 
     $              is_qm(KTH(j))) then
                  if(are_equivalent('hangl', i, j)) ICTH(j) = NTTY
                endif
              endif
            enddo
            optimize(ICTH(i)) = .true.
          endif
        enddo
        write(6, '(A, i6)') '  fm  New types for angles with H:     ', 
     $                      NTTY - old_ntypes
C-------angles not involving hydrogens
        old_ntypes = NTTY
        do i = 1, NTHE
          need_new = .false.
          if (is_qm(IT(i)).and.is_qm(JT(i)).and.is_qm(KT(i))) then
C           this angle is QM only. Now loop over other angles of same type
C           to see if this type occurs also as MM-QM or MM-MM
            do j = 1, NTHE
              if (j.eq.i) cycle
              if (ICT(i).eq.ICT(j)) then
                if (.not.is_qm(IT(j)) .or. .not.is_qm(JT(j)) .or.
     $              .not.is_qm(KT(j))) then
                  need_new = .true.
                  exit
                endif
              endif
            enddo
C           now make sure that we do not have a earlier QM-QM angle of
C           this type with NON-equivalent atoms
            if (.not.need_new .and. optimize(ICT(i))) then
              do j = 1, i-1
                if (ICT(i).eq.ICT(j)) then
                  if (.not.are_equivalent('angl',i,j)) then
                    need_new = .true.
                    exit
                  endif
                endif
              enddo
            endif
            if (.not.need_new) then
              optimize(ICT(i)) = .true.
              cycle
            endif
            old_type = ICT(i)
            NTTY = NTTY + 1
            if (NTTY.gt.MAXTTY) then
              call STOPGM('fm_setup_topology','NTTY.gt.MAXTTY')
            endif
            ICT(i) = NTTY
            CT(NTTY) = CT(old_type)
            T0(NTTY) = T0(old_type)
            ! check for equivalencies
            do j = i+1, NTHE
              if (ICT(j) .eq. old_type) then
                if (is_qm(IT(j)) .and. is_qm(JT(j)) .and. 
     $              is_qm(KT(j))) then
                  if(are_equivalent('angl', i, j)) ICT(j) = NTTY
                endif
              endif
            enddo
            optimize(ICT(i)) = .true.
          endif
        enddo
        write(6, '(A, i6)') '  fm  New types for other angles:      ', 
     $                      NTTY - old_ntypes
        fm_nangopt = 0
        do i = 1, NTTY
          if (optimize(i)) then
            fm_nangopt = fm_nangopt + 1
            fm_angopt(fm_nangopt) = i
          endif
        enddo
C---------------------------------------------
C-------improper dihedrals involving hydrogens
        optimize = .false.
        old_ntypes = NQTY
        do i = 1, NQHIH
          need_new = .false.
          if (is_qm(IQH(i)).and.is_qm(JQH(i)).and.is_qm(KQH(i)).and.
     $        is_qm(LQH(i))) then
C           this angle is QM only. Now loop over other angles of same type
C           to see if this type occurs also as MM-QM or MM-MM
            do j = 1, NQHIH
              if (j.eq.i) cycle
              if (ICQH(i).eq.ICQH(j)) then
                if (.not.is_qm(IQH(j)) .or. .not.is_qm(JQH(j)) .or.
     $              .not.is_qm(KQH(j)) .or. .not.is_qm(LQH(j))) then
                  need_new = .true.
                  exit
                endif
              endif
            enddo
            if (.not.need_new) then
              optimize(ICQH(i)) = .true.
              cycle
            endif
            old_type = ICQH(i)
            NQTY = NQTY + 1
            if (NQTY.gt.MAXQTY) then
              call STOPGM('fm_setup_topology','NQTY.gt.MAXQTY')
            endif
            ICQH(i) = NQTY
            CQ(NQTY) = CQ(old_type)
            Q0(NQTY) = Q0(old_type)
            ! check for equivalencies
            do j = i+1, NQHIH
              if (ICQH(j) .eq. old_type) then
                if (is_qm(IQH(j)) .and. is_qm(JQH(j)) .and. 
     $              is_qm(KQH(j)) .and. is_qm(LQH(j))) then
                  if(are_equivalent('himpd', i, j)) ICQH(j) = NQTY
                endif
              endif
            enddo
            optimize(ICQH(i)) = .true.
          endif
        enddo
        write(6, '(A, i6)') '  fm  New types for impropers with H:  ', 
     $                      NQTY - old_ntypes
C-------improper dihedrals not involving hydrogens
        old_ntypes = NQTY
        do i = 1, NQHI
          need_new = .false.
          if (is_qm(IQ(i)).and.is_qm(JQ(i)).and.is_qm(KQ(i)).and.
     $        is_qm(LQ(i))) then
C           this angle is QM only. Now loop over other angles of same type
C           to see if this type occurs also as MM-QM or MM-MM
            do j = 1, NQHI
              if (j.eq.i) cycle
              if (ICQ(i).eq.ICQ(j)) then
                if (.not.is_qm(IQ(j)) .or. .not.is_qm(JQ(j)) .or.
     $              .not.is_qm(KQ(j)) .or. .not.is_qm(LQ(j))) then
                  need_new = .true.
                  exit
                endif
              endif
            enddo
            if (.not.need_new) then
              optimize(ICQ(i)) = .true.
              cycle
            endif
            old_type = ICQ(i)
            NQTY = NQTY + 1
            if (NQTY.gt.MAXQTY) then
              call STOPGM('fm_setup_topology','NQTY.gt.MAXQTY')
            endif
            ICQ(i) = NQTY
            CQ(NQTY) = CQ(old_type)
            Q0(NQTY) = Q0(old_type)
            ! check for equivalencies
            do j = i+1, NQHI
              if (ICQ(j) .eq. old_type) then
                if (is_qm(IQ(j)) .and. is_qm(JQ(j)) .and. 
     $              is_qm(KQ(j)) .and. is_qm(LQ(j))) then
                  if(are_equivalent('impd', i, j)) ICQ(j) = NQTY
                endif
              endif
            enddo
            optimize(ICQ(i)) = .true.
          endif
        enddo
        write(6, '(A, i6)') '  fm  New types for other impropers:   ', 
     $                      NQTY - old_ntypes
        fm_nimpopt = 0
        do i = 1, NQTY
          if (optimize(i)) then
            fm_nimpopt = fm_nimpopt + 1
            fm_impopt(fm_nimpopt) = i
          endif
        enddo
C----------------------------------------------
C-------proper dihedrals involving hydrogens
C for dihedrals, we only duplicate a type if it has all-QM and QM/MM or
C all-MM atoms, but we do not create new types if it involves
C non-equivalent QM atoms
C we do not fit dihedrals with force constant zero (they occur in Amber;
C I think they are used to signal 1-4 interactions)
        optimize = .false.
        old_ntypes = NPTY
        do i = 1, NPHIH
          if (CP(ICPH(i)) .le. 1.0d-3) cycle
          need_new = .false.
          if (is_qm(IPH(i)).and.is_qm(JPH(i)).and.is_qm(KPH(i)).and.
     $        is_qm(LPH(i))) then
C           this angle is QM only. Now loop over other angles of same type
C           to see if this type occurs also as MM-QM or MM-MM
            do j = 1, NPHIH
              if (j.eq.i) cycle
              if (ICPH(i).eq.ICPH(j)) then
                if (.not.is_qm(IPH(j)) .or. .not.is_qm(JPH(j)) .or.
     $              .not.is_qm(KPH(j)) .or. .not.is_qm(LPH(j))) then
                  need_new = .true.
                  exit
                endif
              endif
            enddo
            if (.not.need_new) then
              optimize(ICPH(i)) = .true.
              cycle
            endif
            old_type = ICPH(i)
            NPTY = NPTY + 1
            if (NPTY.gt.MAXPTY) then
              call STOPGM('fm_setup_topology','NPTY.gt.MAXPTY')
            endif
            ICPH(i) = NPTY
            CP(NPTY) = CP(old_type)
            PD(NPTY) = PD(old_type)
            NP(NPTY) = NP(old_type)
            ! check for equivalencies
            do j = i+1, NPHIH
              if (ICPH(j) .eq. old_type) then
                if (is_qm(IPH(j)) .and. is_qm(JPH(j)) .and. 
     $              is_qm(KPH(j)) .and. is_qm(LPH(j))) then
C                  if(are_equivalent('hdihe', i, j)) ICPH(j) = NPTY
                  ICPH(j) = NPTY
                endif
              endif
            enddo
            optimize(ICPH(i)) = .true.
          endif
        enddo
        write(6, '(A, i6)') '  fm  New types for dihedrals with H:  ', 
     $                      NPTY - old_ntypes
C-------proper dihedrals not involving hydrogens
        old_ntypes = NPTY
        do i = 1, NPHI
          if (CP(ICP(i)) .le. 1.0d-3) cycle
          need_new = .false.
          if (is_qm(IP(i)).and.is_qm(JP(i)).and.is_qm(KP(i)).and.
     $        is_qm(LP(i))) then
C           this angle is QM only. Now loop over other angles of same type
C           to see if this type occurs also as MM-QM or MM-MM
            do j = 1, NPHI
              if (j.eq.i) cycle
              if (ICP(i).eq.ICP(j)) then
                if (.not.is_qm(IP(j)) .or. .not.is_qm(JP(j)) .or.
     $              .not.is_qm(KP(j)) .or. .not.is_qm(LP(j))) then
                  need_new = .true.
                  exit
                endif
              endif
            enddo
            if (.not.need_new) then
              optimize(ICP(i)) = .true.
              cycle
            endif
            old_type = ICP(i)
            NPTY = NPTY + 1
            if (NPTY.gt.MAXPTY) then
              call STOPGM('fm_setup_topology','NPTY.gt.MAXPTY')
            endif
            ICP(i) = NPTY
            CP(NPTY) = CP(old_type)
            PD(NPTY) = PD(old_type)
            NP(NPTY) = NP(old_type)
            ! check for equivalencies
            do j = i+1, NPHI
              if (ICP(j) .eq. old_type) then
                if (is_qm(IP(j)) .and. is_qm(JP(j)) .and. 
     $              is_qm(KP(j)) .and. is_qm(LP(j))) then
C                  if(are_equivalent('dihe', i, j)) ICP(j) = NPTY
                  ICP(j) = NPTY
                endif
              endif
            enddo
            optimize(ICP(i)) = .true.
          endif
        enddo
        write(6, '(A, i6)') '  fm  New types for other dihedrals:   ', 
     $                      NPTY - old_ntypes
        fm_ndihopt = 0
        do i = 1, NPTY
          if (optimize(i)) then
            fm_ndihopt = fm_ndihopt + 1
            fm_dihopt(fm_ndihopt) = i
          endif
        enddo
C---------------------------------------------------------
C--Done, finally!
C Some printing out
        write(6, '(A, i6)') '  fm'
        write(6, '(A, i6)') '  fm  Nr. of bond types to optimize:     ',
     $                         fm_nbonopt
        write(6, '(A, i6)') '  fm  Nr. of angle types to optimize:    ',
     $                         fm_nangopt
        write(6, '(A, i6)') '  fm  Nr. of improper types to optimize: ',
     $                         fm_nimpopt
        write(6, '(A, i6)') '  fm  Nr. of dihedral types to optimize: ',
     $                         fm_ndihopt
        if (fm_nbonopt .gt. 0) then
          write(6, '(A)') '  fm'
          write(6, '(A)') '  fm  List of optimized bond types:'
          write(6, 211) fm_bonopt(1:fm_nbonopt)
        endif
        if (fm_nangopt .gt. 0) then
          write(6, '(A)') '  fm  List of optimized angle types:'
          write(6, 211) fm_angopt(1:fm_nangopt)
        endif
        if (fm_nimpopt .gt. 0) then
          write(6, '(A)') '  fm  List of optimized improper types:'
          write(6, 211) fm_impopt(1:fm_nimpopt)
        endif
        if (fm_ndihopt .gt. 0) then
          write(6, '(A)') '  fm  List of optimized dihedral types:'
          write(6, 211) fm_dihopt(1:fm_ndihopt)
        endif
 211    format('  fm  ',9(i8))
        WRITE(6,*)
     c' fm fm fm fm fm fm fm fm fm fm fm fm fm fm fm fm fm fm fm fm fm'
        

C set force constants of QM bonds, etc., to zero (needed 
C for QM/MM), but store values for later use as an initial guess

        allocate(fm_kbon(fm_nbonopt))
        do i = 1, fm_nbonopt
          fm_kbon(i) = CB(fm_bonopt(i))
          CB(fm_bonopt(i)) = 0.d0
        enddo

        allocate(fm_kang(fm_nangopt))
        do i = 1, fm_nangopt
          fm_kang(i) = CT(fm_angopt(i))
          CT(fm_angopt(i)) = 0.d0
        enddo

        allocate(fm_kimp(fm_nimpopt))
        do i = 1, fm_nimpopt
          fm_kimp(i) = CQ(fm_impopt(i))
          CQ(fm_impopt(i)) = 0.d0
        enddo

        allocate(fm_kdih(fm_ndihopt))
        do i = 1, fm_ndihopt
          fm_kdih(i) = CP(fm_dihopt(i))
          CP(fm_dihopt(i)) = 0.d0
        enddo

        deallocate(optimize)
#endif
        return
      end subroutine fm_setup_topology
C
C     ==================================================================
      logical function are_equivalent(ia, i1, i2)
C     ==--------------------------------------------------------------==
C     checks if bonds, angles, etc. are between equivalent atoms
C     ==================================================================
        implicit none
C       Arguments
C       type of interaction
        character(*)    ia
C       nr of interactions
        integer         i1, i2
#if defined (__GROMOS)
        include 'gromos.h'
        include 'forcematch.inc'
C       Variables
        integer         a1, b1, c1, d1
        integer         a2, b2, c2, d2
C for now, return false...
C        are_equivalent = .false.
C        return

C first get unique type number
        if (ia.eq.'hbond') then
          a1 = grm_equiv(IBH(i1))
          b1 = grm_equiv(JBH(i1))
          a2 = grm_equiv(IBH(i2))
          b2 = grm_equiv(JBH(i2))
        elseif (ia.eq.'bond') then
          a1 = grm_equiv(IB(i1))
          b1 = grm_equiv(JB(i1))
          a2 = grm_equiv(IB(i2))
          b2 = grm_equiv(JB(i2))
        elseif (ia.eq.'hangl') then
          a1 = grm_equiv(ITH(i1))
          b1 = grm_equiv(JTH(i1))
          c1 = grm_equiv(KTH(i1))
          a2 = grm_equiv(ITH(i2))
          b2 = grm_equiv(JTH(i2))
          c2 = grm_equiv(KTH(i2))
        elseif (ia.eq.'angl') then
          a1 = grm_equiv(IT(i1))
          b1 = grm_equiv(JT(i1))
          c1 = grm_equiv(KT(i1))
          a2 = grm_equiv(IT(i2))
          b2 = grm_equiv(JT(i2))
          c2 = grm_equiv(KT(i2))
        elseif (ia.eq.'himpd') then
          a1 = grm_equiv(IQH(i1))
          b1 = grm_equiv(JQH(i1))
          c1 = grm_equiv(KQH(i1))
          d1 = grm_equiv(LQH(i1))
          a2 = grm_equiv(IQH(i2))
          b2 = grm_equiv(JQH(i2))
          c2 = grm_equiv(KQH(i2))
          d2 = grm_equiv(LQH(i2))
        elseif (ia.eq.'impd') then
          a1 = grm_equiv(IQ(i1))
          b1 = grm_equiv(JQ(i1))
          c1 = grm_equiv(KQ(i1))
          d1 = grm_equiv(LQ(i1))
          a2 = grm_equiv(IQ(i2))
          b2 = grm_equiv(JQ(i2))
          c2 = grm_equiv(KQ(i2))
          d2 = grm_equiv(LQ(i2))
        elseif (ia.eq.'hdihe') then
          a1 = grm_equiv(IPH(i1))
          b1 = grm_equiv(JPH(i1))
          c1 = grm_equiv(KPH(i1))
          d1 = grm_equiv(LPH(i1))
          a2 = grm_equiv(IPH(i2))
          b2 = grm_equiv(JPH(i2))
          c2 = grm_equiv(KPH(i2))
          d2 = grm_equiv(LPH(i2))
        elseif (ia.eq.'dihe') then
          a1 = grm_equiv(IP(i1))
          b1 = grm_equiv(JP(i1))
          c1 = grm_equiv(KP(i1))
          d1 = grm_equiv(LP(i1))
          a2 = grm_equiv(IP(i2))
          b2 = grm_equiv(JP(i2))
          c2 = grm_equiv(KP(i2))
          d2 = grm_equiv(LP(i2))
        else
C       we should never arrive here...
          call STOPGM('are_equivalent', 'internal error! unknown ia: '
     $                 // ia)
        endif

C bonds
        if (ia.eq.'bond' .or. ia.eq.'hbond') then
          if (a1.eq.a2 .and. b1.eq.b2) then
            are_equivalent = .true.
          elseif (a1.eq.b2 .and. a2.eq.b1) then
            are_equivalent = .true.
          else
            are_equivalent = .false.
          endif
          return
C angles
        elseif (ia.eq.'angl' .or. ia.eq.'hangl') then
          if (b1.eq.b2) then
            if (a1.eq.a2 .and. c1.eq.c2) then
              are_equivalent = .true.
C            elseif (a1.eq.b2 .and. a2.eq.b1) then
            elseif (a1.eq.c2 .and. a2.eq.c1) then
              are_equivalent = .true.
            else
              are_equivalent = .false.
            endif
          else
            are_equivalent = .false.
          endif
          return
C dihedrals (proper or not)
        elseif( (ia.eq.'hdihe' .or. ia.eq.'dihe') .or.
     $          (ia.eq.'himpd' .or. ia.eq.'impd') ) then
          if(a1.eq.a2 .and. b1.eq.b2 .and. c1.eq.c2 .and. d1.eq.d2)then
            are_equivalent = .true.
          elseif(a1.eq.d2 .and. b1.eq.c2 .and. c1.eq.b2 .and. d1.eq.a2)
     $      then
            are_equivalent = .true.
          else
            are_equivalent = .false.
          endif

        else
C       we should never arrive here...
          call STOPGM('are_equivalent', 'internal error! unknown ia: '
     $                 // ia)
        endif
#else
        are_equivalent = .false.
#endif
      end function are_equivalent
C
C     ==================================================================
      subroutine fm_wopt(trget_size, nw, w, fdiff, iflag)
C     ==--------------------------------------------------------------==
        implicit none
        include 'forcematch.inc'
        integer trget_size, nw, iflag
        real*8  w(nw), fdiff(trget_size)
      end subroutine fm_wopt
C
C     ==================================================================
      subroutine fm_fcn_allcovalent(tgsize, npar, par, fdiff, fjac,
     $                              ldfjac, iflag)
C     ==--------------------------------------------------------------==
        implicit none
        include 'forcematch.inc'
C       Arguments
C       size of the target vector, nr. of parameters, leading dimension
C       of fjac
        integer tgsize, npar, ldfjac
C       parameters, target vector, jacobian matrix
        real*8  par(npar)
        real*8, target  ::  fdiff(tgsize)
        real*8, target  ::  fjac(ldfjac, npar)
C       flag that determines wether to compute fdiff or fjac
        integer iflag
C       Variables
C       point to windows of fdiff and fjac
        real*8, pointer :: fvec(:)
        real*8, pointer :: jac(:, :)
C       atomic coordinates
        real*8  ri(3), rj(3), rk(3), rl(3)
C       forces (or derivatives wrt. parameters)
        real*8  fi(3), fj(3), fk(3), fl(3)
C       absolute and relative RMS on forces
        real*8  rms, rms_rel
C       atom indices, type indices
        integer i, j, k, l
        integer bt, at, it, dt
C       iteration counter
        integer, save   :: itcounter = 0
C       loop counters, temporaries, etc.
        integer iframe, ibon, iang, iimp, idih
        integer lb, ub, ipar, ipar0

        if (iflag.eq.1) itcounter = itcounter + 1
        write(6,'(A)') '  fm  Entering fm_fcn_allcovalent'
        write(6,'(A,i3)') '  fm  iflag is ', iflag
        write(6,'(A,i8)') '  fm  iteration ', itcounter
        write(6,'(A,i8,i8)') '  fm  size of jacobian is ', ldfjac, npar
C       consistency check
        if (tgsize.ne.ldfjac) then
          write(6,'(A)') '  fm  =================================='
          write(6,'(A)') '  fm  ========== ERROR ! ==============='
          write(6,'(A)') '  fm  =================================='
          write(6,'(A)') '  fm   tgsize must be equal ldfjac, but '
          write(6,*) '  fm   they are: ', tgsize, ldfjac
          call STOPGM('fm_fcn_allcovalent','tgsize .ne. ldfjac')
        endif
          
        write(6,'(A)') '  fm  Entering fm_fcn_allcovalent'
        write(6,'(A,i3)') '  fm  iflag is ', iflag
        write(6,'(A,i8)') '  fm  iteration ', itcounter

        if (itcounter.gt.fm_max_kfit_iterations) then
          iflag = -1
          return
        endif

        if (iflag .eq. 1) then
          call AZZERO(fdiff, tgsize)
        else if (iflag .eq. 2) then
          call AZZERO(fjac, ldfjac*npar)
        else
          write(6,'(A)') '  fm  =================================='
          write(6,'(A)') '  fm  ========== ERROR ! ==============='
          write(6,'(A,i8)') '  fm  Illegal value for iflag: ', iflag
          call STOPGM('fm_fcn_allcovalent', 'illegal value for iflag')
        endif

        call fm_kfit_par_to_fmarray(npar, par)

C Loop over all frames. In each frame, we use fvec and jac to point to
C the appropriate "windows" of the full vector fdiff / full matrix fjac,
C to facilitate the indexing business
        do iframe = 1, fm_nframes
          lb = (iframe-1) * 3*fm_nrqm + 1
          ub = lb + 3*fm_nrqm - 1
          fvec => fdiff(lb:ub)
          jac => fjac(lb:ub, :)

C Bonds
          ipar0 = 0
          do ibon = 1, fm_nbon
            bt = fm_ib(0, ibon)
            i = fm_ib(1, ibon)
            j = fm_ib(2, ibon)
            ri = fm_rqm(:, i, iframe)
            rj = fm_rqm(:, j, iframe)

C_DEBUG[
C            write(6,*) "DEBUG"
C            write(6,*) "ibon, bt, i, j:"
C            write(6,*) ibon, bt, i, j
C_DEBUG]
C           compute i,j as index into array fvec
            i = 3*(i-1) + 1
            j = 3*(j-1) + 1
            if (iflag.eq.1) then
              call fm_forces_bon(fi, fj, ri, rj, bt, 'ff')
              fvec(i:i+2) = fvec(i:i+2) + fi
              fvec(j:j+2) = fvec(j:j+2) + fj
            else
              call fm_forces_bon(fi, fj, ri, rj, bt, 'dk')
              ipar = ipar0 + 2*bt - 1
              jac(i:i+2, ipar) = jac(i:i+2, ipar) + fi
              jac(j:j+2, ipar) = jac(j:j+2, ipar) + fj
              call fm_forces_bon(fi, fj, ri, rj, bt, 'd0')
              ipar = ipar + 1
              jac(i:i+2, ipar) = jac(i:i+2, ipar) + fi
              jac(j:j+2, ipar) = jac(j:j+2, ipar) + fj
            endif
          enddo
C Angles
          ipar0 = 2*fm_nbonopt
          do iang = 1, fm_nang
            at = fm_it(0, iang)
            i = fm_it(1, iang)
            j = fm_it(2, iang)
            k = fm_it(3, iang)
            ri = fm_rqm(:, i, iframe)
            rj = fm_rqm(:, j, iframe)
            rk = fm_rqm(:, k, iframe)

            i = 3*(i-1) + 1
            j = 3*(j-1) + 1
            k = 3*(k-1) + 1
            if (iflag.eq.1) then
              call fm_forces_ang(fi, fj, fk, ri, rj, rk, at, 'ff')
              fvec(i:i+2) = fvec(i:i+2) + fi
              fvec(j:j+2) = fvec(j:j+2) + fj
              fvec(k:k+2) = fvec(k:k+2) + fk
            else
              ipar = ipar0 + 2*at - 1
              call fm_forces_ang(fi, fj, fk, ri, rj, rk, at, 'dk')
              jac(i:i+2, ipar) = jac(i:i+2, ipar) + fi
              jac(j:j+2, ipar) = jac(j:j+2, ipar) + fj
              jac(k:k+2, ipar) = jac(k:k+2, ipar) + fk
              ipar = ipar + 1
              call fm_forces_ang(fi, fj, fk, ri, rj, rk, at, 'd0')
              jac(i:i+2, ipar) = jac(i:i+2, ipar) + fi
              jac(j:j+2, ipar) = jac(j:j+2, ipar) + fj
              jac(k:k+2, ipar) = jac(k:k+2, ipar) + fk
            endif
          enddo
C Impropers
          ipar0 = ipar0 + 2*fm_nangopt
          do iimp = 1, fm_nimp
            it = fm_iq(0, iimp)
            i = fm_iq(1, iimp)
            j = fm_iq(2, iimp)
            k = fm_iq(3, iimp)
            l = fm_iq(4, iimp)
            ri = fm_rqm(:, i, iframe)
            rj = fm_rqm(:, j, iframe)
            rk = fm_rqm(:, k, iframe)
            rl = fm_rqm(:, l, iframe)

            i = 3*(i-1) + 1
            j = 3*(j-1) + 1
            k = 3*(k-1) + 1
            l = 3*(l-1) + 1
            if (iflag.eq.1) then
              call fm_forces_imp(fi,fj,fk,fl,ri,rj,rk,rl,it,'ff')
              fvec(i:i+2) = fvec(i:i+2) + fi
              fvec(j:j+2) = fvec(j:j+2) + fj
              fvec(k:k+2) = fvec(k:k+2) + fk
              fvec(l:l+2) = fvec(l:l+2) + fl
            else
              ipar = ipar0 + it
              call fm_forces_imp(fi,fj,fk,fl,ri,rj,rk,rl,it,'dk')
              jac(i:i+2, ipar) = jac(i:i+2, ipar) + fi
              jac(j:j+2, ipar) = jac(j:j+2, ipar) + fj
              jac(k:k+2, ipar) = jac(k:k+2, ipar) + fk
              jac(l:l+2, ipar) = jac(l:l+2, ipar) + fl
            endif
          enddo
C Dihedrals
          ipar0 = ipar0 + fm_nimpopt
          do idih = 1, fm_ndih
            dt = fm_ip(0, idih)
            i = fm_ip(1, idih)
            j = fm_ip(2, idih)
            k = fm_ip(3, idih)
            l = fm_ip(4, idih)
            ri = fm_rqm(:, i, iframe)
            rj = fm_rqm(:, j, iframe)
            rk = fm_rqm(:, k, iframe)
            rl = fm_rqm(:, l, iframe)

            i = 3*(i-1) + 1
            j = 3*(j-1) + 1
            k = 3*(k-1) + 1
            l = 3*(l-1) + 1
            if (iflag.eq.1) then
              call fm_forces_dih(fi,fj,fk,fl,ri,rj,rk,rl,dt,'ff')
              fvec(i:i+2) = fvec(i:i+2) + fi
              fvec(j:j+2) = fvec(j:j+2) + fj
              fvec(k:k+2) = fvec(k:k+2) + fk
              fvec(l:l+2) = fvec(l:l+2) + fl
            else
              ipar = ipar0 + dt
              call fm_forces_dih(fi,fj,fk,fl,ri,rj,rk,rl,dt,'dk')
              jac(i:i+2, ipar) = jac(i:i+2, ipar) + fi
              jac(j:j+2, ipar) = jac(j:j+2, ipar) + fj
              jac(k:k+2, ipar) = jac(k:k+2, ipar) + fk
              jac(l:l+2, ipar) = jac(l:l+2, ipar) + fl
            endif
          enddo
        enddo ! loop over all frames

        if (iflag.eq.1) then
          fdiff = fdiff - reshape(fm_fqm_ref, (/tgsize/))
          call fm_kfit_compute_frms(rms, rms_rel, tgsize, fdiff, 
     $                              fm_fqm_ref)
          write(6,'(A)') '  fm  Abs. and rel. RMS on forces:'
          write(6,'(e10.4,1x,f8.3)') rms, rms_rel
        endif

        return
      end subroutine fm_fcn_allcovalent
C
C     ==================================================================
      subroutine fm_kfit_compute_frms(rms, rms_rel, fsize, fdiff, fref)
        implicit none
C     ==--------------------------------------------------------------==
C       Arguments
C       output: rms and relative rms
        real*8 rms, rms_rel
C       input: size of arrays, fdiff (force - ref.force), reference forces
        integer fsize
        real*8  fdiff(fsize), fref(fsize)
C       Variables
C       loop counter
        integer i

        rms = 0.d0
        rms_rel = 0.d0

C       we abuse rms_rel to sum over square reference forces
        do i = 1, fsize
          rms = rms + fdiff(i)**2
          rms_rel = rms_rel + fref(i)**2
        enddo
        rms_rel = sqrt(rms / rms_rel)
        rms = sqrt(rms / fsize)
        return
      end subroutine fm_kfit_compute_frms
C
C     ==================================================================
      subroutine fm_forces_bon(fi, fj, ri, rj, bt, flag)
C     ==--------------------------------------------------------------==
        implicit none
        include 'forcematch.inc'
C       Arguments
C       output: forces or derivatives, depending on flag
        real*8  fi(3), fj(3)
C       atomic coordinates
        real*8  ri(3), rj(3)
C       nr of bond type
        integer bt
C       flag that determines type of calculation:
C       'ff': calculate forces
C       'dk': derivative wrt. to kb
C       'd0': derivative wrt. to b0
        character*2     flag
C       Variables
C       distance, distance vector
        real*8  r, rij(3)
C       force constant, eq. bond length
        real*8  kb, b0

        call fm_dist(r, rij, ri, rj)
        kb = fm_kb(bt)
        b0 = fm_b0(bt)
        if (flag.eq.'ff') then
          if (fm_amber) then
            fi = -kb * (r - b0) * rij / r 
            fj = -fi
          else
            fi = -kb * (r**2 - b0**2) * rij
            fj = -fi
          endif
        elseif (flag.eq.'dk') then
          if (fm_amber) then
            fi = -(r - b0) * rij / r 
            fj = -fi
          else
            fi = -(r**2 - b0**2) * rij
            fj = -fi
          endif
        elseif (flag.eq.'d0') then
          if (fm_amber) then 
            fi = kb * rij / r
            fj = -fi
          else
            fi = 2.d0 * kb * b0 * rij
            fj = -fi
          endif
        else
          call STOPGM('fm_forces_bon', 'Unknown flag: ' // flag)
        endif

        return
      end subroutine fm_forces_bon
C
C     ==================================================================
      subroutine fm_forces_ang(fi,fj,fk,ri,rj,rk,at,flag)
C     ==--------------------------------------------------------------==
        implicit none
        include 'forcematch.inc'
C       Arguments
C       forces or derivatives
        real*8  fi(3), fj(3), fk(3)
C       coordinates
        real*8  ri(3), rj(3), rk(3)
C       angle type
        integer at
C       flag that determines what is calculated:
C       'ff': compute forces
C       'dk': derivative wrt. kt
C       'd0': derivative wrt. t0
        character*2     flag
C       Variables
C       distances, distance vectors
        real*8  dij, dkj
        real*8  rij(3), rkj(3)
C       angle or cosine of angle
        real*8  th
C       force constant, equilibrium angle
        real*8  kt, t0
C       scalar factor for amber
        real*8  fscal

C compute angle
        call fm_dist(dij, rij, ri, rj)
        call fm_dist(dkj, rkj, rk, rj)
        th = dot_product(rij, rkj) / (dij*dkj)

        kt = fm_kt(at)
        t0 = fm_t0(at)
        if (fm_amber) then
          th = acos(th)
          t0 = acos(t0)
        endif

        if (flag.eq.'ff' .or. flag.eq.'dk') then
          if (fm_amber) then
            fscal = (th - t0) / sin(th)
            fi = ( rkj*dij*dkj - dot_product(rij,rkj)*dkj*rij/dij ) 
     &         / (dij**2 * dkj**2)
            fi = fscal * fi
            fk = ( rij*dij*dkj - dot_product(rij,rkj)*dij*rkj/dkj )
     $         / (dij**2 * dkj**2)
            fk = fscal * fk
            fj = -fi - fk
          else
            fi = -(th - t0) * (rkj/dkj - (rij/dij)*th) / dij
            fk = -(th - t0) * (rij/dij - (rkj/dkj)*th) / dkj
            fj = -fi - fk
          endif
          if (flag.eq.'ff') then
            fi = kt * fi
            fj = kt * fj
            fk = kt * fk
          endif 
        elseif (flag.eq.'d0') then
          if (fm_amber) then
            fi = ( rkj*dij*dkj - dot_product(rij,rkj)*dkj*rij/dij ) 
     &         / (dij**2 * dkj**2)
            fi = kt * fi
            fk = ( rij*dij*dkj - dot_product(rij,rkj)*dij*rkj/dkj )
     $         / (dij**2 * dkj**2)
            fk = kt * fk
            fj = -fi - fk
          else
            fi = kt * (rkj/dkj - (rij/dij)*th) / dij
            fk = kt * (rij/dij - (rkj/dkj)*th) / dkj
            fj = -fi - fk
          endif
        else
          call STOPGM('fm_forces_ang', 'Unknown flag: ' // flag)
        endif  

        return
      end subroutine fm_forces_ang
C
C     ==================================================================
      subroutine fm_forces_imp(fi,fj,fk,fl,ri,rj,rk,rl,it,flag)
C     ==--------------------------------------------------------------==
        implicit none
        include 'forcematch.inc'
C       Arguments
C       forces or derivatives
        real*8  fi(3), fj(3), fk(3), fl(3)
C       coordinates
        real*8  ri(3), rj(3), rk(3), rl(3)
C       improper type
        integer it
C       flag that determines what is calculated:
C       'ff': compute forces
C       'dk': derivative wrt. kt
        character*2     flag
C       Variables
C       distances, distance vectors
        real*8  dij, dkj, dkl, dmj, dnk
        real*8  rij(3), rkj(3), rkl(3), rmj(3), rnk(3)
C       improper angle and its sign
        real*8  zeta, signzeta
C       impropers in GROMOS lingo
        real*8  q, q0, kq

C compute improper angle zeta
        call fm_dist(dij, rij, ri, rj)
        call fm_dist(dkj, rkj, rk, rj)
        call fm_dist(dkl, rkl, rk, rl)
        
        call fm_crossproduct(rmj, rij, rkj)
        call fm_crossproduct(rnk, rkj, rkl)
        dmj = sqrt(dot_product(rmj, rmj))
        dnk = sqrt(dot_product(rnk, rnk))

        signzeta = sign(1.d0, dot_product(rij,rnk))
        zeta = dot_product(rmj, rnk) / (dmj*dnk)
        zeta = signzeta * acos(zeta)
        
        q = zeta
        q0 = fm_q0(it)

        fi = -(q - q0) * rmj * dkj/dmj**2
        fl = +(q - q0) * rnk * dkj/dnk**2
        fj = ( dot_product(rij,rkj)/rkj**2 - 1.d0) * fi
     $     - ( dot_product(rkl,rkj)/rkj**2 ) * fl
        fk = -fi - fj - fl
        if (flag .eq. 'dk') then
          continue ! we're done
        else if (flag .eq. 'ff') then
          kq = fm_kq(it)
          fi = kq * fi
          fj = kq * fj
          fk = kq * fk
          fl = kq * fl
        else
          call STOPGM('fm_forces_imp', 'Unknown flag: ' // flag)
        endif

        return
      end subroutine fm_forces_imp
C
C     ==================================================================
      subroutine fm_forces_dih(fi,fj,fk,fl,ri,rj,rk,rl,dt,flag)
C     ==--------------------------------------------------------------==
        implicit none
        include 'forcematch.inc'
C       Arguments
C       forces or derivatives
        real*8  fi(3), fj(3), fk(3), fl(3)
C       coordinates
        real*8  ri(3), rj(3), rk(3), rl(3)
C       dihedral type
        integer dt
C       flag that determines what is calculated:
C       'ff': compute forces
C       'dk': derivative wrt. kt
        character*2     flag
C       Variables
C       distances, distance vectors
        real*8  dij, dkj, dkl, dim, dln
        real*8  rij(3), rkj(3), rkl(3), rim(3), rln(3), rt(3)
C       dihedral angle and sign, cosine(phi) and dcos(m*phi)/dcos(phi),
C       phase (see GROMOS bible page II-27)
        real*8  phi, signphi, cosphi, cosmphi, dcosmphi, phase
C       multiplicity, force constant
        integer m
        real*8  kp

C compute dihedral angle phi (or rather the cosine of it, that's all we
C need..
        call fm_dist(dij, rij, ri, rj)
        call fm_dist(dkj, rkj, rk, rj)
        call fm_dist(dkl, rkl, rk, rl)
        
        rim = rij - dot_product(rij,rkj) * rkj / dkj**2
        rln = -rkl + dot_product(rkl,rkj) * rkj / dkj**2

        dim = sqrt(dot_product(rim,rim))
        dln = sqrt(dot_product(rln,rln))

        call fm_crossproduct(rt, rkj, rkl)
        signphi = sign(1.d0, dot_product(rij, rt))
        cosphi = dot_product(rim, rln) / (dim*dln)

C now compute forces 
        phase = fm_p0(dt)
        m = int(fm_pn(dt))
        call fm_cosder(dcosmphi, cosmphi, cosphi, m)

        fi = -phase * dcosmphi * ( rln/dln - cosphi*rim/dim) / dim
        fl = -phase * dcosmphi * ( rim/dim - cosphi*rln/dln) / dln
        fj = ( dot_product(rij,rkj) / dkj**2 - 1.d0) * fi
     $     - dot_product(rkl,rkj) / dkj**2 * fl
        fk = -fi - fj - fl

        if (flag .eq. 'dk') then
          continue ! we're done already
        else if (flag .eq. 'ff') then
          kp = fm_kp(dt)
          fi = kp * fi
          fj = kp * fj
          fk = kp * fk
          fl = kp * fl
        else
          call STOPGM('fm_forces_dih', 'Unknown flag: ' // flag)
        endif

        return
      end subroutine fm_forces_dih
C
C     ==================================================================
      subroutine fm_kfit_par_to_fmarray(npar, par)
C     ==--------------------------------------------------------------==
C     Write parameters obtained from Optimizer back into fm_arrays
C     ==================================================================
        implicit none
        include 'forcematch.inc'
C       Arguments
C       nr. of parameters, params themselves
        integer npar
        real*8  par(npar)
C       Variables
C       loop counters, etc.
        integer i, ipar

        ipar = 0
C Bonds
        do i = 1, fm_nbonopt
          ipar = ipar + 1
          fm_kb(i) = par(ipar)
          if (.not.fm_fit_fc_only) then
            ipar = ipar + 1
            fm_b0(i) = par(ipar)
          endif
        enddo
C Angles
        do i = 1, fm_nangopt
          ipar = ipar + 1
          fm_kt(i) = par(ipar)
          if (.not.fm_fit_fc_only) then
            ipar = ipar + 1
            fm_t0(i) = par(ipar)
          endif
        enddo
C Impropers
        do i = 1, fm_nimpopt
          ipar = ipar + 1
          fm_kq(i) = par(ipar)
        enddo
C Dihedrals
        do i = 1, fm_ndihopt
          ipar = ipar + 1
          fm_kp(i) = par(ipar)
        enddo

        if (ipar.ne.npar) call STOPGM('fm_kfit_par_to_fmarray', 
     $                                ' ipar .ne. npar')

        return
      end subroutine fm_kfit_par_to_fmarray
C
C     ==================================================================
      subroutine fm_kfit_writeback(npar, par)
C     ==--------------------------------------------------------------==
C     Write optimized parameters back into GROMOS topology
        implicit none
C       Arguments
C       nr. of parameters and parameters themselves
        integer npar
        real*8  par(npar)
#if defined (__GROMOS)
        include 'gromos.h'
        include 'forcematch.inc'
        include 'fm_cnst.inc'
C       Variables
C       loop counters, temporaries, etc.
        integer i

C Bonds
        do i = 1, fm_nbonopt
          if (fm_amber) then
            fm_kb(i) = fm_kb(i) / (2.d0*fm_b0(i)**2)
          endif
          B0(fm_bonopt(i)) = BOHR_NM * fm_b0(i)
          CB(fm_bonopt(i)) = kb_au2grm * fm_kb(i)
        enddo
C Angles
        do i = 1, fm_nangopt
          T0(fm_angopt(i)) = fm_t0(i)
          if (fm_amber) then
            CT(fm_angopt(i)) = AU_KJM * fm_kt(i) 
     $                       / dsin(dacos(fm_t0(i)))**2
          else
            CT(fm_angopt(i)) = AU_KJM * fm_kt(i)
          endif
        enddo
C Impropers
        do i = 1, fm_nimpopt
          CQ(fm_impopt(i)) = AU_KJM * fm_kq(i)
        enddo
C Dihedrals
        do i = 1, fm_ndihopt
          CP(fm_dihopt(i)) = AU_KJM * fm_kp(i)
        enddo

        return
#endif
      end subroutine fm_kfit_writeback
C
C     ==================================================================
      subroutine fm_dist(r, rij, ri, rj)
C     ==--------------------------------------------------------------==
C     returns the distance r and distance vector rij between
C     two points ri, rj (with rij = ri - rj)
C     ==================================================================
        implicit none
C       Arguments
C       output: distance, and normalized distance vector
        real*8  r, rij(3)
C       input: coordinates of points i, j
        real*8  ri(3), rj(3)

        rij = ri - rj
        r = sqrt(dot_product(rij, rij))
        return
      end subroutine fm_dist
C
C     ==================================================================
      subroutine fm_crossproduct(z, x, y)
C     ==--------------------------------------------------------------==
C     compute the cross product of two (three-dimensional!) vectors x,y
C     ==================================================================
        implicit none
C       Arguments
C       output: cross product z
        real*8  z(3)
C       input: two vectors ri, rj
        real*8  x(3), y(3)

        z(1) = x(2)*y(3) - x(3)*y(2)
        z(2) = x(3)*y(1) - x(1)*y(3)
        z(3) = x(1)*y(2) - x(2)*y(1)
        return
      end subroutine fm_crossproduct
C
C     ==================================================================
      subroutine fm_cosder(dcmp, cmp, cp, m)
C     ==--------------------------------------------------------------==
C     For computing dihedral forces, we need cos(m*phi), and the derivative
C     dcos(m*phi)/dcos(phi), where phi is the dihedral angle and m the
C     multiplicity (see GROMOS bible, page II-27)
C     ==================================================================
        implicit none
C       Arguments
C       output: derivate of cos(m*phi) wrt. cos(phi), and cos(m*phi)
        real*8  dcmp, cmp
C       input: cos(phi), multiplicity
        real*8  cp
        integer m

        if (m.eq.0) then
          cmp = 1.d0
          dcmp = 0.d0
        else if (m.eq.1) then
          cmp = cp
          dcmp = 1.d0
        else if (m.eq.2) then
          cmp = 2.d0*cp**2 - 1.d0
          dcmp = 4.d0 * cp
        else if (m.eq.3) then
          cmp = 4.d0*cp**3 - 3.d0*cp
          dcmp = 12.d0*cp**2 - 3.d0
        else if (m.eq.4) then
          cmp = 8.d0*cp**4 - 8.d0*cp**2 + 1.d0
          dcmp = 32.d0*cp**3 - 16.d0*cp
        else if (m.eq.5) then
          cmp = 16.d0*cp**5 - 20.d0*cp**3 + 5.d0*cp
          dcmp = 80.d0*cp**4 - 60.d0*cp**2 + 5.d0
        else if (m.eq.6) then
          cmp = 32.d0*cp**6 - 48.d0*cp**4 + 18.d0*cp**2 - 1.d0
          dcmp = 192.d0*cp**5 - 192.d0*cp**3 + 36.d0*cp
        else
          write(6,*) "=========== ERROR ! =============="
          write(6,*) " Illegal value for m: ", m
          call STOPGM('fm_cosder', 'Illegal value for multiplicity')
        endif

        return
      end subroutine fm_cosder
C
C     ==================================================================
      subroutine fm_kfit_rms_per_atom(nframes, nr_qm, f_fit_err, 
     $                                f_qm_ref)
C     ==--------------------------------------------------------------==
        implicit none
C       Arguments
        integer nframes, nr_qm
        real*8  f_fit_err(3, nr_qm, nframes)
        real*8  f_qm_ref(3, nr_qm, nframes)
C       Variables
C       rms per atom
        real*8  rms_per_atom(4, nr_qm)
C       length of ref force vector
        real*8  f_ref(nr_qm)
C       loop counters, etc.
        integer iframe, iqm

        write(6,'(A)') '  fm  computing RMS per atom'
C       compute RMS
        call AZZERO(rms_per_atom, 4*nr_qm)
        call AZZERO(f_ref, nr_qm)

        do iframe = 1, nframes
          do iqm = 1, nr_qm
            rms_per_atom(1:3,iqm) = rms_per_atom(1:3,iqm)
     $                            + f_fit_err(:,iqm,iframe)**2
            rms_per_atom(4,iqm) = rms_per_atom(4,iqm) 
     $                          + dot_product(f_fit_err(:,iqm,iframe), 
     $                                        f_fit_err(:,iqm,iframe))
            f_ref(iqm) = f_ref(iqm) 
     $                 + dot_product(f_qm_ref(:, iqm, iframe),
     $                               f_qm_ref(:, iqm, iframe))
          enddo
        enddo
        f_ref = dsqrt(rms_per_atom(4, :) / f_ref)
        rms_per_atom = dsqrt(rms_per_atom / nframes)

C       print out
        write(6,'(A)') '  fm  ========================================='
        write(6,'(A)') '  fm  RMS per atom'
        write(6,'(A)') '  fm  ========================================='
        do iqm = 1, nr_qm
          write(6,222) rms_per_atom(:, iqm), f_ref(iqm)
        enddo
        write(6,'(A)') '  fm  ========================================='

        return

 222    format('  fm  ', 5(e9.3, 1x))
      end subroutine fm_kfit_rms_per_atom
C

