C     ==================================================================
      SUBROUTINE META_EXT_MUL(TAUP,VELP,TSCR,SCR,LSCR,LQUENCH,LMETARES,
     &                        RESETCV,EKINC,EKINP)
C     ==--------------------------------------------------------------==
C     ==  Extended Lagrangean: Collective Variables                   ==
C     ==      L = T_elec + T_ions - E_ks({r_i},{R_I})
C     ==          + T_CV                   : kinetic energy of CV     ==
C     ==          -SUM_icv k_icv(S({R_I},icv)-S0(icv))^2              ==
C                                          : harmonic potential       ==
C                                            that couples the ionic   ==
C                                            DoF to the CV            ==
C     ==          -V_hills(time,S0(icv))   : time dep., given by      == 
C                                            progressive accumulation == 
C                                            of hills, whose shape    == 
C                                            is NCV-dim. Gaussian like== 
C     ==          -V_walls(S0(icv))        : boundary conditions      ==
C     ==                                                              ==
C     ==  This routine calculates the contributions of the harmonic   ==
C     ==  part of potential to be added to  the forces on ions        ==
C     ==                                                              ==
C     ==  It calculates the dynamics of the collective variables      ==
C     ==  that is determined by the kinetic energy                    ==
C     ==  and the added potential terms, and depends on the choice of ==
C     ==  masses and harmonic constants for the CV                    ==
C     ==                                                              ==
C     ==  The path followed by the CV in the CV space is stored in    ==
C     ==  several outfiles, which contain CV position and also        == 
C     ==  informations about the hills filling the PES wells and      ==
C     ==  other deteils regarding the dynamics
C     ==--------------------------------------------------------------==

C     NCOLVAR   : # of collective variables
C     LHILLS    : true when hills are used in the dynamics of the constraints
C     I_META    : # of steps in the meta-dynamics
C     I_META_MAX: max # of steps in the meta-dynamics
C     I_META_RES: if restart from previous meta dynamics, 
C                 # of steps already done
C     I_CVST    : # of steps since last HILL has been added
C     CV_IST    : actual values of collective variables, dim NCOLVAR
C     CV_DYN    : value of the dynamically evolving collective variables 
C     CV_VEL    : velocity of the collective variables
C     CVD_IST   : actual values of diffusion coefficient of CV
C     HLLH      : hills altitude 
C     HLLW      : hills amplitude 
C     CSCL_FAC  : scale factors required for an homogeneous dynamic in the
C                 MCNSTR directions in the space of phases
C                 they can be read from input  and they  can be tuned on the
C                 amplitudes of the variables fluctuations
C     IBOUND    : 1 if a wall limiting the constraint in one direction 
C                 has to be set, otherwise 0 (integer, dimension MCNSTR)
C     VBOUND    : parameters for the definition of constraining potential
C                 (real, dimension 4*NCOLVAR)
C                 if WALL+ VBOUND(2) is the barrier for CV_DYN > VBOUND(1)
C                 if WALL- VBOUND(4) is the barrier for CV_DYN < VBOUND(3)
C     FI_HARM   : force contribution on ions due to the harm. pot. 
C                 dim (NODIM)
C     F_HARM    : force contribution due to the harm. pot.
C                 and acting on the dynamical CV
C                 dim (NCOLVAR)
C     F_HILL    : force contribution due to the time dep. hill potentiel
C                 and acting on the dynamical CV
C                 dim (NCOLVAR)
C     F_WALL    : force contribution due to the boundary potential 
C                 and acting on the dynamical CV
C                 dim (NCOLVAR)
C     META_RESTART: restart for a previous meta-dynamics
C                 acculators and the constraints path have to be read 
C                 from the right files
C                 the output is appended 
C     CV_PATH   : CV history (dim NTOT_ITER*NCOLVAR)
C     CSCL_VAL  : CV scaling factors history (dim NTOT_ITER*NCOLVAR)
C     HLLW_VAL  : hills amplitude (dim NTOT_ITER*NSUBSYS)
C     HLLH_VAL  : hills altitude  (dim NTOT_ITER*NSUBSYS)
C     F_AVER    : accumulator for the calculation of the average force
C                 acting on the CV, as a contribution of the 
C                 underlaying potential 
C     IF_AVER   : counter for the calculation of the average
C     IW_CV     : counter for writing the CV values on the cvmdck_mtd file
C                 in order to monitor the behavior along the MD trajectory
C     DISP_SYS  : norm of the displacment in CV space (dim NSUBSYS)
C     CV_MASS   : masses of collective variables, if not assigned in 
C                 the input file, they are automathically calculated
C                 dim (NCOLVAR)
C     KHARM     : constant of the harmonic part of the potential, if not
C                 assigned in input file, default values are chosen
C                 dim (NCOLVAR)
C     ==--------------------------------------------------------------==

      IMPLICIT NONE
      INCLUDE  'system.h'
      INCLUDE  'ions.inc'
      INCLUDE  'cotr.inc'
      INCLUDE  'cnst_dyn.inc'
      INCLUDE  'irat.inc'
      INCLUDE  'tpar.inc'
      INCLUDE  'ropt.inc'
      INCLUDE  'soft.inc'
      INCLUDE  'store.inc'
      INCLUDE  'adat.inc'
      INCLUDE  'cnst.inc'
      INCLUDE  'fileopen.inc'

C     Arguments
      INTEGER       LSCR
      REAL*8        TAUP(3,NAX,NSX),VELP(3,NAX,NSX),
     &              TSCR(3,NAX,NSX),SCR(LSCR),EKINC,EKINP
      LOGICAL       LQUENCH,LMETARES,RESETCV
C     Variables
      INTEGER       NTOT_ITER
      REAL*8        FI_HARM(NODIM)
      pointer       (IP_FI_HARM,FI_HARM)

      REAL*8        CVD_IST(NCOLVAR),CV_DISP(NCOLVAR),
     &              HC_LAST(NCOLVAR),
     &              F_HARM(NCOLVAR),F_HILL(NCOLVAR),F_WALL(NCOLVAR),
     &              F_AVER(NCOLVAR),HHH,HHHT,
     &              HH_TEST(NCOLVAR),CV_SCL(NCOLVAR),CVD_SCL(NCOLVAR)

      pointer      (IP_CVD_IST,CVD_IST),(IP_CV_DISP,CV_DISP),
     &             (IP_HC_LAST,HC_LAST),
     &             (IP_F_HARM,F_HARM),(IP_F_HILL,F_HILL),
     &             (IP_F_WALL,F_WALL),(IP_F_AVER,F_AVER),
     &             (IP_HH_TEST,HH_TEST),(IP_CV_SCL,CV_SCL),
     &             (IP_CVD_SCL,CVD_SCL)
      SAVE          IP_CVD_IST,IP_CV_SCL,IP_CVD_SCL 
      DIMENSION     DOF_INCV(NODIM,NCOLVAR,18),
     &              DET_COLVAR(NODIM,NCOLVAR)
      DIMENSION     CV_PATH(I_META_MAX+I_META_RES,NCOLVAR),
     &              CSCL_VAL(I_META_RES+I_META_MAX,NCOLVAR),
     &              HLLW_VAL(I_META_RES+I_META_MAX,NSUBSYS),
     &              HLLH_VAL(I_META_RES+I_META_MAX,NSUBSYS)
      DIMENSION     VBOUND(4,NCOLVAR),CSCL_FAC(3,NCOLVAR),
     &             IBOUND(NCOLVAR)

      INTEGER       IFIRST,I_META,I_CVST,I_TEMP,IF_AVER,IW_CV
      DATA          IFIRST,I_META,I_CVST,I_TEMP,IF_AVER,IW_CV / 6*0 /
      SAVE          IFIRST,I_META,I_CVST,I_TEMP,IF_AVER,IW_CV
      INTEGER       IFIRSTP
      DATA          IFIRSTP / 0 /
      SAVE          IFIRSTP


      CHARACTER*100 OUTCHECK(10)
      CHARACTER*10  FILE9
      DATA          FILE9 /'cvmdck_mtd'/

      INTEGER       IA,IS,I,IE,IFILE
      CHARACTER*20  FLAG

      LOGICAL       HILL_ADD
      LOGICAL       LSKIP, FINAL
      CHARACTER     TAG,LINEFORM*100,CHNUM*10

      SAVE          IP_HC_LAST,IP_F_AVER
      SAVE          IP_F_HARM,IP_F_HILL,IP_F_WALL

      REAL*8        TOLLM,MAXF2,FACT,FACT2,DIF_CV,
     &              HH_MIN,HH_MAX,HH,HLLH0(10),DISPLACEMENT,
     &              DISP_SYS(NSUBSYS),GPOT_SYS(NSUBSYS)
      POINTER       (IP_DISP_SYS,DISP_SYS),(IP_GPOT_SYS,GPOT_SYS)
      INTEGER       ISYS(NCOLVAR)
      POINTER       (IP_ISYS,ISYS)
      INTEGER       ISUB,II,ICV,IJ,JJ,LENGTH,ISC2,
     &              LSCR2,IP,IDOF,msglen
      INTEGER       L_CV_DISP,L_F_CV,L_FI_HARM,
     &              L_CV_SCL,L_CVD_SCL,L_SCR_HIL,L_SUBSYS

      LOGICAL       FERROR
      REAL*8        TEMP1,TEMP2,TEMPP
      DATA          TEMP1,TEMP2 /0.0D0,0.0D0/
      SAVE          TEMP1,TEMP2,TEMPP
C     ==--------------------------------------------------------------==
C     ==--------------------------------------------------------------==
C     Set time for the routine
      CALL TISET(' META_EXTL',ISUB)
      FINAL = .FALSE.

      IF(NSUBSYS .GT. 10) CALL STOPGM(' META_EXTL', 
     &         'Too Many Subsystems for Multi-Metadynamics, max is 10')
C     ==--------------------------------------------------------------==
      CALL AZZERO(SCR,LSCR)
      IF(IFIRSTP .EQ. 0) THEN
        I_META  = I_META_RES+1
        I_TEMP  = 0
        I_CVST  = 0
        IF_AVER = 0

C     Allocate Memory

        CALL MEMORY(IP_CV_DYN,NCOLVAR,'CV_DYN')
        CV_ASSOCIATED = .true.
        IFIRSTP = 1
      ENDIF
      IF(.NOT.PARENT) GOTO 9999

C     ==--------------------------------------------------------------==
C     Path of output files
      DO IS = 1,NSUBSYS
        WRITE(FLAG,'(I5)') IS
        CALL XSTRING(FLAG,IA,IE)
        OUTCHECK(IS)   = FILE9//'_S'//FLAG(IA:IE)
      ENDDO

C     ==--------------------------------------------------------------==
C     Temporary Array

#ifdef _BGL 
      L_FI_HARM =  NODIM+MOD(NODIM,2)
      L_CV_DISP = NCOLVAR+MOD(NCOLVAR,2)
      L_F_CV    = NCOLVAR+MOD(NCOLVAR,2)
      L_CV_SCL  = NCOLVAR+MOD(NCOLVAR,2)
      L_CVD_SCL = NCOLVAR+MOD(NCOLVAR,2)
      L_SCR_HIL = 2*NCOLVAR*NCOLVAR+NODIM+NODIM*NCOLVAR
      L_SUBSYS  = 2*NSUBSYS+NCOLVAR+MOD(2*NSUBSYS+NCOLVAR,2)
#else
      L_FI_HARM =  NODIM
      L_CV_DISP = NCOLVAR
      L_F_CV    = NCOLVAR
      L_CV_SCL  = NCOLVAR
      L_CVD_SCL = NCOLVAR
      L_SCR_HIL = 2*NCOLVAR*NCOLVAR+NODIM+NODIM*NCOLVAR
      L_SUBSYS  = 2*NSUBSYS+NCOLVAR
#endif

      LENGTH  = L_FI_HARM+L_CV_DISP+L_F_CV+L_CV_SCL+
     &          L_CVD_SCL+L_SCR_HIL+L_SUBSYS

      CALL TEST_SCR('META_EXTLAGR',TAG,LSCR,LENGTH)

      CALL  GET_ADDR(IP_FI_HARM ,SCR(1))
      CALL  GET_ADDR(IP_CV_DISP ,SCR(1+L_FI_HARM))
      CALL  GET_ADDR(IP_HH_TEST ,SCR(1+L_FI_HARM+L_CV_DISP))
      CALL  GET_ADDR(IP_CV_SCL  ,SCR(1+L_FI_HARM+L_CV_DISP+
     &                    L_F_CV))
      CALL  GET_ADDR(IP_CVD_SCL ,SCR(1+L_FI_HARM+L_CV_DISP+
     &                    L_F_CV+L_CV_SCL))
      CALL  GET_ADDR(IP_DISP_SYS,SCR(1+L_FI_HARM+L_CV_DISP+
     &                    L_F_CV+L_CV_SCL+L_CVD_SCL))
      CALL  GET_ADDR(IP_GPOT_SYS,SCR(1+L_FI_HARM+L_CV_DISP+
     &                    L_F_CV+L_CV_SCL+L_CVD_SCL+NSUBSYS))
      CALL  GET_ADDR(IP_ISYS,SCR(1+L_FI_HARM+L_CV_DISP+
     &            L_F_CV+L_CV_SCL+L_CVD_SCL+NSUBSYS+NSUBSYS))
      ISC2         = 1+L_FI_HARM+L_CV_DISP+
     &                    L_F_CV+L_CV_SCL+L_CVD_SCL+L_SUBSYS
      LSCR2        = LSCR - L_FI_HARM+L_CV_DISP+
     &                    L_F_CV+L_CV_SCL+L_CVD_SCL+L_SUBSYS

      NTOT_ITER = I_META_MAX+I_META_RES
      TOLLM     = TOLL_AVCV


      IF(IFIRST .EQ. 0) THEN
C     Allocate Memory

        CALL DCOPY(NCOLVAR,CV_IST,1,CV_DYN,1)
        CALL MEMORY(IP_CV_VEL,NCOLVAR,'CV_VEL')
        CALL AZZERO(CV_VEL,NCOLVAR)
        CALL MEMORY(IP_CVD_IST,NCOLVAR,'CVD_IST')
        CALL AZZERO(CVD_IST,NCOLVAR)
        CALL MEMORY(IP_HC_LAST,NCOLVAR,'HC_LAST')
        CALL AZZERO(HC_LAST,NCOLVAR)
        CALL MEMORY(IP_F_AVER,NCOLVAR,'F_AVER')
        CALL AZZERO(F_AVER,NCOLVAR)
        CALL MEMORY(IP_F_HARM,NCOLVAR,'F_HARM')
        CALL AZZERO(F_HARM,NCOLVAR)
        CALL MEMORY(IP_F_HILL,NCOLVAR,'F_HILL')
        CALL AZZERO(F_HILL,NCOLVAR)
        CALL MEMORY(IP_F_WALL,NCOLVAR,'F_WALL')
        CALL AZZERO(F_WALL,NCOLVAR)

        CALL MEMORY(IP_CV_PATH,NCOLVAR*NTOT_ITER,'CV_PATH')
        CALL AZZERO(CV_PATH,NCOLVAR*NTOT_ITER)
        CALL MEMORY(IP_CSCL_VAL,NCOLVAR*NTOT_ITER,'CSCL_VAL')
        CALL AZZERO(CSCL_VAL,NCOLVAR*NTOT_ITER)
        CALL MEMORY(IP_HLLW_VAL,NTOT_ITER*NSUBSYS,'HLLH_VAL')
        CALL AZZERO(HLLW_VAL,NTOT_ITER*NSUBSYS)
        CALL MEMORY(IP_HLLH_VAL,NTOT_ITER*NSUBSYS,'HLLH_VAL')
        CALL AZZERO(HLLH_VAL,NTOT_ITER*NSUBSYS)


        JJ = 0
        DO IS=1,NSUBSYS
          HLLH0(IS) = HHM(IS)
          DO ICV = 1,NCVSYS(I)
            ISYS(JJ+ICV) = IS
          ENDDO
          JJ = JJ + NCVSYS(I)
        ENDDO

C     If META_RESTART read data from proper input
        IF(META_RESTART  .AND. .NOT. TCVANALYSIS) THEN
          IF(TRESFILE) THEN
            CALL RMTDRESM(50,CV_DISP,.TRUE.,NTOT_ITER)
          ELSE
            CALL CV_READ_OUTM(CV_DISP)
          ENDIF  ! TRESFILE
!$OMP parallel do private(ICV)
          DO ICV = 1,NCOLVAR
            HC_LAST(ICV) = CV_PATH(I_META_RES,ICV)+CV_DISP(ICV)
          ENDDO
          CALL DCOPY(NCOLVAR,HC_LAST,1,CV_DYN,1)
        ENDIF   !META_RESTART

        IF(.NOT. TCVANALYSIS) THEN
C     Masses
          IF(CV_TEMP0 .NE. 0.0D0) THEN
            FACT = CV_TEMP0*KBOLTZ*10.D0
          ELSE
            FACT = TEMPW   *KBOLTZ*10.D0
          ENDIF
          FACT2 = FACT * DELT_IONS*DBLE(INTER_HILL)* 
     &            DELT_IONS*DBLE(INTER_HILL)
          DO ICV = 1,NCOLVAR
            IF(CV_MASS(ICV) .EQ. 0.0D0) THEN
              CV_MASS(ICV) = FACT2/
     &                 ((CSCL_FAC(1,ICV)*HWM(ISYS(ICV)))*
     &                  (CSCL_FAC(1,ICV)*HWM(ISYS(ICV))))
              WRITE(6,'(A,I5,A,f15.4,A)') 'Default Mass for CV #',ICV,
     &                        ' is assigned  = ',
     &                   CV_MASS(ICV)/SCMASS,' a.m.u.(a.u.)^2/(u.CV)^2'
              CV_MASS(ICV) = CV_MASS(ICV)*SCMASS
            ELSE
              WRITE(6,'(A,I5,A,f15.4,A)') 'Mass for CV #',ICV,
     &                        ' is assigned  = ',
     &                   CV_MASS(ICV) ,' a.m.u.(a.u.)^2/(u.CV)^2'
              CV_MASS(ICV) =  SCMASS*CV_MASS(ICV)
            ENDIF

C     Harmonic Potential
            IF(KHARM(ICV) .EQ. 0.0D0) THEN
              KHARM(ICV) = FACT/
     &                 ((CSCL_FAC(1,ICV)*HLLW)*(CSCL_FAC(1,ICV)*HLLW))
              WRITE(6,'(A,I5,A,f10.4,A)') 
     &                     'Default harmonic constant for CV #',ICV,
     &                     ' is assigned ',KHARM(ICV),' a.u./(u.CV)^2'
            ENDIF
            IF(.NOT. LKFIX) LCHEKHARM = .TRUE.
          ENDDO

C     Initialization of CV Velocity
          IF(I_META .EQ. 1) THEN
            CALL RINVELCV(CV_VEL,CV_MASS,CSCL_FAC,NCOLVAR,
     &                    CV_TEMP0,EKINCV)
            WRITE(6,'(2x,A,f10.4)') 
     &           'CV velocity have been initialized at T = ',CV_TEMP0
            WRITE(6,'(2x,A,f10.4)') 
     &           'the initial CV kinetic energy is = ',EKINCV
          ENDIF

          IF(LCVTC)THEN
            TEMP1=CV_TEMP+CV_DTEMP
            TEMP2=CV_TEMP-CV_DTEMP
          ENDIF
          TEMPP = CV_TEMP0
        ELSE
          CALL AZZERO(CV_VEL,NCOLVAR)
        ENDIF
C       ==--------------------------------------------------------------==
 
        IF(TCVMONITOR) THEN
          DO IS = 1, NSUBSYS
            IFILE = 444+IS
            CALL FILEOPEN(IFILE,OUTCHECK(IS),FO_APP+FO_VERB,FERROR)
          ENDDO
        ENDIF
      ENDIF

C     ==--------------------------------------------------------------==
C     Calculate the Collective Variables and their Derivatives
      CALL COLVAROFR(TAUP,TSCR)

      IF(RESETCV .AND. TLOCALIZESPIN) THEN
        DO ICV = 1,NCOLVAR
          IF(ICV_SPIN(ICV) .NE. 0) THEN
            CV_DYN(ICV) = CV_IST(ICV)
          ENDIF 
        ENDDO
        RESETCV = .FALSE.
      ENDIF
C     ==--------------------------------------------------------------==
C     Initialization
      
      IF(IFIRST.EQ.0) THEN
        IF(I_META.EQ.1)THEN
          DO ICV = 1,NCOLVAR
            HC_LAST(ICV) = CV_IST(ICV)
          ENDDO
        ENDIF
        IFIRST=1
        IF(.NOT. META_RESTART .AND. TLOCALIZESPIN) THEN
          CALL DCOPY(NCOLVAR,CV_IST,1,CV_DYN,1)
        ENDIF
        GOTO 200
      ENDIF

      HILL_ADD = .FALSE.
      IF(TTRAJOVRR) NTRAJ=999999999

      I_CVST = I_CVST + 1
      IW_CV=IW_CV+1


!     ==--------------------------------------------------------------==
!     Check  CV_DYN wrt the center of the last Hill (HC_LAST)
!     to be verified conditions: 
!                               minimum time interval 
!                               minimum displacement in CV space
!     If the conditions are sadisfied HILL_ADD = true 
!     and a new hill will be added
      IP = ISC2
      CALL CALC_POS_MDYN(TOLLM,I_CVST,I_TEMP,I_META,HC_LAST,
     &             SCR(IP),LSCR2,HILL_ADD)
      IF(LMETARES) THEN
        FINAL = .TRUE.
        GOTO 9999
      END IF
!     ==--------------------------------------------------------------==
!     Tune the Harmonic Potential according to the PES

      IF(LCHEKHARM .AND. HILL_ADD) THEN
        CALL CV_FORCES3(TAUP,TSCR,VELP,HH_TEST,SCR(ISC2),LSCR2)
        DO ICV = 1,NCOLVAR
          DIF_CV = CV_IST(ICV)-CV_DYN(ICV)
          IF (IANGCV(ICV) .EQ. 1 .AND. DIF_CV .GT. PI) THEN
            DIF_CV = DIF_CV -2.0D0*PI
          ELSEIF (IANGCV(ICV) .EQ. 1 .AND. DIF_CV .LT. -PI) THEN
            DIF_CV = DIF_CV +2.0D0*PI
          ENDIF
          IF(HH_TEST(ICV).LE.1.D+1 .AND. HH_TEST(ICV).GT.0.2D0) THEN
            FACT = DABS(DIF_CV)/CSCL_FAC(1,ICV)
            IF(FACT.LT. HLLW) THEN
              KHARM(ICV) = DABS(HH_TEST(ICV)/2.D0/HWM(ISYS(ICV)))
            ELSE
              KHARM(ICV) = DABS(HH_TEST(ICV)/2.D0/DIF_CV)
            ENDIF
          ELSE
          ENDIF
        ENDDO
      ENDIF

!     ==--------------------------------------------------------------==
!     ==--------------------------------------------------------------==
!     Calculation of the forces on the collective variables
!     This is used in order to determine the optimal hills parameters 
!     In F_aver the forces on CV are accumulated in order to calculate
!     their average value when the meta step is concluded
!  MULTY
      IF(LTUNE_HH) THEN
        DO ICV = 1,NCOLVAR
          F_AVER(ICV) = F_AVER(ICV) 
c     &        + 2.0D0*KHARM(ICV)*DIF_CV
     &        + F_HARM(ICV)+F_HILL(ICV)/CSCL_FAC(1,ICV)
        ENDDO
        IF_AVER = IF_AVER + 1
      ENDIF
!     ==--------------------------------------------------------------==
!     Monitoring of the CV along the MD trajectory. Even if no hill is added
!     at this point, if IW_CV is a multiple of WCV_FREQ, 
!     the values of the CV_IST and of the CV_DYN
!     are written in the cvmdck_mtd file
!     in the first column the IW_CV counter is reported, that indicates the
!     number of MD steps of the actual run, in the second column the global 
!     metastep is reported, afterwards the values of the two CV sets
      IF(TCVMONITOR .AND. MOD(IW_CV,WCV_FREQ) .EQ. 0) THEN
        DO IS = 1,NSUBSYS
          IFILE = 444 + IS
          WRITE(CHNUM,'(I5)') NCVSYS(IS)
          CALL XSTRING(CHNUM,IA,IE)
          LINEFORM =
     &               '(1X,2I7,'//CHNUM(IA:IE)//
     &               'E14.6,'//CHNUM(IA:IE)//'E14.6,E14.6)'
          write(IFILE,LINEFORM) IW_CV,NFI,
     &               (CV_IST(ICV),ICV=1,NCVSYS(IS)),
     &               (CV_DYN(ICV),ICV=1,NCVSYS(IS)),TEMPP
          call my_flush(IFILE)
        ENDDO
      ENDIF

!     ==--------------------------------------------------------------==
!     New Step of Meta-Dynamics in the Space of CV
      IF(HILL_ADD) THEN
        IF(MOD(I_META,TR_FREQ) .EQ. 0 .OR. I_META .EQ. 1 ) THEN
          IF(TTRAJOVRR) NTRAJ=1
        ENDIF

        IF(.NOT. TCVANALYSIS) THEN
C       ==--------------------------------------------------------------==
C       Squared Norm of the Displacement in the CV space


C       MULTY
        JJ = 0
        DO IS = 1,NSUBSYS
          DISP_SYS(IS) = 0.0D0
          DO ICV = 1,NCVSYS(IS)
            DIF_CV = CV_DYN(JJ+ICV)-HC_LAST(JJ+ICV)
            IF (IANGCV(JJ+ICV) .EQ. 1 .AND. DIF_CV .GT. PI) THEN
              DIF_CV = DIF_CV - 2.0D0*PI
            ELSEIF (IANGCV(JJ+ICV) .EQ. 1 .AND. DIF_CV .LT. -PI) THEN
              DIF_CV = DIF_CV + 2.0D0*PI
            ENDIF
            FACT = DIF_CV/CSCL_FAC(1,JJ+ICV)
            DISP_SYS(IS) = DISP_SYS(IS) + FACT*FACT
          ENDDO
          DISP_SYS(IS) = DSQRT(DISP_SYS(IS))
          JJ = JJ + NCVSYS(IS)
        ENDDO

C       ==--------------------------------------------------------------==
C       Tuning Hills parameters 

        IF(LTUNE_HH) THEN
C MULTY
          JJ = 0
          DO IS = 1,NSUBSYS
            HH_MIN = 1000.D0
            HH_MAX = 0.0D0
            LSKIP = .FALSE.

            HHHT = 0.0D0
            DO ICV = 1,NCVSYS(IS)
              F_AVER(JJ+ICV)  = F_AVER(JJ+ICV) / DBLE(IF_AVER)
              F_AVER(JJ+ICV)  = F_AVER(JJ+ICV) 
              DIF_CV = CV_DYN(JJ+ICV)-HC_LAST(JJ+ICV)
              IF (IANGCV(JJ+ICV) .EQ. 1 .AND. DIF_CV .GT. PI) THEN
                DIF_CV = DIF_CV - 2.0D0*PI
              ELSEIF (IANGCV(JJ+ICV) .EQ. 1 .AND. DIF_CV .LT. -PI) THEN
                DIF_CV = DIF_CV + 2.0D0*PI
              ENDIF
              FACT = DIF_CV/(CSCL_FAC(1,JJ+ICV)*CSCL_FAC(1,JJ+ICV))*
     &               (1.0D0/(HWM(IS)*HWM(IS))
     &               +1.0D0/(DISP_SYS(IS)*DISP_SYS(IS)))
              HHH  = 0.5D0*dabs(F_AVER(JJ+ICV)*DEXP(0.5D0)/FACT)
              HHHT = HHHT + HHH

              WRITE(6,'(2x,A,I4,3F14.6)') 'FCV ',JJ+ICV,
     &            F_AVER(JJ+ICV),HHH,FACT

              HH_MIN       = MIN(HH_MIN,HHH)
              HH_MAX       = MAX(HH_MAX,HHH)
            ENDDO
            HH       = HHHT / DBLE(NCVSYS(IS))
            IF(HH_MIN .GT. HTHM(IS) .OR. HH_MAX .LT. HLHM(IS)) THEN
              LSKIP = .TRUE.
              WRITE(6,'(A,I4,A,f10.5,A,f10.5,A)') 'Set CV', IS, 
     &                 '  HM min > ', HTHM(IS)*AU_KCM,
     &                 ' Kcal, or HH max < ',HLHM(IS)*AU_KCM,' Kcal'
            ENDIF
            IF(.NOT. LSKIP .AND. HH .LT. HLHM(IS)) THEN
              HHM(IS) = (HLHM(IS)+HH_MAX)*0.5D0
            ELSEIF(.NOT. LSKIP .AND. HH .GT. HTOP) THEN
              HHM(IS) = (HH_MIN+HTHM(IS))*0.5D0
            ELSEIF(.NOT. LSKIP ) THEN
              HHM(IS) = HH
            ELSE
              HHM(IS) = HHM(IS)*0.2D0+HLLH0(IS)*0.8D0
            ENDIF
            WRITE(6,'(/A,I4,3(2x,A,f10.5)/)') 'Set CV', IS,
     &              'HH min = ', HH_MIN,'HH max = ', HH_MAX,
     &              ' HH used ',HHM(IS)
            JJ = JJ + NCVSYS(IS)
            IF(THILLVOL) THEN
              HWM(I)=(HVM0(I)/HHM(I))**(1.D0/DBLE(NCVSYS(I)))
            ENDIF
          ENDDO

          IF_AVER = 0
          CALL AZZERO(F_AVER,NCOLVAR)
        ENDIF

C       ==--------------------------------------------------------------==
C       Scaling Factors Tuning
        IF(LTUNE_CSCL) THEN
  
          CALL SCF_TUNE(I_META,HC_LAST)

        ENDIF
C       ==--------------------------------------------------------------==
C       Diffusion calculation
        CALL CV_DIFF_MD(HC_LAST,CVD_IST,CVD_SCL,CV_SCL,I_META,
     &            SCR(ISC2),LSCR2,NTOT_ITER)

C       ==--------------------------------------------------------------==

C       Open output file
        CALL CV_EXL_OUTM(I_META,HC_LAST,F_HARM,F_HILL,F_WALL,
     &           CVD_IST,DISP_SYS,EKINC,EKINP)

C       ==--------------------------------------------------------------==

        DO ICV = 1,NCOLVAR
          HC_LAST(ICV)   = CV_DYN(ICV)
        ENDDO
        ELSE
!         ==--------------------------------------------------------------==
!         Open output file
          CALL CV_EXL_OUTM2(I_META,EKINC,EKINP)

!         ==--------------------------------------------------------------==

        ENDIF

        I_META = I_META + 1
        I_TEMP = 0
        I_CVST = 0
      ENDIF        !HILL_ADD

!     ==--------------------------------------------------------------==
!     ==--------------------------------------------------------------==
!     Calculation of the dynamics of the collective variables
!     A velocity Verlet Scheme is used

!     ==--------------------------------------------------------------==
!     Update Velocity : first half step
!                       using forces calculated in the previous step
      DO ICV = 1,NCOLVAR
        FACT = DT_IONS/(2.D0*CV_MASS(ICV))
        CV_VEL(ICV) = CV_VEL(ICV) 
     &      + FACT*(F_HARM(ICV)+F_HILL(ICV)/CSCL_FAC(1,ICV)+F_WALL(ICV))
      ENDDO

!     ==--------------------------------------------------------------==
!     Update Position : second order dynamics

      DO ICV = 1,NCOLVAR
        FACT = DT_IONS
        IF (IANGCV(ICV) .EQ. 1 )  THEN
          IF(CV_DYN(ICV).LT.0.D0) CV_DYN(ICV)= CV_DYN(ICV)+2.D0*PI
          IF(CV_DYN(ICV).GT.2.D0*PI) CV_DYN(ICV)= CV_DYN(ICV)-2.D0*PI
        ELSEIF(IANGCV(ICV) .EQ. 2 )  THEN
          IF(CV_DYN(ICV).LT.0.D0) CV_DYN(ICV)= -CV_DYN(ICV)
          IF(CV_DYN(ICV).GT. PI) CV_DYN(ICV)= PI - (CV_DYN(ICV) - PI)
        ENDIF
        CV_DYN(ICV) = CV_DYN(ICV) + FACT*CV_VEL(ICV)
      ENDDO

 200  CONTINUE

C     ==--------------------------------------------------------------==
C     Calculation of the contributions to the forces on ions 
C     due to the coupling with the collective variables
C     V_ARM = SUM_icv (K_icv(CV_IST_icv - CV_DYN_icv))^2
C
      CALL AZZERO(FI_HARM,NODIM)
      DO ICV = 1,NCOLVAR
        DIF_CV = CV_IST(ICV)-CV_DYN(ICV)
        IF (IANGCV(ICV) .EQ. 1 .AND. DIF_CV .GT. PI) THEN
          DIF_CV = DIF_CV -2.0D0*PI
        ELSEIF (IANGCV(ICV) .EQ. 1 .AND. DIF_CV .LT. -PI) THEN
         DIF_CV = DIF_CV +2.0D0*PI
        ENDIF
        FACT = KHARM(ICV)*2.0D0*DIF_CV
        DO IDOF = 1,NODIM
          FI_HARM(IDOF) = FI_HARM(IDOF) - 
     &                    FACT * DET_COLVAR(IDOF,ICV)
        ENDDO
      ENDDO

C     ==--------------------------------------------------------------==
C     Update forces on ions

      CALL AZZERO(TSCR,3*NAX*NSX)
      CALL GETTAU(TSCR,FI_HARM)
#ifdef __SR11000
*poption parallel, tlocal(IS,IA,FACT)
*voption indep(FHILLS,TSCR)
#endif
      DO IS=1,NSP
        FACT= DT_IONS*DTB2MI(IS)
        DO IA=1,NA(IS)
           FHILLS(1,IA,IS) = TSCR(1,IA,IS)
           FHILLS(2,IA,IS) = TSCR(2,IA,IS)
           FHILLS(3,IA,IS) = TSCR(3,IA,IS)
        ENDDO
      ENDDO

C     ==--------------------------------------------------------------==
C     Calculate contribution of the Harmonic Potential (recall)

      CALL AZZERO(F_HARM,NCOLVAR)
      DO ICV = 1,NCOLVAR
        DIF_CV = CV_IST(ICV)-CV_DYN(ICV)
        IF (IANGCV(ICV) .EQ. 1 .AND. DIF_CV .GT. PI) THEN
          DIF_CV = DIF_CV - 2.0D0*PI
        ELSEIF (IANGCV(ICV) .EQ. 1 .AND. DIF_CV .LT. -PI) THEN
          DIF_CV = DIF_CV + 2.0D0*PI
        ENDIF
        F_HARM(ICV) =  KHARM(ICV)*2.0D0*DIF_CV
      ENDDO

C     Calculate hills contribution to forces
      IF(LHILLS) THEN
C MULTY
        JJ = 0
        DO IS = 1,NSUBSYS
          CALL AZZERO(F_HILL(JJ+1),NCVSYS(IS))
          IF(HLORE) THEN
            CALL HILLS_LOR(CV_DYN(JJ+1),NCVSYS(IS),I_META,
     &                   NTOT_ITER,F_HILL(JJ+1),
     &                   SCR(ISC2),LSCR2,IS,JJ)
          ELSEIF(HRATIO) THEN
            CALL HILLS_RATIO(CV_DYN(JJ+1),NCVSYS(IS),I_META,
     &                   NTOT_ITER,F_HILL(JJ+1),HC_LAST(JJ+1),
     &                   SCR(ISC2),LSCR2,I_CVST,IS,JJ)
          ELSEIF(HSHIFT) THEN
            CALL HILLS_SALS_SHIFT(CV_DYN(JJ+1),NCVSYS(IS),I_META,
     &                   NTOT_ITER,F_HILL(JJ+1),HC_LAST(JJ+1),
     &                   SCR(ISC2),LSCR2,I_CVST,IS,JJ)
          ELSE IF(SPHERE) THEN
            CALL HILLS(CV_DYN,NCVSYS(IS),I_META,NTOT_ITER,F_HILL(JJ+1),
     &                   SCR(ISC2),LSCR2,IS,JJ)
          ELSE
            CALL HILLS_SALS(CV_DYN(JJ+1),NCVSYS(IS),I_META,
     &                   NTOT_ITER,F_HILL(JJ+1),HC_LAST(JJ+1),
     &                   SCR(ISC2),LSCR2,I_CVST,IS,JJ)
          ENDIF
          GPOT_SYS(IS) = GAUSSPOT
          JJ = JJ + NCVSYS(IS)
        ENDDO
        GAUSSPOT = 0.D0
        DO IS = 1,NSUBSYS
          GAUSSPOT = GAUSSPOT + GPOT_SYS(IS)
        ENDDO
      ENDIF

C     Calculate walls contribution to forces
      CALL AZZERO(F_WALL,NCOLVAR)
      MAXF2 = 0.0D0
      DO ICV = 1,NCOLVAR
        IF(IBOUND(ICV) .EQ. 1) THEN
          CALL SETWALL(CV_DYN(ICV),VBOUND(1,ICV),F_WALL(ICV))
        ELSEIF(IBOUND(ICV) .EQ. 2) THEN
          CALL SETWALL_OLD(CV_DYN(ICV),TYCVAR(ICV),VBOUND(1,ICV),
     &                     F_WALL(ICV))
        ENDIF
        IF(F_WALL(ICV) .NE. 0.0D0) THEN
          write(6,'(72A)') ('*',i = 1,72) 
          write(6,'(10x,A,I5)') 
     &           'WARNING: Boundaries active for CV ', ICV
          write(6,*)  ICV, F_WALL(ICV)
          write(6,'(72A)')  ('*',i = 1,72)
       ENDIF
      ENDDO

C     ==--------------------------------------------------------------==
C     Update Velocity : second half step 
      DO ICV = 1,NCOLVAR
        FACT = DT_IONS/(2.D0*CV_MASS(ICV))
        CV_VEL(ICV) = CV_VEL(ICV) 
     &    + FACT*(F_HARM(ICV)+F_HILL(ICV)/CSCL_FAC(1,ICV)+F_WALL(ICV))
      ENDDO

C     ==--------------------------------------------------------------==
C     Calculation of   Temperature of CV: CV_TEMP
C     If CV_TEMP_CONTROL, check the condition for rescaling T

      CALL RSCVELCV(TEMP1,TEMP2,TEMPP)

C     ==--------------------------------------------------------------==
C     ==--------------------------------------------------------------==
C     IF MAX_STEP reached or EXIT file exist

      IF(INFI.EQ.NOMORE .OR. I_META-1.EQ.NTOT_ITER) THEN
        EXSOFT = .TRUE.

        CALL  COLVARPR
        IF(TCVMONITOR) THEN
          DO IS = 1,NSUBSYS
            IFILE = 444+IS
            CALL FILECLOSE(IFILE)
          ENDDO
        ENDIF

      ENDIF
 9999 CONTINUE
      CALL MY_SYNC(ALLGRP)

      MSGLEN = NCOLVAR * 8
      CALL MY_BCAST(CV_DYN,MSGLEN,SOURCE,ALLGRP)

      MSGLEN = 1 * 8/IRAT
      CALL MY_BCAST(EXSOFT,MSGLEN,SOURCE,ALLGRP)

      MSGLEN = 1 * 8/IRAT
      CALL MY_BCAST(I_META ,MSGLEN,SOURCE,ALLGRP)

      MSGLEN = 1 * 8/IRAT
      CALL MY_BCAST(I_CVST ,MSGLEN,SOURCE,ALLGRP)

      IF(TLOCALIZESPIN) THEN
        MSGLEN = NCOLVAR*3 *8
        CALL MY_BCAST(RCC0,MSGLEN,SOURCE,ALLGRP)
      ENDIF


C     WRITE RESTART
      IF((MOD(I_META,ST_FREQ) .EQ. 0  .AND. I_CVST .EQ. 0)
     &                        .OR. EXSOFT .OR. LMETARES) THEN
        LMETARES = .TRUE.
        IF(I_META .GT. 1) THEN
           CALL WMTDRESM(50,NTOT_ITER,I_META)
        ENDIF

      ENDIF

C     QUENCH BO
      IF(.NOT.TMDBO.AND.
     &    MOD(I_META,QW_FREQ) .EQ. 0  .AND. I_CVST .EQ. 0) 
     &    LQUENCH = .TRUE.

      IF(FINAL .AND. PARENT ) THEN
C     Deallocate
        CALL FREEM(IP_HC_LAST)
        CALL FREEM(IP_CV_PATH)
        CALL FREEM(IP_CSCL_VAL)
        CALL FREEM(IP_HLLW_VAL)
        CALL FREEM(IP_HLLH_VAL)
        CALL FREEM(IP_CV_DYN)
        CALL FREEM(IP_CV_VEL)
        CALL FREEM(IP_CVD_IST)

        CALL FREEM(IP_F_AVER)
        CALL FREEM(IP_F_HARM)
        CALL FREEM(IP_F_HILL)
        CALL FREEM(IP_F_WALL)

      ENDIF
C     ==--------------------------------------------------------------==
C     ==--------------------------------------------------------------==
C     Halt time for the routine
      CALL TIHALT(' META_EXTL',ISUB)

      RETURN
      END
C     ==================================================================




