C     ==================================================================
      SUBROUTINE META_EXTLAGR(TAUP,VELP,TSCR,SCR,LSCR,LQUENCH,LMETARES,
     &                        RESETCV,EKINC,EKINP)
C     ==--------------------------------------------------------------==
C     ==  Extended Lagrangean: Collective Variables                   ==
C     ==      L = T_elec + T_ions - E_ks({r_i},{R_I})
C     ==          + T_CV                   : kinetic energy of CV     ==
C     ==          -SUM_icv k_icv(S({R_I},icv)-S0(icv))^2              ==
C                                          : harmonic potential       ==
C                                            that couples the ionic   ==
C                                            DoF to the CV            ==
C     ==          -V_hills(time,S0(icv))   : time dep., given by      == 
C                                            progressive accumulation == 
C                                            of hills, whose shape    == 
C                                            is NCV-dim. Gaussian like== 
C     ==          -V_walls(S0(icv))        : boundary conditions      ==
C     ==                                                              ==
C     ==  This routine calculates the contributions of the harmonic   ==
C     ==  part of potential to be added to  the forces on ions        ==
C     ==                                                              ==
C     ==  It calculates the dynamics of the collective variables      ==
C     ==  that is determined by the kinetic energy                    ==
C     ==  and the added potential terms, and depends on the choice of ==
C     ==  masses and harmonic constants for the CV                    ==
C     ==                                                              ==
C     ==  The path followed by the CV in the CV space is stored in    ==
C     ==  several outfiles, which contain CV position and also        == 
C     ==  informations about the hills filling the PES wells and      ==
C     ==  other deteils regarding the dynamics
C     ==--------------------------------------------------------------==

C     NCOLVAR   : # of collective variables
C     LHILLS    : true when hills are used in the dynamics of the constraints
C     I_META    : # of steps in the meta-dynamics
C     I_META_MAX: max # of steps in the meta-dynamics
C     I_META_RES: if restart from previous meta dynamics, 
C                 # of steps already done
C     I_CVST    : # of steps since last HILL has been added
C     CV_IST    : actual values of collective variables, dim NCOLVAR
C     CV_DYN    : value of the dynamically evolving collective variables 
C     CV_VEL    : velocity of the collective variables
C     CVD_IST   : actual values of diffusion coefficient of CV
C     HLLH      : hills height
C     HLLW      : hills amplitude 
C     CSCL_FAC  : scale factors required for an homogeneous dynamics in the
C                 NCOLVAR dimensions of the CV space 
C                 they can be read from input  and they  can be tuned on the
C                 amplitudes of the variables fluctuations
C     IBOUND    : 1 if a wall constrains the CV within a certain region of space
C                 otherwise 0 (integer, dimension NCOLVAR)
C     VBOUND    : parameters for the definition of constraining potential
C                 (real, dimension 4*NCOLVAR)
C                 if WALL+ VBOUND(2) is the barrier for CV_DYN > VBOUND(1)
C                 if WALL- VBOUND(4) is the barrier for CV_DYN < VBOUND(3)
C     FI_HARM   : force contribution on ions due to the harm. pot. 
C                 dim (NODIM)
C     F_HARM    : force contribution due to the harm. pot.
C                 and acting on the dynamical CV
C                 dim (NCOLVAR)
C     F_HILL    : force contribution due to the time dep. hill potentiel
C                 and acting on the dynamical CV
C                 dim (NCOLVAR)
C     F_WALL    : force contribution due to the boundary potential 
C                 and acting on the dynamical CV
C                 dim (NCOLVAR)
C     META_RESTART: restart for a previous meta-dynamics
C                 acculators and the constraints path have to be read 
C                 from the right files
C                 the output is appended 
C     CV_PATH   : CV history (dim NTOT_ITER*NCOLVAR)
C     CSCL_VAL  : CV scaling factors history (dim NTOT_ITER*NCOLVAR)
C     HLLW_VAL  : hills amplitude (dim NTOT_ITER)
C     HLLH_VAL  : hills altitude  (dim NTOT_ITER)
C     F_AVER    : accumulator for the calculation of the average force
C                 acting on the CV, as a contribution of the 
C                 underlaying potential 
C     IF_AVER   : counter for the calculation of the average
C     IW_CV     : counter for writing the CV values on the cvmdck_mtd file
C                 in order to monitor the behavior along the MD trajectory
C     DISPLACEMENT: norm of the displacment in CV space
C     CV_MASS   : masse of CV, if not assigned in 
C                 the input file, they are automathically calculated
C                 dim (NCOLVAR)
C     KHARM     : spring constant for the harmonic potential, if not
C                 assigned in input file, default values are chosen
C                 dim (NCOLVAR)
C     ==--------------------------------------------------------------==

      IMPLICIT NONE
      INCLUDE  'system.h'
      INCLUDE  'ions.inc'
      INCLUDE  'cotr.inc'
      INCLUDE  'cnst_dyn.inc'
      INCLUDE  'irat.inc'
      INCLUDE  'tpar.inc'
      INCLUDE  'ropt.inc'
      INCLUDE  'soft.inc'
      INCLUDE  'store.inc'
      INCLUDE  'adat.inc'
      INCLUDE  'cnst.inc'
      INCLUDE  'strs.inc'
      INCLUDE  'fileopen.inc'

C     Arguments
      INTEGER       LSCR
      REAL*8        TAUP(3,NAX,*),VELP(3,NAX,*),
     &              TSCR(3,NAX,*),SCR(*),EKINC,EKINP
      LOGICAL       LQUENCH,LMETARES,RESETCV
C     Variables
      INTEGER       NTOT_ITER
      REAL*8        FI_HARM(*),GASDEV
      EXTERNAL      GASDEV
      POINTER       (IP_FI_HARM,FI_HARM)

      REAL*8        CVD_IST(NCOLVAR),CV_DISP(NCOLVAR),
     &              HC_LAST(NCOLVAR),
     &              F_HARM(NCOLVAR),F_HILL(NCOLVAR),F_WALL(NCOLVAR),
     &              F_AVER(NCOLVAR),HHH,HHHT,
     &              HH_TEST(NCOLVAR),CV_SCL(NCOLVAR),CVD_SCL(NCOLVAR)

      pointer      (IP_CVD_IST,CVD_IST),(IP_CV_DISP,CV_DISP),
     &             (IP_HC_LAST,HC_LAST),
     &             (IP_F_HARM,F_HARM),(IP_F_HILL,F_HILL),
     &             (IP_F_WALL,F_WALL),(IP_F_AVER,F_AVER),
     &             (IP_HH_TEST,HH_TEST),(IP_CV_SCL,CV_SCL),
     &             (IP_CVD_SCL,CVD_SCL)
      SAVE          IP_CVD_IST,IP_CV_SCL,IP_CVD_SCL 
      DIMENSION     DOF_INCV(NODIM,NCOLVAR,18),
     &              DET_COLVAR(NODIM,NCOLVAR),DET_CELVAR(3,3,NCOLVAR)
      DIMENSION     CV_PATH(I_META_MAX+I_META_RES,NCOLVAR),
     &              CSCL_VAL(I_META_RES+I_META_MAX,NCOLVAR),
     &              HLLW_VAL(I_META_RES+I_META_MAX),
     &              HLLH_VAL(I_META_RES+I_META_MAX)
      DIMENSION     VBOUND(4,NCOLVAR),CSCL_FAC(3,NCOLVAR),
     &             IBOUND(NCOLVAR)

      INTEGER       IFIRST,I_META,I_CVST,I_TEMP,IF_AVER,IW_CV
      DATA          IFIRST,I_META,I_CVST,I_TEMP,IF_AVER,IW_CV / 6*0 /
      SAVE          IFIRST,I_META,I_CVST,I_TEMP,IF_AVER,IW_CV
      INTEGER       IFIRSTP
      DATA          IFIRSTP / 0 /
      SAVE          IFIRSTP

      CHARACTER*20  FILE9
      DATA          FILE9 /'cvmdck_mtd' /

      INTEGER       IA,IS,I,IE

      LOGICAL       HILL_ADD
      LOGICAL       LSKIP, FINAL
      CHARACTER     TAG ,LINEFORM*100,CHNUM*10

      SAVE          IP_HC_LAST,IP_F_AVER
      SAVE          IP_F_HARM,IP_F_HILL,IP_F_WALL

      REAL*8        TOLLM,MAXF2,FACT,FACT2,DIF_CV,DISP2,HLLW2,
     &              HH_MIN,HH_MAX,HH,HLLH0,DISPLACEMENT,TWOPI
      INTEGER       ISUB,II,ICV,IJ,LENGTH,ISC2,
     &              LSCR2,IP,IDOF,msglen
      INTEGER       L_CV_DISP,L_F_CV,L_FI_HARM,
     &              L_CV_SCL,L_CVD_SCL,L_SCR_HIL

      LOGICAL       FERROR
      REAL*8        TEMP1,TEMP2,TEMPP
      DATA          TEMP1,TEMP2 /0.0D0,0.0D0/
      SAVE          TEMP1,TEMP2,TEMPP,HLLH0
C     ==--------------------------------------------------------------==
C     Set time for the routine
      CALL TISET(' META_EXTL',ISUB)

      FINAL = .FALSE.
      TWOPI=2.0D0*PI
C     ==--------------------------------------------------------------==
      CALL AZZERO(SCR,LSCR)
      IF(IFIRSTP .EQ. 0) THEN
        I_META  = I_META_RES+1
        I_TEMP  = 0
        I_CVST  = 0
        IF_AVER = 0

C     Allocate Memory
        CALL MEMORY(IP_CV_DYN,NCOLVAR,'CV_DYN')
        CV_ASSOCIATED = .true.
        IFIRSTP = 1
      ENDIF
      IF(.NOT.PARENT) GOTO 9999
C     ==--------------------------------------------------------------==
C     Temporary Array

#ifdef _BGL 
      L_FI_HARM =  NODIM+MOD(NODIM,2)
      L_CV_DISP = NCOLVAR+MOD(NCOLVAR,2)
      L_F_CV    = NCOLVAR+MOD(NCOLVAR,2)
      L_CV_SCL  = NCOLVAR+MOD(NCOLVAR,2)
      L_CVD_SCL = NCOLVAR+MOD(NCOLVAR,2)
      L_SCR_HIL = 2*NCOLVAR*NCOLVAR+NODIM+NODIM*NCOLVAR
#else
      L_FI_HARM =  NODIM
      L_CV_DISP = NCOLVAR
      L_F_CV    = NCOLVAR
      L_CV_SCL  = NCOLVAR
      L_CVD_SCL = NCOLVAR
      L_SCR_HIL = 2*NCOLVAR*NCOLVAR+NODIM+NODIM*NCOLVAR
#endif

      LENGTH  = L_FI_HARM+L_CV_DISP+L_F_CV+L_CV_SCL+
     &          L_CVD_SCL+L_SCR_HIL

      CALL TEST_SCR('META_EXTLAGR',TAG,LSCR,LENGTH)

      CALL  GET_ADDR(IP_FI_HARM ,SCR(1))
      CALL  GET_ADDR(IP_CV_DISP ,SCR(1+L_FI_HARM))
      CALL  GET_ADDR(IP_HH_TEST ,SCR(1+L_FI_HARM+L_CV_DISP))
      CALL  GET_ADDR(IP_CV_SCL  ,SCR(1+L_FI_HARM+L_CV_DISP+
     &                    L_F_CV))
      CALL  GET_ADDR(IP_CVD_SCL ,SCR(1+L_FI_HARM+L_CV_DISP+
     &                    L_F_CV+L_CV_SCL))
      ISC2         = 1+L_FI_HARM+L_CV_DISP+
     &                    L_F_CV+L_CV_SCL+L_CVD_SCL
      LSCR2        = LSCR - L_FI_HARM+L_CV_DISP+
     &                    L_F_CV+L_CV_SCL+L_CVD_SCL

      NTOT_ITER = I_META_MAX+I_META_RES
      TOLLM     = TOLL_AVCV


      IF(IFIRST .EQ. 0) THEN

C     Allocate Memory
        CALL DCOPY(NCOLVAR,CV_IST,1,CV_DYN,1)
        DO i=1,NCOLVAR ! ale-cmb
          IF(INITIAL_VALUE(i).eqv. .true.) CV_DYN(i)=CV_DYN_0(i)
        ENDDO
        CALL MEMORY(IP_CV_VEL,NCOLVAR,'CV_VEL')
        CALL AZZERO(CV_VEL,NCOLVAR)
        CALL MEMORY(IP_CVD_IST,NCOLVAR,'CVD_IST')
        CALL AZZERO(CVD_IST,NCOLVAR)
        CALL MEMORY(IP_HC_LAST,NCOLVAR,'HC_LAST')
        CALL AZZERO(HC_LAST,NCOLVAR)
        CALL MEMORY(IP_F_AVER,NCOLVAR,'F_AVER')
        CALL AZZERO(F_AVER,NCOLVAR)
        CALL MEMORY(IP_F_HARM,NCOLVAR,'F_HARM')
        CALL AZZERO(F_HARM,NCOLVAR)
        CALL MEMORY(IP_F_HILL,NCOLVAR,'F_HILL')
        CALL AZZERO(F_HILL,NCOLVAR)
        CALL MEMORY(IP_F_WALL,NCOLVAR,'F_WALL')
        CALL AZZERO(F_WALL,NCOLVAR)
        CALL MEMORY(IP_RA,NCOLVAR,'RA')
        CALL AZZERO(RA,NCOLVAR)

        CALL MEMORY(IP_CV_PATH,NCOLVAR*NTOT_ITER,'CV_PATH')
        CALL AZZERO(CV_PATH,NCOLVAR*NTOT_ITER)
        CALL MEMORY(IP_CSCL_VAL,NCOLVAR*NTOT_ITER,'CSCL_VAL')
        CALL AZZERO(CSCL_VAL,NCOLVAR*NTOT_ITER)
        CALL MEMORY(IP_HLLW_VAL,NTOT_ITER ,'HLLH_VAL')
        CALL AZZERO(HLLW_VAL,NTOT_ITER )
        CALL MEMORY(IP_HLLH_VAL,NTOT_ITER ,'HLLH_VAL')
        CALL AZZERO(HLLH_VAL,NTOT_ITER )


!     If META_RESTART read data from proper input
        IF(META_RESTART .AND. .NOT. TCVANALYSIS) THEN
          IF (TRESFILE) THEN
            CALL RMTDRES(50,CV_DISP,.TRUE.,NTOT_ITER)
          ELSE 
            CALL CV_READ_OUT(CV_DISP)
          ENDIF !TRESFILE
#ifdef __SR11000
*poption parallel, tlocal(ICV)
*voption indep(HC_LAST,CV_PATH,CV_DISP)
#else
!$OMP parallel do private(ICV)
#endif
          DO ICV = 1,NCOLVAR
            HC_LAST(ICV) = CV_PATH(I_META_RES,ICV)+CV_DISP(ICV)
          ENDDO
          CALL DCOPY(NCOLVAR,HC_LAST,1,CV_DYN,1)
        ENDIF   !META_RESTART

        IF(.NOT. TCVANALYSIS) THEN
C       Masses
          IF(CV_TEMP0 .NE. 0.0D0) THEN
            FACT = CV_TEMP0*KBOLTZ*10.D0
          ELSE
            FACT = TEMPW   *KBOLTZ*10.D0
          ENDIF
          FACT2 = FACT * DELT_IONS*DBLE(INTER_HILL)* 
     &            DELT_IONS*DBLE(INTER_HILL)
          DO ICV = 1,NCOLVAR
            IF(CV_MASS(ICV) .EQ. 0.0D0) THEN
              CV_MASS(ICV) = FACT2 /
     &                 ((CSCL_FAC(1,ICV)*HLLW)*(CSCL_FAC(1,ICV)*HLLW))
              WRITE(6,'(A,I5,A,f15.4,A)') 'Default Mass for CV #',ICV,
     &                        ' is assigned  = ',
     &                      CV_MASS(ICV)/SCMASS,
     &                      ' a.m.u.(a.u.)^2/(u.CV)^2'
            ELSE
              WRITE(6,'(A,I5,A,f15.4,A)') 'Mass for CV #',ICV,
     &                        ' is assigned  = ',
     &                      CV_MASS(ICV) ,' a.m.u.(a.u.)^2/(u.CV)^2'
              CV_MASS(ICV) =  SCMASS*CV_MASS(ICV)
            ENDIF

C       Harmonic Potential
            IF(KHARM(ICV) .EQ. 0.0D0) THEN
              KHARM(ICV) = FACT/
     &                 ((CSCL_FAC(1,ICV)*HLLW)*(CSCL_FAC(1,ICV)*HLLW))
              WRITE(6,'(A,I5,A,f10.4,A)') 
     &                     'Default harmonic constant for CV #',ICV,
     &                     ' is assigned ',KHARM(ICV),' a.u./(u.CV)^2'
            ENDIF
            IF(.NOT. LKFIX) LCHEKHARM = .TRUE.
          ENDDO
C     Initialization of CV Velocity
          IF(I_META .EQ. 1) THEN
            CALL RINVELCV(CV_VEL,CV_MASS,CSCL_FAC,NCOLVAR,
     &                    CV_TEMP0,EKINCV)
            WRITE(6,'(2x,A,f10.4)') 
     &           'CV velocity have been initialized at T = ',CV_TEMP0
            WRITE(6,'(2x,A,f10.4)') 
     &           'the initial CV kinetic energy is = ',EKINCV
          ENDIF

          IF(LCVTC)THEN
            TEMP1=CV_TEMP+CV_DTEMP
            TEMP2=CV_TEMP-CV_DTEMP
          ENDIF
          TEMPP = CV_TEMP0
        ELSE
          CALL AZZERO(CV_VEL,NCOLVAR)
        ENDIF

C       ==--------------------------------------------------------------==
        IF(TCVMONITOR) THEN
          CALL FILEOPEN(444,FILE9,FO_APP+FO_VERB,FERROR)
        ENDIF

      ENDIF

C     ==--------------------------------------------------------------==
C     Calculate the Collective Variables and their Derivatives
      CALL COLVAROFR(TAUP,TSCR)

cmb - do we really need to treat the spin CV differently ?
Cale  IF(RESETCV .AND. TLOCALIZESPIN) THEN
Cale    DO ICV = 1,NCOLVAR
Cale      IF(ICV_SPIN(ICV) .NE. 0) THEN
Cale        CV_DYN(ICV) = CV_IST(ICV)
Cale      ENDIF
Cale    ENDDO
Cale    RESETCV = .FALSE.
Cale  ENDIF
C     ==--------------------------------------------------------------==
!     Initialization

      IF(IFIRST.EQ.0) THEN
        IF(I_META.EQ.1)THEN
!$OMP parallel do private(ICV)
          DO ICV = 1,NCOLVAR
            HC_LAST(ICV) = CV_IST(ICV)
          ENDDO
        ENDIF
        HLLH0 = HLLH
        IFIRST=1
cmb - do we really need to treat the spin CV differently ?
Cale    IF(.NOT. META_RESTART .AND. TLOCALIZESPIN) THEN
Cale      CALL DCOPY(NCOLVAR,CV_IST,1,CV_DYN,1)
Cale    ENDIF
        GOTO 200
      ENDIF

      HILL_ADD = .FALSE.
      IF(TTRAJOVRR) NTRAJ=999999999

      I_CVST = I_CVST + 1
      IW_CV=IW_CV+1

!     ==--------------------------------------------------------------==
!     Check  CV_DYN wrt the center of the last Hill (HC_LAST)
!     to be verified conditions: 
!                               minimum time interval 
!                               minimum displacement in CV space
!     If the conditions are sadisfied HILL_ADD = true 
!     and a new hill will be added
      IP = ISC2
      CALL CALC_POS_DYN(TOLLM,I_CVST,I_TEMP,I_META,HC_LAST,
     &             SCR(IP),LSCR2,HILL_ADD)

      IF(LMETARES) THEN
        FINAL = .TRUE.
        GOTO 9999
      END IF
!     ==--------------------------------------------------------------==
!     Tune the Harmonic Potential according to the PES

      IF(LCHEKHARM .AND. HILL_ADD) THEN
        CALL CV_FORCES3(TAUP,TSCR,VELP,HH_TEST,SCR(ISC2),LSCR2)
        DO ICV = 1,NCOLVAR
          DIF_CV = CV_IST(ICV)-CV_DYN(ICV)
          IF (IANGCV(ICV) .EQ. 1 .AND. DIF_CV .GT. PI) THEN
            DIF_CV = DIF_CV - TWOPI
          ELSEIF (IANGCV(ICV) .EQ. 1 .AND. DIF_CV .LT. -PI) THEN
            DIF_CV = DIF_CV + TWOPI
          ENDIF
          IF(HH_TEST(ICV).LE.1.D+1 .AND. HH_TEST(ICV).GT.0.2D0) THEN
            FACT = DABS(DIF_CV)/CSCL_FAC(1,ICV)
            IF(FACT.LT. HLLW) THEN
              KHARM(ICV) = DABS(HH_TEST(ICV)/2.D0/HLLW)
            ELSE
              KHARM(ICV) = DABS(HH_TEST(ICV)/2.D0/DIF_CV)
            ENDIF
          ELSE
          ENDIF
        ENDDO
      ENDIF

!     ==--------------------------------------------------------------==
!     Calculation of the forces on the collective variables
!     This is used in order to determine the optimal hills parameters 
!     In F_aver the forces on CV are accumulated in order to calculate
!     their average value when the meta step is concluded
      IF(LTUNE_HH) THEN
        DO ICV = 1,NCOLVAR
          F_AVER(ICV) = F_AVER(ICV) 
!     &        + 2.0D0*KHARM(ICV)*DIF_CV
     &        + F_HARM(ICV)+F_HILL(ICV)/CSCL_FAC(1,ICV)
        ENDDO
        IF_AVER = IF_AVER + 1
      ENDIF
!     ==--------------------------------------------------------------==
!     Monitoring of the CV along the MD trajectory. Even if no hill is added
!     at this point, if IW_CV is a multiple of WCV_FREQ, 
!     the values of the CV_IST and of the CV_DYN
!     are written in the cvmdck_mtd file
!     in the first column the IW_CV counter is reported, that indicates the
!     number of MD steps of the actual run, in the second column the global 
!     metastep is reported, afterwards the values of the two CV sets and 
!     instantaneous temperature in K
      IF(TCVMONITOR .AND. MOD(IW_CV,WCV_FREQ) .EQ. 0) THEN
        WRITE(CHNUM,'(I5)') NCOLVAR
        CALL XSTRING(CHNUM,IA,IE)
        LINEFORM =
     &     '(1X,2I7,'//CHNUM(IA:IE)//
     &     'E14.6,'//CHNUM(IA:IE)//'E14.6,E14.6)'
        write(444,LINEFORM)IW_CV,NFI ,(CV_IST(ICV),ICV=1,NCOLVAR),
     &               (CV_DYN(ICV),ICV=1,NCOLVAR),TEMPP
        call my_flush(444)
      ENDIF

!     ==--------------------------------------------------------------==
!     New Step of Meta-Dynamics in the Space of CV
      IF(HILL_ADD) THEN
          IF(MOD(I_META,TR_FREQ) .EQ. 0 .OR. I_META .EQ. 1 ) THEN
            IF(TTRAJOVRR) NTRAJ=1
          ENDIF
        IF(.NOT. TCVANALYSIS) THEN
!         ==--------------------------------------------------------------==
!         Squared Norm of the Displacement in the CV space

          DISPLACEMENT = 0.0D0
          DO ICV = 1,NCOLVAR
            DIF_CV = CV_DYN(ICV)-HC_LAST(ICV)
            IF (IANGCV(ICV) .EQ. 1 .AND. DIF_CV .GT. PI) THEN
              DIF_CV = DIF_CV - TWOPI
            ELSEIF (IANGCV(ICV) .EQ. 1 .AND. DIF_CV .LT. -PI) THEN
              DIF_CV = DIF_CV + TWOPI
            ENDIF
            FACT = DIF_CV/CSCL_FAC(1,ICV)
            DISPLACEMENT = DISPLACEMENT + FACT*FACT
          ENDDO
          DISP2 = DISPLACEMENT
          DISPLACEMENT = DSQRT(DISPLACEMENT)
 
!         ==--------------------------------------------------------------==
!         Tuning Hills parameters 

          IF(LTUNE_HH) THEN
            HH_MIN = 1000.D0
            HH_MAX = 0.0D0
            LSKIP = .FALSE.
            HLLW2 = HLLW*HLLW
          
            HHHT = 0.0D0
            DO ICV = 1,NCOLVAR
              F_AVER(ICV)  = F_AVER(ICV) / DBLE(IF_AVER)
              DIF_CV = CV_DYN(ICV)-HC_LAST(ICV)
              IF (IANGCV(ICV) .EQ. 1 .AND. DIF_CV .GT. PI) THEN
                DIF_CV = DIF_CV - TWOPI
              ELSEIF (IANGCV(ICV) .EQ. 1 .AND. DIF_CV .LT. -PI) THEN
                DIF_CV = DIF_CV + TWOPI
              ENDIF

              FACT = CSCL_FAC(1,ICV)*CSCL_FAC(1,ICV)/DIF_CV *
     &              dexp(0.5D0)*HLLW2*DISP2/
     &             (DISP2*dexp(-.5D0*DISP2/HLLW2)+
     &               HLLW2*(dexp(-.5D0*DISP2/HLLW2)-
     &               dexp(-.5D0*RSHIFT*RSHIFT)))

              HHH     = 0.5D0*dabs(F_AVER(ICV)*FACT)
              HHHT = HHHT + HHH

              WRITE(6,'(2x,A,I4,3F14.6,2x,2F14.6)') 'FCV ',
     &          ICV,F_AVER(ICV),HHH,FACT

              HH_MIN       = MIN(HH_MIN,HHH)
              HH_MAX       = MAX(HH_MAX,HHH)
            ENDDO
            HH       =  HHHT / DBLE(NCOLVAR)
            IF(HH_MIN .GT. HTOP .OR. HH_MAX .LT. HLOW) THEN
              LSKIP = .TRUE.
              WRITE(6,'(A,f10.5,A,f10.5,A)') 'Hill top > ', HTOP*AU_KCM,
     &                 ' Kcal, or Hill top < ',HLOW*AU_KCM,' Kcal'
            ENDIF
            IF(.NOT. LSKIP .AND. HH .LT. HLOW) THEN
              HLLH = (HLOW+HH_MAX)*0.5D0
            ELSEIF(.NOT. LSKIP .AND. HH .GT. HTOP) THEN
              HLLH = (HH_MIN+HTOP)*0.5D0
            ELSEIF(.NOT. LSKIP ) THEN
              HLLH = HH
            ELSE
              HLLH = HLLH*0.2D0+HLLH0*0.8D0
            ENDIF
            WRITE(6,'(/A,f10.5,3(3x,A,f10.5)/)')
     &              '||DISP|| = ',DISPLACEMENT, 
     &              'HH tuned min = ', HH_MIN,'HH tuned max = ', HH_MAX,
     &              ' HH used ',HLLH
            IF_AVER = 0
            CALL AZZERO(F_AVER,NCOLVAR)

            IF(THILLVOL) THEN
              HLLW = (HVOL0/HLLH)**(1.0D0/DBLE(NCOLVAR))
            ENDIF
          ENDIF

!         ==--------------------------------------------------------------==
!         Scaling Factors Tuning
          IF(LTUNE_CSCL) THEN
  
            CALL SCF_TUNE(I_META,HC_LAST)

          ENDIF
!         ==--------------------------------------------------------------==
!         Diffusion calculation
          CALL CV_DIFFUSION(HC_LAST,CVD_IST,CVD_SCL,CV_SCL,I_META,
     &            SCR(ISC2),LSCR2,NTOT_ITER)

!         ==--------------------------------------------------------------==
!         Open output file
          CALL CV_EXLAGR_OUT(I_META,HC_LAST,F_HARM,F_HILL,F_WALL,
     &           CVD_IST,DISPLACEMENT,EKINC,EKINP)

!         ==--------------------------------------------------------------==
!$OMP parallel do private(ICV)
          DO ICV = 1,NCOLVAR
            HC_LAST(ICV)   = CV_DYN(ICV)
          ENDDO

        ELSE
!         ==--------------------------------------------------------------==
!         Open output file
          CALL CV_EXLAGR_OUT2(I_META,EKINC,EKINP)

C         ==--------------------------------------------------------------==

        ENDIF
        I_META = I_META + 1
        I_TEMP = 0
        I_CVST = 0
      ENDIF        !HILL_ADD

!     ==--------------------------------------------------------------==
!     Calculation of the dynamics of the collective variables
!     A velocity Verlet Scheme is used

!     ==--------------------------------------------------------------==
!     Update Velocity : first half step
!                       using forces calculated in the previous step
!$OMP parallel do private(ICV,FACT)
      DO ICV = 1,NCOLVAR
        FACT = DT_IONS/(2.D0*CV_MASS(ICV))
        CV_VEL(ICV) = CV_VEL(ICV) 
     &      + FACT*(F_HARM(ICV)+F_HILL(ICV)/CSCL_FAC(1,ICV)+F_WALL(ICV))
      ENDDO

      IF(TCVLANGEVIN) THEN
        DO ICV=1,NCOLVAR
          FACT = DT_IONS/(2.D0*CV_MASS(ICV))
          RA(ICV)=GASDEV()
          CV_VEL(ICV)=CV_VEL(ICV)-CV_MASS(ICV)*CV_VEL(ICV)*CV_LANGAMMA
     &        *FACT+DSQRT(FACT*(2.D0*CV_MASS(ICV)))*0.5D0*DSQRT(2.D0
     &        *CV_LANGEVINTEMP*KBOLTZ/CV_MASS(ICV)*CV_LANGAMMA)*RA(ICV)
        ENDDO
      ENDIF                                                            

!     ==--------------------------------------------------------------==
!     Update Position : second order dynamics

      DO ICV = 1,NCOLVAR
        FACT = DT_IONS
        IF (IANGCV(ICV) .EQ. 1 )  THEN
          IF(CV_DYN(ICV).LT.0.D0) CV_DYN(ICV)= CV_DYN(ICV)+TWOPI
          IF(CV_DYN(ICV).GT.TWOPI) CV_DYN(ICV)= CV_DYN(ICV)-TWOPI
        ELSEIF(IANGCV(ICV) .EQ. 2 )  THEN
          IF(CV_DYN(ICV).LT.0.D0) CV_DYN(ICV)= -CV_DYN(ICV)
          IF(CV_DYN(ICV).GT. PI) CV_DYN(ICV)= PI - (CV_DYN(ICV) - PI)
        ENDIF
        CV_DYN(ICV) = CV_DYN(ICV) + FACT*CV_VEL(ICV)
      ENDDO

!     ==--------------------------------------------------------------==
!     Calculate forces with the new positions

 200  CONTINUE
!     ==--------------------------------------------------------------==
!     Calculation of the contributions to the forces on ions 
!     due to the coupling with the collective variables
!     V_ARM = SUM_icv (K_icv(CV_IST_icv - CV_DYN_icv))^2

      CALL AZZERO(FI_HARM,NODIM)
      DO ICV = 1,NCOLVAR
        DIF_CV = CV_IST(ICV)-CV_DYN(ICV)
        IF (IANGCV(ICV) .EQ. 1 .AND. DIF_CV .GT. PI) THEN
          DIF_CV = DIF_CV - TWOPI
        ELSEIF (IANGCV(ICV) .EQ. 1 .AND. DIF_CV .LT. -PI) THEN
         DIF_CV = DIF_CV + TWOPI
        ENDIF
        FACT = KHARM(ICV)*2.0D0*DIF_CV
        DO IDOF = 1,NODIM
          FI_HARM(IDOF) = FI_HARM(IDOF) - 
     &                    FACT *  DET_COLVAR(IDOF,ICV)
        ENDDO
      ENDDO

C     ==--------------------------------------------------------------==
C     Update forces on ions

      CALL AZZERO(TSCR,3*NAX*NSX)
      CALL GETTAU(TSCR,FI_HARM)
#ifdef __SR11000
*poption parallel, tlocal(IS,IA)
*voption indep(FHILLS,TSCR)
#else
!$OMP parallel do private(IS,IA) shared(NA)
#endif
      DO IS=1,NSP
        DO IA=1,NA(IS)
           FHILLS(1,IA,IS) = TSCR(1,IA,IS)
           FHILLS(2,IA,IS) = TSCR(2,IA,IS)
           FHILLS(3,IA,IS) = TSCR(3,IA,IS)
        ENDDO
      ENDDO

C     ==--------------------------------------------------------------==
C     Update stress tensor: contributions of CV as functions of HT
      IF(TCVCELL) THEN
        CALL AZZERO(DSTRCV,6)
        DO ICV = 1,NCOLVAR
          DIF_CV = CV_IST(ICV)-CV_DYN(ICV)
          FACT = -2.0D0*KHARM(ICV)*DIF_CV
          DSTRCV(1) = DSTRCV(1) + 
     &                FACT*DET_CELVAR(ALPHA(1),BETA(1),ICV)
          DSTRCV(2) = DSTRCV(2) + 
     &                FACT*DET_CELVAR(ALPHA(2),BETA(2),ICV)
          DSTRCV(3) = DSTRCV(3) + 
     &                FACT*DET_CELVAR(ALPHA(3),BETA(3),ICV)
          DSTRCV(4) = DSTRCV(4) + 
     &                FACT*DET_CELVAR(ALPHA(4),BETA(4),ICV)
          DSTRCV(5) = DSTRCV(5) + 
     &                FACT*DET_CELVAR(ALPHA(5),BETA(5),ICV)
          DSTRCV(6) = DSTRCV(6) + 
     &                FACT*DET_CELVAR(ALPHA(6),BETA(6),ICV)
        ENDDO
      ENDIF

C     ==--------------------------------------------------------------==
C     Calculate contribution of the Harmonic Potential (recall)

      CALL AZZERO(F_HARM,NCOLVAR)
      DO ICV = 1,NCOLVAR
        DIF_CV = CV_IST(ICV)-CV_DYN(ICV)
        IF (IANGCV(ICV) .EQ. 1 .AND. DIF_CV .GT. PI) THEN
          DIF_CV = DIF_CV - TWOPI
        ELSEIF (IANGCV(ICV) .EQ. 1 .AND. DIF_CV .LT. -PI) THEN
          DIF_CV = DIF_CV + TWOPI
        ENDIF
        F_HARM(ICV) =  KHARM(ICV)*2.0D0*DIF_CV
      ENDDO

C     Calculate hills contribution to forces
      IF(LHILLS) THEN
        CALL AZZERO(F_HILL,NCOLVAR)
        IF(HLORE) THEN
          CALL HILLS_LOR(CV_DYN,NCOLVAR,I_META,NTOT_ITER,F_HILL,
     &                   SCR(ISC2),LSCR2,1,0)
        ELSEIF(HRATIO) THEN
          CALL HILLS_RATIO(CV_DYN,NCOLVAR,I_META,NTOT_ITER,F_HILL,
     &         HC_LAST,SCR(ISC2),LSCR2,I_CVST,1,0)
        ELSEIF(HSHIFT) THEN
          CALL HILLS_SALS_SHIFT(CV_DYN,NCOLVAR,I_META,NTOT_ITER,F_HILL,
     &         HC_LAST,SCR(ISC2),LSCR2,I_CVST,1,0)
        ELSE IF(SPHERE) THEN
          CALL HILLS(CV_DYN,NCOLVAR,I_META,NTOT_ITER,F_HILL,
     &                   SCR(ISC2),LSCR2,1,0)
        ELSE
          CALL HILLS_SALS(CV_DYN,NCOLVAR,I_META,NTOT_ITER,F_HILL,
     &         HC_LAST,SCR(ISC2),LSCR2,I_CVST,1,0)
        ENDIF
      ENDIF

!     Calculate walls contribution to forces
      CALL AZZERO(F_WALL,NCOLVAR)
      MAXF2 = 0.0D0
      DO ICV = 1,NCOLVAR
        IF(IBOUND(ICV) .EQ. 1) THEN
          CALL SETWALL(CV_DYN(ICV),VBOUND(1,ICV),F_WALL(ICV))
        ELSEIF(IBOUND(ICV) .EQ. 2) THEN
          CALL SETWALL_OLD(CV_DYN(ICV),TYCVAR(ICV),VBOUND(1,ICV),
     &                     F_WALL(ICV))
        ENDIF 
        IF(F_WALL(ICV) .NE. 0.0D0) THEN
          write(6,'(72A)') ('*',i = 1,72) 
          write(6,'(10x,A,I5)') 
     &           'WARNING: Boundaries active for CV ', ICV
          write(6,*)  ICV, F_WALL(ICV)
          write(6,'(72A)')  ('*',i = 1,72)
       ENDIF
      ENDDO

!     ==--------------------------------------------------------------==
!     Update Velocity : second half step 
      DO ICV = 1,NCOLVAR
        FACT = DT_IONS/(2.D0*CV_MASS(ICV))
        CV_VEL(ICV) = CV_VEL(ICV) 
     &    + FACT*(F_HARM(ICV)+F_HILL(ICV)/CSCL_FAC(1,ICV)+F_WALL(ICV))
      ENDDO

      IF(TCVLANGEVIN) THEN
        DO ICV=1,NCOLVAR
          FACT = DT_IONS/(2.D0*CV_MASS(ICV))
          CV_VEL(ICV)=CV_VEL(ICV)-CV_MASS(ICV)*CV_VEL(ICV)*CV_LANGAMMA
     &        *FACT+SQRT(FACT*(2.D0*CV_MASS(ICV)))*0.5D0*SQRT(2
     &        *CV_LANGEVINTEMP*KBOLTZ/CV_MASS(ICV)*CV_LANGAMMA)*RA(ICV)
        ENDDO
      ENDIF
!     ==--------------------------------------------------------------==
!     Calculation of   Temperature of CV: CV_TEMP
!     If CV_TEMP_CONTROL, check the condition for rescaling T

      CALL RSCVELCV(TEMP1,TEMP2,TEMPP)

!     ==--------------------------------------------------------------==
!     IF MAX_STEP reached or EXIT file exist

      IF(INFI.EQ.NOMORE .OR. I_META-1.EQ.NTOT_ITER) THEN
        EXSOFT = .TRUE.

        CALL  COLVARPR

        IF(TCVMONITOR) THEN
          CALL FILECLOSE(444)
        ENDIF
      ENDIF
 9999 CONTINUE
      CALL MY_SYNC(ALLGRP)

      MSGLEN = NCOLVAR * 8
      CALL MY_BCAST(CV_DYN,MSGLEN,SOURCE,ALLGRP)

      MSGLEN = 1 * 8/IRAT
      CALL MY_BCAST(EXSOFT,MSGLEN,SOURCE,ALLGRP)

      MSGLEN = 1 * 8/IRAT
      CALL MY_BCAST(I_META ,MSGLEN,SOURCE,ALLGRP)

      MSGLEN = 1 * 8/IRAT
      CALL MY_BCAST(I_CVST ,MSGLEN,SOURCE,ALLGRP)

      IF(TLOCALIZESPIN) THEN
        MSGLEN = NCOLVAR*3 *8
        CALL MY_BCAST(RCC0,MSGLEN,SOURCE,ALLGRP)
      ENDIF

C     WRITE RESTART
      IF((MOD(I_META,ST_FREQ) .EQ. 0  .AND. I_CVST .EQ. 0)
     &                      .OR. EXSOFT .OR. LMETARES) THEN
        LMETARES = .TRUE.
        IF(I_META .GT. 1) THEN
           CALL WMTDRES(50,NTOT_ITER,I_META)
        ENDIF
      ENDIF

C     QUENCH BO
      IF(.NOT.TMDBO.AND.
     &    MOD(I_META,QW_FREQ) .EQ. 0  .AND. I_CVST .EQ. 0) 
     &    LQUENCH = .TRUE.

      IF(FINAL .AND. PARENT )  THEN
C     Deallocate
        CALL FREEM(IP_HC_LAST)
        CALL FREEM(IP_CV_PATH)
        CALL FREEM(IP_CSCL_VAL)
        CALL FREEM(IP_HLLW_VAL)
        CALL FREEM(IP_HLLH_VAL)
        CALL FREEM(IP_CV_DYN)
        CALL FREEM(IP_CV_VEL)
        CALL FREEM(IP_CVD_IST)
        CALL FREEM(IP_F_AVER)
        CALL FREEM(IP_F_HARM)
        CALL FREEM(IP_F_HILL)
        CALL FREEM(IP_F_WALL)
      ENDIF
C     ==--------------------------------------------------------------==
C     Halt time for the routine
      CALL TIHALT(' META_EXTL',ISUB)

      RETURN
      END
C
C     ==================================================================
      SUBROUTINE GIVE_SCR_META_EXTLAGR(LMTD,TAG)
C     ==================================================================
C     Estimates the scratch length necessary for META_EXLAGR
C     ==================================================================
      INCLUDE 'system.h'
      INCLUDE 'ions.inc'        !NAT
      INCLUDE 'cnst_dyn.inc'    !NCOLVAR
      INCLUDE 'mm_dim.inc'
C     Arguments
      INTEGER    LMTD,LQUENBO
      LOGICAL    oldstatus
      CHARACTER  TAG*30
C
      LMTD=0
      LQUENBO=0
      IF(.NOT.LEXTLAGRANGE)RETURN
      call mm_dim(mm_go_mm,oldstatus)
      LMTD=3*NAT+4*NCOLVAR+2*NCOLVAR*NCOLVAR+3*NAT+3*NAT*NCOLVAR
#ifdef _BGL
      LMTD=LMTD+MOD(3*NAT,2)+4*MOD(NCOLVAR,2)
#endif
      call mm_dim(mm_revert,oldstatus)
      CALL GIVE_SCR_QUENBO(LQUENBO,TAG)
      LMTD=MAX(LMTD,LQUENBO)
      RETURN
      END
C     ==================================================================

C     ==================================================================
C     MULTIPLE WALKERS
C     ==================================================================
      SUBROUTINE META_EXTLAGR_MW(TAUP,VELP,TSCR,SCR,LSCR,LQUENCH,
     &                        LMETARES,
     &                        RESETCV,EKINC,EKINP)
C     ==--------------------------------------------------------------==

      IMPLICIT NONE
      INCLUDE  'system.h'
      INCLUDE  'ions.inc'
      INCLUDE  'cotr.inc'
      INCLUDE  'mw.inc'
      INCLUDE  'cnst_dyn.inc'
      INCLUDE  'irat.inc'
      INCLUDE  'tpar.inc'
      INCLUDE  'ropt.inc'
      INCLUDE  'soft.inc'
      INCLUDE  'store.inc'
      INCLUDE  'adat.inc'
      INCLUDE  'cnst.inc'
      INCLUDE  'strs.inc'
      INCLUDE  'pimd.inc'
      INCLUDE  'fileopen.inc'

C     Arguments
      INTEGER       LSCR
      REAL*8        TAUP(3,NAX,*),VELP(3,NAX,*),
     &              TSCR(3,NAX,*),SCR(LSCR),EKINC,EKINP
      LOGICAL       LQUENCH,LMETARES,RESETCV
C     Variables
      INTEGER       NTOT_ITER
      REAL*8        FI_HARM(NODIM,NWALK)
      pointer       (IP_FI_HARM,FI_HARM)

      REAL*8        CVD_IST(NCOLVAR*NWALK),CV_DISP(NCOLVAR*NWALK), !MW
     &              HC_LAST(NCOLVAR*NWALK), !MW 
     &              F_HARM(NCOLVAR*NWALK),F_HILL(NCOLVAR*NWALK),
     &              F_WALL(NCOLVAR*NWALK),
     &              F_AVER(NCOLVAR),HHH,HHHT,
     &              HH_TEST(NCOLVAR),CV_SCL(NCOLVAR*NWALK),
     &              CVD_SCL(NCOLVAR*NWALK)

      pointer      (IP_CVD_IST,CVD_IST),(IP_CV_DISP,CV_DISP),
     &             (IP_HC_LAST,HC_LAST),
     &             (IP_F_HARM,F_HARM),(IP_F_HILL,F_HILL),
     &             (IP_F_WALL,F_WALL),(IP_F_AVER,F_AVER),
     &             (IP_HH_TEST,HH_TEST),(IP_CV_SCL,CV_SCL),
     &             (IP_CVD_SCL,CVD_SCL)
      SAVE          IP_CVD_IST,IP_CV_SCL,IP_CVD_SCL 
      DIMENSION     DOF_INCV(NODIM,NCOLVAR,18),
     &              DET_COLVAR(NODIM,NCOLVAR),DET_CELVAR(3,3,NCOLVAR)
      DIMENSION     CV_PATH(I_META_MAX+I_META_RES,NCOLVAR), !MW
     &              CSCL_VAL(I_META_RES+I_META_MAX,NCOLVAR),
     &              HLLW_VAL(I_META_RES+I_META_MAX),
     &              HLLH_VAL(I_META_RES+I_META_MAX)
      DIMENSION     VBOUND(4,NCOLVAR),CSCL_FAC(3,NCOLVAR),
     &             IBOUND(NCOLVAR)

      INTEGER       IFIRST,I_META,I_CVST,I_TEMP,IF_AVER,IW_CV
      DATA          IFIRST,I_META,I_CVST,I_TEMP,IF_AVER,IW_CV / 6*0 /
      SAVE          IFIRST,I_META,I_CVST,I_TEMP,IF_AVER,IW_CV
      INTEGER       IFIRSTP
      DATA          IFIRSTP / 0 /
      SAVE          IFIRSTP
CMW
      INTEGER       I_TEMP_MW(NWALK), I_CVST_MW(NWALK)
      POINTER       (IP_I_TEMP_MW,I_TEMP_MW),(IP_I_CVST_MW,I_CVST_MW)
      SAVE          IP_I_TEMP_MW,IP_I_CVST_MW

      CHARACTER*20  FILE9
      DATA          FILE9 /'cvmdck_mtd' /

      INTEGER       IA,IS,I,IE,J,K

      LOGICAL       HILL_ADD
      LOGICAL       LSKIP, FINAL
      CHARACTER     TAG ,LINEFORM*100,CHNUM*10

      SAVE          IP_HC_LAST,IP_F_AVER
      SAVE          IP_F_HARM,IP_F_HILL,IP_F_WALL

      REAL*8        TOLLM,MAXF2,FACT,FACT2,DIF_CV,DISP2,HLLW2,
     &              HH_MIN,HH_MAX,HH,HLLH0,DISPLACEMENT,
     &              DISPLACEMENT_MW(NWALK)
      INTEGER       ISUB,II,ICV,IJ,LENGTH,ISC2,
     &              LSCR2,IP,IDOF,msglen
      INTEGER       L_CV_DISP,L_F_CV,L_FI_HARM,
     &              L_CV_SCL,L_CVD_SCL,L_SCR_HIL

      LOGICAL       FERROR
      REAL*8        TEMP1,TEMP2,TEMPP
      DATA          TEMP1,TEMP2 /0.0D0,0.0D0/
      SAVE          TEMP1,TEMP2,TEMPP,HLLH0
CMW
      INTEGER       IWALK,IPW,IWALK1,IWALK2,I_META_tmp,I_ADD
CMW
      REAL*8        DET_COLVAR_MW(NODIM,NCOLVAR,NWALK),
     &              TAUPW_SCR(3,NAX,NSX,NWALK),
     &              TAUPW(3,NAX,NSX,NWALK),CV_IST_SCR(NCOLVAR,NWALK)
      pointer      (IP_DET_COLVAR_MW,DET_COLVAR_MW),
     &              (IP_TAUPW_SCR,TAUPW_SCR),
     &              (IP_TAUPW,TAUPW),
     &              (IP_CV_IST_SCR,CV_IST_SCR)
      SAVE          IP_DET_COLVAR_MW,IP_TAUPW_SCR,IP_TAUPW,IP_CV_IST_SCR
      LOGICAL       HILL_ADD_MW(NWALK)
      POINTER       (IP_HILL_ADD_MW,HILL_ADD_MW)
      SAVE          IP_HILL_ADD_MW 
      REAL*8        time_dbg(20,20),HLLH_BAK 
      
C     ==--------------------------------------------------------------==
C     Set time for the routine
      CALL MY_SYNC(SUPERGROUP)
      CALL TISET('META_EX_MW',ISUB)
C
#ifndef PARALLEL
      CALL STOPGM(' META_EXLAGR_MW',
     &     'MULTIPLE WALKERS ONLY WITH PARALLEL RUN')
#endif
C TODO the following features have to be fixed!
      IF(LKFIX)CALL STOPGM('META_EXLAGR_MW',
     &    'MULTIPLE WALKER AND ADAPTIVE K NOT IMPLEMENTED')
      IF(LTUNE_HH)CALL STOPGM('META_EXLAGR_MW',
     &    'MULTIPLE WALKER AND ADAPTIVE HILL HEIGHT NOT IMPLEMENTED')
      IF(LTUNE_CSCL)CALL STOPGM('META_EXLAGR_MW',
     &    'MULTIPLE WALKER AND SCALING FACTOR TUNING NOT IMPLEMENTED')
       IF(TCVCELL)CALL STOPGM('META_EXLAGR_MW',
     &    'MULTIPLE WALKER AND CV CELL NOT IMPLEMENTED')
       IF(TCVLANGEVIN)CALL STOPGM('META_EXLAGR_MW',
     &    'MULTIPLE WALKER AND LANGEVIN THERMOSTAT NOT IMPLEMENTED')
      IF((.NOT.SPHERE).AND.PARENT.AND.(IFIRSTP.EQ.0))THEN
         WRITE(6,'(/,A)')' !!! WARNING !!!'
         WRITE(6,'(A,/)')
     &    '!!! MULTIPLE WALKER AND WITH REQUESTED HILL TYPE'//
     &    ' NOT FULLY TESTED !!!'
      END IF
C
      IWALK=WALKER_ID
      FINAL = .FALSE.
C     ==--------------------------------------------------------------==
      CALL AZZERO(SCR,LSCR)
      IF(IFIRSTP .EQ. 0) THEN
        I_META  = I_META_RES+1
        I_CVST = 0
        IF_AVER = 0
        CALL MEMORY(IP_I_TEMP_MW,NWALK,'IP_I_TEMP_MW')
        CALL IAZZERO(I_TEMP_MW,NWALK)
        CALL MEMORY(IP_I_CVST_MW,NWALK,'IP_I_CVST_MW')
        CALL IAZZERO(I_CVST_MW,NWALK)
C     Allocate Memory
        CALL MEMORY(IP_CV_DYN,NCOLVAR*NWALK,'CV_DYN')
        CV_ASSOCIATED = .true.
        IFIRSTP = 1
        CALL MEMORY(IP_TAUPW_SCR,NAX*NSX*3*NWALK,'IP_TAUPW_SCR')
        CALL MEMORY(IP_TAUPW,NAX*NSX*3*NWALK,'IP_TAUPW')
        CALL MEMORY(IP_EKINCV_WALK,NWALK,'EKINCV_WALK')
        CALL MEMORY(IP_VHARM_WALK,NWALK,'VHARM_WALK')
      ENDIF
      CALL AZZERO(EKINCV_WALK,NWALK)
      CALL AZZERO(VHARM_WALK,NWALK)
C
      CALL AZZERO(TAUPW_SCR,NSX*NAX*3*NWALK)
      CALL AZZERO(TAUPW,NSX*NAX*3*NWALK)
      IF(PARENT)THEN
        DO I=1,3 
          DO J=1,NAX
             DO K=1,NSX
                TAUPW_SCR(I,J,K,IWALK)=TAUP(I,J,K)
             END DO
          END DO
        END DO
      END IF
C
      MSGLEN=3*NAX*NSX*NWALK*8
      CALL MY_COMBINE(TAUPW_SCR,TAUPW,MSGLEN,1,SUPERGROUP)
C
CMW   all processors are zeroed. Needed for final MY_COMBINE
      CALL AZZERO(TSCR,3*NAX*NSX)

      IF(.NOT.GRANDPARENT) GOTO 9999
C     ==--------------------------------------------------------------==
C     Temporary Array
#ifdef _BGL 
      L_FI_HARM =  NODIM*NWALK+MOD(NODIM*NWALK,2)
      L_CV_DISP = NCOLVAR*NWALK+MOD(NCOLVAR*NWALK,2)
      L_F_CV    = NCOLVAR*NWALK+MOD(NCOLVAR*NWALK,2)
      L_CV_SCL  = NCOLVAR*NWALK+MOD(NCOLVAR*NWALK,2)
      L_CVD_SCL = NCOLVAR*NWALK+MOD(NCOLVAR*NWALK,2)
      L_SCR_HIL = 2*NCOLVAR*NCOLVAR+NODIM+NODIM*NCOLVAR
#else
      L_FI_HARM =  NODIM*NWALK
      L_CV_DISP = NCOLVAR*NWALK
      L_F_CV    = NCOLVAR*NWALK
      L_CV_SCL  = NCOLVAR*NWALK
      L_CVD_SCL = NCOLVAR*NWALK
      L_SCR_HIL = 2*NCOLVAR*NCOLVAR+NODIM+NODIM*NCOLVAR
#endif

      LENGTH  = L_FI_HARM+L_CV_DISP+L_F_CV+L_CV_SCL+
     &          L_CVD_SCL+L_SCR_HIL

      CALL TEST_SCR('META_EXTLAGR',TAG,LSCR,LENGTH)

      CALL  GET_ADDR(IP_FI_HARM ,SCR(1))
      CALL  GET_ADDR(IP_CV_DISP ,SCR(1+L_FI_HARM))
      CALL  GET_ADDR(IP_HH_TEST ,SCR(1+L_FI_HARM+L_CV_DISP))
      CALL  GET_ADDR(IP_CV_SCL  ,SCR(1+L_FI_HARM+L_CV_DISP+
     &                    L_F_CV))
      CALL  GET_ADDR(IP_CVD_SCL ,SCR(1+L_FI_HARM+L_CV_DISP+
     &                    L_F_CV+L_CV_SCL))
      ISC2         = 1+L_FI_HARM+L_CV_DISP+
     &                    L_F_CV+L_CV_SCL+L_CVD_SCL
      LSCR2        = LSCR - L_FI_HARM+L_CV_DISP+
     &                    L_F_CV+L_CV_SCL+L_CVD_SCL

      NTOT_ITER = I_META_MAX+I_META_RES
      TOLLM     = TOLL_AVCV


      IF(IFIRST .EQ. 0) THEN

C     Allocate Memory
        CALL MEMORY(IP_CV_VEL,NCOLVAR*NWALK,'CV_VEL')
        CALL AZZERO(CV_VEL,NCOLVAR*NWALK)
        CALL MEMORY(IP_CVD_IST,NCOLVAR*NWALK,'CVD_IST')
        CALL AZZERO(CVD_IST,NCOLVAR*NWALK)
        CALL MEMORY(IP_HC_LAST,NCOLVAR*NWALK,'HC_LAST') !NN must be shared HC_LAST:NCOLVAR*NWALK
        CALL AZZERO(HC_LAST,NCOLVAR*NWALK)
        CALL MEMORY(IP_F_AVER,NCOLVAR,'F_AVER')
        CALL AZZERO(F_AVER,NCOLVAR)
        CALL MEMORY(IP_F_HARM,NCOLVAR*NWALK,'F_HARM')
        CALL AZZERO(F_HARM,NCOLVAR*NWALK)
        CALL MEMORY(IP_F_HILL,NCOLVAR*NWALK,'F_HILL')
        CALL AZZERO(F_HILL,NCOLVAR*NWALK)
        CALL MEMORY(IP_F_WALL,NCOLVAR*NWALK,'F_WALL')
        CALL AZZERO(F_WALL,NCOLVAR*NWALK)

        CALL MEMORY(IP_CV_PATH,NCOLVAR*NTOT_ITER,'CV_PATH')
        CALL AZZERO(CV_PATH,NCOLVAR*NTOT_ITER)
        CALL MEMORY(IP_CSCL_VAL,NCOLVAR*NTOT_ITER,'CSCL_VAL')
        CALL AZZERO(CSCL_VAL,NCOLVAR*NTOT_ITER)
        CALL MEMORY(IP_HLLW_VAL,NTOT_ITER ,'HLLH_VAL')
        CALL AZZERO(HLLW_VAL,NTOT_ITER)
        CALL MEMORY(IP_HLLH_VAL,NTOT_ITER ,'HLLH_VAL')
        CALL AZZERO(HLLH_VAL,NTOT_ITER)
        CALL MEMORY(IP_DET_COLVAR_MW,NODIM*NCOLVAR*NWALK,
     &                               'DET_COLVAR_MW')
        CALL AZZERO(DET_COLVAR_MW,NODIM*NCOLVAR*NWALK)
        CALL MEMORY(IP_CV_IST_SCR,NCOLVAR*NWALK,'CV_IST_SCR')
        CALL AZZERO(CV_IST_SCR,NCOLVAR*NWALK)
        CALL MEMORY(IP_HILL_ADD_MW,NWALK,'HILL_ADD_MW')
        CALL MEMORY(IP_SKIPHILL,NWALK,'SKIPHILL')
        DO IWALK1=1,NWALK
           HILL_ADD_MW(IWALK1)=.FALSE.
           SKIPHILL(IWALK1)=.FALSE. !skip the first hill; setting height to zero
        END DO

!     If META_RESTART read data from proper input
        IF(META_RESTART .AND. .NOT. TCVANALYSIS) THEN
             DO IWALK1=1,NWALK
               CALL MW_FILENAME('MTD_RESTART_',FMTDRES,IWALK1)
               IF (TRESFILE) THEN
C               here the cv_path should read as cv_path(1:nmtd,1:ncolvar)
C               and thus reading all the positions in one shot
C               cv_disp and cv_vel are read seperately from each MTD_RESTART_i file
                  CALL RMTDRES_MW(50,CV_DISP((IWALK1-1)*NCOLVAR+1),
     &                 .TRUE.,NTOT_ITER,IWALK1) 
               ELSE 
!MW TODO
                  WRITE(6,*)"!! WARNING!!! READING MTD OUTPUT FILES"//
     &                      " FOR RESTART IS YET NOT TESTED !!"
                  CALL CV_READ_OUT(CV_DISP)
               ENDIF !TRESFILE
             END DO
C         reconstructing the hc_last from the diffusin and the last hill position
            DO IWALK1=1,NWALK
              DO ICV = 1,NCOLVAR
                 IPW=(IWALK1-1)*NCOLVAR
                 HC_LAST(ICV+IPW)=
     &             CV_PATH(I_META_RES,ICV)+CV_DISP(ICV+IPW)
              END DO
            ENDDO !IWALK1
            CALL DCOPY(NCOLVAR*NWALK,HC_LAST,1,CV_DYN,1)
        ENDIF   !META_RESTART



        IF(.NOT. TCVANALYSIS) THEN
C       Masses
            IF(CV_TEMP0 .NE. 0.0D0) THEN
              FACT = CV_TEMP0*0.3166734D-5*10.D0
            ELSE
              FACT = TEMPW   *0.3166734D-5*10.D0
            ENDIF
            FACT2 = FACT * DELT_IONS*DBLE(INTER_HILL)* 
     &              DELT_IONS*DBLE(INTER_HILL)
          DO ICV = 1,NCOLVAR
              IF(CV_MASS(ICV) .EQ. 0.0D0) THEN
                CV_MASS(ICV) = FACT2 /
     &                         ((CSCL_FAC(1,ICV)*HLLW)*
     &                         (CSCL_FAC(1,ICV)*HLLW))
                WRITE(6,'(A,I5,A,f15.4,A)') 'Default Mass for CV #',ICV,
     &                          ' is assigned  = ',
     &                      CV_MASS(ICV)/SCMASS,
     &                      ' a.m.u.(a.u.)^2/(u.CV)^2'
              ELSE
                WRITE(6,'(A,I5,A,f15.4,A)') 'Mass for CV #',ICV,
     &                        ' is assigned  = ',
     &                      CV_MASS(ICV) ,' a.m.u.(a.u.)^2/(u.CV)^2'
                CV_MASS(ICV) =  SCMASS*CV_MASS(ICV)
              ENDIF


C       Harmonic Potential
              IF(KHARM(ICV) .EQ. 0.0D0) THEN
                KHARM(ICV) = FACT/
     &                 ((CSCL_FAC(1,ICV)*HLLW)*
     &                  (CSCL_FAC(1,ICV)*HLLW))
                WRITE(6,'(A,I5,A,f10.4,A)') 
     &                     'Default harmonic constant for CV #',ICV,
     &                     ' is assigned ',KHARM(ICV),' a.u./(u.CV)^2'
              ENDIF
              IF(.NOT. LKFIX) LCHEKHARM = .TRUE.
          ENDDO !ICV

          DO IWALK1=1,NWALK
            IPW=(IWALK1-1)*NCOLVAR
C     Initialization of CV Velocity
            IF((I_META.EQ.1).OR.RANDWALK) THEN
              CALL RINVELCV_MW(CV_VEL(IPW+1),CV_MASS,CSCL_FAC,NCOLVAR,
     &                    CV_TEMP0,EKINCV) 
              WRITE(6,'(2x,A,i10,A,f10.4)')'WALKER:',IWALK1, 
     &             'CV velocity have been initialized at T = ',CV_TEMP0
              WRITE(6,'(2x,A,i10,A,f10.4)')'WALKER:',IWALK1, 
     &             'the initial CV kinetic energy is = ',EKINCV
            ENDIF
            EKINCV_WALK(IWALK1)=EKINCV

            IF(LCVTC)THEN
              TEMP1=CV_TEMP+CV_DTEMP
              TEMP2=CV_TEMP-CV_DTEMP
            ENDIF
            TEMPP = CV_TEMP0
          END DO !IWALK1
        ELSE
            CALL AZZERO(CV_VEL,NCOLVAR*NWALK)
        ENDIF

C       ==--------------------------------------------------------------==
        IF(TCVMONITOR) THEN
            FILE9='cvmdck_mtd'
            CALL FILEOPEN(444,FILE9,FO_APP+FO_VERB,FERROR)
        ENDIF

        IF((I_META.EQ.1).OR.SKIPHILL_MW)THEN
          DO IWALK1=2,NWALK
             SKIPHILL(IWALK1)=.TRUE. !skip the first hill; setting height to zero
          END DO
        ENDIF

      ENDIF


C     ==--------------------------------------------------------------==
C     Calculate the Collective Variables and their Derivatives
      DO IWALK1=1,NWALK
        CALL COLVAROFR(TAUPW(1,1,1,IWALK1),TSCR)
        CALL DCOPY(NCOLVAR,CV_IST,1,CV_IST_SCR(1,IWALK1),1)
        CALL DCOPY(NODIM*NCOLVAR,DET_COLVAR,1,
     &                           DET_COLVAR_MW(1,1,IWALK1),1)
      END DO
      CALL DCOPY(NCOLVAR*NWALK,CV_IST_SCR,1,CV_IST,1)
      IF(IFIRST.EQ.0)CALL DCOPY(NCOLVAR*NWALK,CV_IST_SCR,1,CV_DYN,1)
      
C     ==--------------------------------------------------------------==
!     Initialization

      IF(IFIRST.EQ.0) THEN
        IF(I_META.EQ.1)THEN
          DO IWALK1=1,NWALK
            IPW=(IWALK1-1)*NCOLVAR
            DO ICV = 1,NCOLVAR
              HC_LAST(ICV+IPW) = CV_IST_SCR(ICV,IWALK1)
            ENDDO
          END DO
        ENDIF
        HLLH0 = HLLH
        IFIRST=1
        GOTO 200

      ENDIF

      HILL_ADD = .FALSE.
      DO IWALK1=1,NWALK
        HILL_ADD_MW(IWALK1)=.FALSE.
      END DO

      IF(TTRAJOVRR) NTRAJ=999999999

      I_CVST = I_CVST + 1
      DO IWALK1=1,NWALK
        I_CVST_MW(IWALK1) = I_CVST_MW(IWALK1) + 1
      END DO


      IW_CV=IW_CV+1

!     if a hill_add is true, it is known to all processors at this point
      IP = ISC2
      CALL CALC_POS_DYN_MW(TOLLM,I_CVST_MW,I_TEMP_MW,I_META,HC_LAST,
     &             SCR(IP),LSCR2,HILL_ADD_MW)

      IF(LMETARES) THEN
        FINAL = .TRUE.
        GOTO 9999
      END IF

C   each parent will write to unit 444 with its own file name assosicated seperately
      IF(TCVMONITOR .AND. MOD(IW_CV,WCV_FREQ) .EQ. 0) THEN
        WRITE(CHNUM,'(I5)') NCOLVAR*NWALK
        CALL XSTRING(CHNUM,IA,IE)
        LINEFORM =
     &     '(1X,2I7,'//CHNUM(IA:IE)//
     &     'E14.6,'//CHNUM(IA:IE)//'E14.6)'
        write(444,LINEFORM)IW_CV,NFI ,
     &       ((CV_IST_SCR(ICV,IWALK1),ICV=1,NCOLVAR),IWALK1=1,NWALK),
     &               (CV_DYN(ICV),ICV=1,NCOLVAR*NWALK)
        call my_flush(444)
      ENDIF


!     ==--------------------------------------------------------------==
!     New Step of Meta-Dynamics in the Space of CV
!      loop is done on the grand parent
      I_ADD=-1
      DO IWALK1=1,NWALK
       IF(HILL_ADD_MW(IWALK1)) THEN
         I_ADD=I_ADD+1
         IF(MOD(I_META,TR_FREQ) .EQ. 0 .OR. I_META .EQ. 1 ) THEN
            IF(TTRAJOVRR) NTRAJ=1
         ENDIF
         IF(.NOT. TCVANALYSIS) THEN
!         ==--------------------------------------------------------------==
!         Squared Norm of the Displacement in the CV space

          DISPLACEMENT = 0.0D0
          CALL AZZERO(DISPLACEMENT_MW,NWALK)
          IPW=(IWALK1-1)*NCOLVAR
          DO ICV = 1,NCOLVAR
            DO IWALK2=1,NWALK
              DIF_CV = CV_DYN(ICV+IPW)-HC_LAST(ICV+(IWALK2-1)*NCOLVAR) ! s - s(t_mtd) 
              IF (IANGCV(ICV) .EQ. 1 .AND. DIF_CV .GT. PI) THEN
                DIF_CV = DIF_CV - 2.0D0*PI
              ELSEIF (IANGCV(ICV) .EQ. 1 .AND. DIF_CV .LT. -PI) THEN
                DIF_CV = DIF_CV + 2.0D0*PI
              ENDIF
              FACT = DIF_CV/CSCL_FAC(1,ICV)
              DISPLACEMENT = DISPLACEMENT + FACT*FACT
              DISP2 = DISPLACEMENT
              DISPLACEMENT = DSQRT(DISPLACEMENT)
              IF(IWALK2.EQ.1)DISPLACEMENT_MW(IWALK1)=DISPLACEMENT
              DISPLACEMENT_MW(IWALK1)=
     &            MIN(DISPLACEMENT,DISPLACEMENT_MW(IWALK1))
            ENDDO !IWALK2
          END DO !ICV

!         Diffusion calculation
          IPW=(IWALK1-1)*NCOLVAR+1
           
          I_META_tmp=I_META+I_ADD
          HLLH_BAK=HLLH
          IF(SKIPHILL(IWALK1))HLLH=0.0d0 !if a hill has to be skipped, height is set to zero
          CALL CV_DIFFUSION(HC_LAST(IPW),CVD_IST(IPW),CVD_SCL(IPW),
     &            CV_SCL(IPW),I_META_tmp,
     &            SCR(ISC2),LSCR2,NTOT_ITER)

!         ==--------------------------------------------------------------==
!         Open output file
          IPW=(IWALK1-1)*NCOLVAR+1
          CALL CV_EXLAGR_OUT_MW(I_META_tmp,HC_LAST(IPW),F_HARM(IPW),
     &         F_HILL(IPW),F_WALL(IPW),
     &         CVD_IST(IPW),DISPLACEMENT_MW(IWALK1),EKINC,EKINP,IWALK1)
          IF(SKIPHILL(IWALK1))THEN
            HLLH=HLLH_BAK
            SKIPHILL(IWALK1)=.FALSE.
          END IF
!         ==--------------------------------------------------------------==
          IPW=(IWALK1-1)*NCOLVAR
          DO ICV = 1,NCOLVAR
            HC_LAST(ICV+IPW)   = CV_DYN(ICV+IPW)
          ENDDO
        ELSE
!         ==--------------------------------------------------------------==
!         Open output file
          CALL CV_EXLAGR_OUT2(I_META,EKINC,EKINP)

C         ==--------------------------------------------------------------==

        ENDIF
        I_TEMP_MW(IWALK1) = 0 
        I_CVST = 0
        I_CVST_MW(IWALK1) = 0 
       ENDIF        !HILL_ADD

!     ==--------------------------------------------------------------==
!     Calculation of the dynamics of the collective variables
!     A velocity Verlet Scheme is used

!     ==--------------------------------------------------------------==
!     Update Velocity : first half step
!                       using forces calculated in the previous step
        IPW=(IWALK1-1)*NCOLVAR
        DO ICV = 1,NCOLVAR
          FACT = DT_IONS/(2.D0*CV_MASS(ICV))
          CV_VEL(ICV+IPW) = CV_VEL(ICV+IPW) 
     &        + FACT*(F_HARM(ICV+IPW)+F_HILL(ICV+IPW)/
     &          CSCL_FAC(1,ICV)+F_WALL(ICV+IPW))
        END DO

!     ==--------------------------------------------------------------==
!     Update Position : second order dynamics

        DO ICV = 1,NCOLVAR
          FACT = DT_IONS
          IF (IANGCV(ICV) .EQ. 1 )  THEN
            IF(CV_DYN(ICV+IPW).LT.0.D0) 
     &         CV_DYN(ICV+IPW)= CV_DYN(ICV+IPW)+2.D0*PI
            IF(CV_DYN(ICV+IPW).GT.2.D0*PI) 
     &         CV_DYN(ICV+IPW)= CV_DYN(ICV+IPW)-2.D0*PI
          ELSEIF(IANGCV(ICV) .EQ. 2 )  THEN
            IF(CV_DYN(ICV+IPW).LT.0.D0) 
     &           CV_DYN(ICV+IPW)= -CV_DYN(ICV+IPW)
            IF(CV_DYN(ICV+IPW).GT. PI) 
     &           CV_DYN(ICV+IPW)= PI - (CV_DYN(ICV+IPW) - PI)
          ENDIF
          CV_DYN(ICV+IPW) = CV_DYN(ICV+IPW) + FACT*CV_VEL(ICV+IPW)
        ENDDO
      ENDDO !IWALK1=1,NWALK

!     ==--------------------------------------------------------------==
!     Calculate forces with the new positions

 200  CONTINUE
!     ==--------------------------------------------------------------==
!     Calculation of the contributions to the forces on ions 
!     due to the coupling with the collective variables
!     V_ARM = SUM_icv (K_icv(CV_IST_icv - CV_DYN_icv))^2

      CALL AZZERO(FI_HARM,NODIM*NWALK)
      DO IWALK1=1,NWALK
        IPW=(IWALK1-1)*NCOLVAR
        DO ICV = 1,NCOLVAR
          DIF_CV = CV_IST_SCR(ICV,IWALK1)-CV_DYN(ICV+IPW)
          IF (IANGCV(ICV) .EQ. 1 .AND. DIF_CV .GT. PI) THEN
            DIF_CV = DIF_CV -2.0D0*PI
          ELSEIF (IANGCV(ICV) .EQ. 1 .AND. DIF_CV .LT. -PI) THEN
           DIF_CV = DIF_CV +2.0D0*PI
          ENDIF
          FACT = KHARM(ICV)*2.0D0*DIF_CV
          DO IDOF = 1,NODIM
            FI_HARM(IDOF,IWALK1) = FI_HARM(IDOF,IWALK1) - 
     &                      FACT *  DET_COLVAR_MW(IDOF,ICV,IWALK1) 
          ENDDO
        ENDDO!ICV
      END DO !IWALK1

C     ==--------------------------------------------------------------==
C     Update forces on ions

        CALL AZZERO(TAUPW_SCR,3*NAX*NSX*NWALK)
        DO IWALK1=1,NWALK
          CALL GETTAU(TAUPW_SCR(1,1,1,IWALK1),FI_HARM(1,IWALK1))  
        END DO
C     ==--------------------------------------------------------------==
C     Calculate contribution of the Harmonic Potential (recall)

      CALL AZZERO(F_HARM,NCOLVAR*NWALK)
      DO IWALK1=1,NWALK
        IPW=(IWALK1-1)*NCOLVAR
        DO ICV = 1,NCOLVAR
          DIF_CV = CV_IST_SCR(ICV,IWALK1)-CV_DYN(ICV+IPW)
          IF (IANGCV(ICV) .EQ. 1 .AND. DIF_CV .GT. PI) THEN
            DIF_CV = DIF_CV - 2.0D0*PI
          ELSEIF (IANGCV(ICV) .EQ. 1 .AND. DIF_CV .LT. -PI) THEN
            DIF_CV = DIF_CV + 2.0D0*PI
          ENDIF
          F_HARM(ICV+IPW) =  KHARM(ICV)*2.0D0*DIF_CV
        ENDDO
      END DO !IWALK1

C     Calculate hills contribution to forces
      CALL AZZERO(F_HILL,NCOLVAR*NWALK)
      DO IWALK1=1,NWALK
       IF(HILL_ADD_MW(IWALK1))I_META = I_META + 1 
       IPW=(IWALK1-1)*NCOLVAR+1
       IF(LHILLS) THEN
         IF(HLORE) THEN
           CALL HILLS_LOR(CV_DYN(IPW),NCOLVAR,I_META,
     &                    NTOT_ITER,F_HILL(IPW),
     &                    SCR(ISC2),LSCR2,1,0)
         ELSEIF(HRATIO) THEN
           CALL HILLS_RATIO(CV_DYN(IPW),NCOLVAR,I_META,
     &          NTOT_ITER,F_HILL(IPW),
     &          HC_LAST(IPW),SCR(ISC2),LSCR2,I_CVST,1,0)
         ELSEIF(HSHIFT) THEN
           CALL HILLS_SALS_SHIFT(CV_DYN(IPW),NCOLVAR,I_META,
     &          NTOT_ITER,F_HILL(IPW),
     &          HC_LAST(IPW),SCR(ISC2),LSCR2,I_CVST,1,0)
         ELSE IF(SPHERE) THEN
           CALL HILLS(CV_DYN(IPW),NCOLVAR,I_META,
     &                    NTOT_ITER,F_HILL(IPW),
     &                    SCR(ISC2),LSCR2,1,0)
         ELSE
           CALL HILLS_SALS(CV_DYN(IPW),NCOLVAR,I_META,
     &          NTOT_ITER,F_HILL(IPW),
     &          HC_LAST(IPW),SCR(ISC2),LSCR2,I_CVST,1,0)
         ENDIF!HLORE
       ENDIF !LHILLS
      END DO !IWALK1

!     Calculate walls contribution to forces
      CALL AZZERO(F_WALL,NCOLVAR*NWALK)
      DO IWALK1=1,NWALK
        IPW=(IWALK1-1)*NCOLVAR
        MAXF2 = 0.0D0
        DO ICV = 1,NCOLVAR
          IF(IBOUND(ICV) .EQ. 1) THEN
            CALL SETWALL(CV_DYN(ICV+IPW),VBOUND(1,ICV),
     &                   F_WALL(ICV+IPW))
          ELSEIF(IBOUND(ICV) .EQ. 2) THEN
            CALL SETWALL_OLD(CV_DYN(ICV+IPW),TYCVAR(ICV),VBOUND(1,ICV),
     &                     F_WALL(ICV+IPW))
          ENDIF 
          IF(GRANDPARENT)THEN
            IF(F_WALL(ICV) .NE. 0.0D0) THEN
              write(6,'(72A)') ('*',i = 1,72) 
              write(6,'(10x,A,I5,A,I5)') 
     &           'WARNING: Boundaries active for WALKER ',IWALK1,
     &           ' on CV ', ICV
              write(6,'(72A)')  ('*',i = 1,72)
            ENDIF
          END IF
        END DO
      ENDDO
!     ==--------------------------------------------------------------==
!     Update Velocity : second half step 
      DO IWALK1=1,NWALK
        IPW=(IWALK1-1)*NCOLVAR
        DO ICV = 1,NCOLVAR
          FACT = DT_IONS/(2.D0*CV_MASS(ICV))
          CV_VEL(ICV+IPW) = CV_VEL(ICV+IPW) 
     &      + FACT*(F_HARM(ICV+IPW)+F_HILL(ICV+IPW)/
     &        CSCL_FAC(1,ICV)+F_WALL(ICV+IPW))
        ENDDO
      END DO
!     ==--------------------------------------------------------------==
!     Calculation of   Temperature of CV: CV_TEMP
!     If CV_TEMP_CONTROL, check the condition for rescaling T

      CALL RSCVELCV(TEMP1,TEMP2,TEMPP)
!     ==--------------------------------------------------------------==
!     IF MAX_STEP reached or EXIT file exist

      IF(INFI.EQ.NOMORE .OR. I_META-1.EQ.NTOT_ITER) THEN
        EXSOFT = .TRUE.
        DO IWALK1=1,NWALK
          CALL DCOPY(NCOLVAR,CV_IST_SCR(1,IWALK1),1,CV_IST,1)
          IF(GRANDPARENT)CALL  COLVARPR
        END DO

        IF(TCVMONITOR) THEN
          CALL FILECLOSE(444)
        ENDIF
      ENDIF
 9999 CONTINUE


!We need to sum the FHILLS on all processors. H^mtd=H_wal1+H_walk2+H_walk3
      IF(.NOT.GRANDPARENT)CALL AZZERO(TAUPW_SCR,3*NAX*NSX*NWALK) !Zero on all processors except grandparent
      CALL AZZERO(FHILLS,3*NAX*NSX)

      MSGLEN = NCOLVAR * NWALK * 8
      CALL MY_SYNC(SUPERGROUP)
      CALL MY_BCAST(CV_DYN,MSGLEN,SUPERSOURCE,SUPERGROUP)

      MSGLEN = 1 * 8/IRAT
      CALL MY_SYNC(SUPERGROUP)
      CALL MY_BCAST(EXSOFT,MSGLEN,SUPERSOURCE,SUPERGROUP)

      MSGLEN = 1 * 8/IRAT
      CALL MY_SYNC(SUPERGROUP)
      CALL MY_BCAST(I_META ,MSGLEN,SUPERSOURCE,SUPERGROUP)

      MSGLEN = 1 * 8/IRAT
      CALL MY_SYNC(SUPERGROUP)
      CALL MY_BCAST(I_CVST ,MSGLEN,SUPERSOURCE,SUPERGROUP)

      MSGLEN = NWALK * 8/IRAT
      CALL MY_SYNC(SUPERGROUP)
      CALL MY_BCAST(I_CVST_MW,MSGLEN,SUPERSOURCE,SUPERGROUP)

      MSGLEN = NWALK * 8
      CALL MY_SYNC(SUPERGROUP)
      CALL MY_BCAST(EKINCV_WALK,MSGLEN,SUPERSOURCE,SUPERGROUP)
      CALL MY_BCAST(VHARM_WALK,MSGLEN,SUPERSOURCE,SUPERGROUP)
      EKINCV = EKINCV_WALK(IWALK)
      VHARM  = VHARM_WALK(IWALK)

      MSGLEN = 3*NAX*NSX*8*NWALK
      CALL MY_SYNC(SUPERGROUP)
      CALL MY_BCAST(TAUPW_SCR,MSGLEN,SUPERSOURCE,SUPERGROUP)
      IF(PARENT)THEN
        CALL DCOPY(3*NAX*NSX,TAUPW_SCR(1,1,1,IWALK),1,FHILLS,1)
      END IF
      CALL MY_SYNC(SUPERGROUP)

      IF(TLOCALIZESPIN) THEN
        CALL MY_SYNC(SUPERGROUP)
        MSGLEN = NCOLVAR*3 *8
        CALL MY_BCAST(RCC0,MSGLEN,SUPERSOURCE,SUPERGROUP)
      ENDIF

C     WRITE RESTART
      DO IWALK1=1,NWALK
        IF((MOD(I_META,ST_FREQ) .EQ. 0  .AND. I_CVST_MW(IWALK1).EQ.0)
     &     .OR. EXSOFT .OR. LMETARES) LMETARES = .TRUE.
      END DO
C
      IF(LMETARES.AND.I_META.GT.1.AND.GRANDPARENT)THEN
        CALL DCOPY(NWALK*NCOLVAR,CV_VEL,1,CV_IST_SCR,1)
        DO IWALK1=1,NWALK
          CALL DCOPY(NCOLVAR,CV_IST_SCR(1,IWALK1),1,CV_VEL,1)
          CALL MW_FILENAME('MTD_RESTART_',FMTDRES,IWALK1)
          CALL WMTDRES_MW(50,NTOT_ITER,I_META,IWALK1)
        END DO
        CALL DCOPY(NWALK*NCOLVAR,CV_IST_SCR,1,CV_VEL,1)
      END IF

      CALL MY_SYNC(SUPERGROUP)
      MSGLEN = 1 * 8/IRAT
      CALL MY_BCAST(LMETARES,MSGLEN,SUPERSOURCE,SUPERGROUP)

C     QUENCH BO
      IF(.NOT.TMDBO.AND.
     &    MOD(I_META,QW_FREQ) .EQ. 0  .AND. I_CVST .EQ. 0) 
     &    LQUENCH = .TRUE.

      IF(FINAL)  THEN
C     Deallocate
        IF(GRANDPARENT)THEN
          CALL FREEM(IP_HC_LAST)
          CALL FREEM(IP_CV_PATH)
          CALL FREEM(IP_CSCL_VAL)
          CALL FREEM(IP_HLLW_VAL)
          CALL FREEM(IP_HLLH_VAL)
          CALL FREEM(IP_CV_VEL)
          CALL FREEM(IP_CVD_IST)
          CALL FREEM(IP_F_AVER)
          CALL FREEM(IP_F_HARM)
          CALL FREEM(IP_F_HILL)
          CALL FREEM(IP_F_WALL)
        ENDIF
        CALL FREEM(IP_I_TEMP_MW)
        CALL FREEM(IP_I_CVST_MW)
        CALL FREEM(IP_CV_DYN)
        CALL FREEM(IP_TAUPW_SCR)
        CALL FREEM(IP_TAUPW)
      END IF
C     ==--------------------------------------------------------------==
C     Halt time for the routine
      CALL MY_SYNC(SUPERGROUP)
      CALL TIHALT('META_EX_MW',ISUB)
      RETURN
      END
C     ==================================================================

