c     ==================================================================
      subroutine v1xc_p(drhoe,v,psi,scr,lscr)
c     ==--------------------------------------------------------------==
C     CALCULATES the exchange-correlation contribution of the 
C     perturbation density n(1) and adds it to the potential (v)
C     and energy (exc1) terms, and ADDS the final potential function
C     v_xc[n(1)] to the total local potential, V.
C     INPUT:  drhoe = n1 in R space
C     \       v     = the (spin up+down) potential in R space
C     OUTPUT: v     = the xc-contribution is added, and the
C     \               spin splitting is done. Output is
C     \               v(nnr1,1) alpha spin potential
C     \               v(nnr1,2) alpha spin potential
C     \       exc1  = the xc-energy
C
C     The routine works in fundamentally different ways for
C     LDA and GGA:
C
C     In LDA, we calculate once for all the expression
C     k_xc(r,rp) = d^2 e_xc[n]  /  dn(rp) dn(r)
C     and then, for each step, we do
C     v1_xc(r) = Int d^3rp  k_xc(r,rp) * n1(rp).
C
C     For GGA, we compute for each step the second derivative
C     times n1 by a finite difference technique:
C     v1_xc(r) =  Int d^3r   k_xc(rp,r)  * n1(rp)
C     \        =  v_xc(n0+eps*n1)(r) - v_xc(n0-eps*n1)(r) / 2 eps
C     or even simpler:
C     \        =  v_xc(n0+eps*n1)(r) - v_xc(n0)(r)  / eps
C
C
C     The array VXC0 has two different functions in those cases.
C     LDA/analytic calculation:  VXC0 = d^2 Exc / d n^2
C     GGA/finite differences:    VXC0 = Vxc [ n0 ]
c     ==--------------------------------------------------------------==
      IMPLICIT NONE
      include 'system.h'
      include 'fft.inc'
      include 'spin.inc'
      include 'response_p.inc'
      include 'nlcc.inc'
C     Parameters:
      integer   lscr
      real*8    drhoe(nnr1,nlsd),v(nnr1,nlsd),psi(*),scr(*)
C     Variables
      complex*16 vxctemp(maxfft,nlsd)
      real*8    rho2,smooth,v1xc,gceps2
      real*8    gradients(nnr1,4*nlsd),rho01(nnr1,nlsd),
     &     vxc1(nnr1,nlsd),vscr(nnr1*3)
      pointer   (ip_gradients,gradients),(ip_vxc1,vxc1),
     &     (ip_rho01,rho01),
     &     (ip_vxctemp,vxctemp),(ip_vscr,vscr)
      integer   ifirst,ir,isub,scr_index
      save      ifirst
      data      ifirst /0/
      character*30 tag
      real*8     eps,rho,v1
      parameter  (eps=0.002d0)
c     ==--------------------------------------------------------------==
      call tiset('     V1XC_P',isub)
      if (tinlc) call stopgm('V1XC_P',
     &     'NonLinear CoreCorrection not implemented.')
#ifdef _BGL 
      scr_index    = 1
      CALL GET_ADDR(IP_vscr,scr(scr_index))
      scr_index    = scr_index + 3*nnr1 + mod(3*nnr1,2)
      CALL GET_ADDR(IP_vxctemp,scr(scr_index))
      scr_index    = scr_index + 2*maxfft*nlsd
      CALL GET_ADDR(IP_vxc1,scr(scr_index))
      scr_index    = scr_index + max(2*nhg*nlsd,nnr1*nlsd) +
     *               mod( max(2*nhg*nlsd,nnr1*nlsd),2)
      CALL GET_ADDR(IP_rho01,scr(scr_index))
      scr_index    = scr_index + nnr1*nlsd + mod(nnr1*nlsd,2)
      CALL GET_ADDR(IP_gradients,scr(scr_index))
      scr_index    = scr_index + 4*nnr1*nlsd
#else
      scr_index    = 1
      CALL GET_ADDR(IP_vscr,scr(scr_index))
      scr_index    = scr_index + 3*nnr1
      CALL GET_ADDR(IP_vxctemp,scr(scr_index))
      scr_index    = scr_index + 2*maxfft*nlsd
      CALL GET_ADDR(IP_vxc1,scr(scr_index))
      scr_index    = scr_index + max(2*nhg*nlsd,nnr1*nlsd)
      CALL GET_ADDR(IP_rho01,scr(scr_index))
      scr_index    = scr_index + nnr1*nlsd
      CALL GET_ADDR(IP_gradients,scr(scr_index))
      scr_index    = scr_index + 4*nnr1*nlsd
#endif
C     GRADIENTS contains 4 densities for each spin.

      tag = ' approx 10*nnr1*nlsd '
      call  test_scr('V1XC_P',tag,lscr,scr_index-1)
c     ==--------------------------------------------------------------==
         do ir=1,nnr1*nlsd
            rho01(ir,1) = max(0d0, rho0(ir,1) + eps * drhoe(ir,1))
         enddo                  ! prevent rho from being negative
         call vxcofrho(rho01, vxc1, vxctemp, vscr, psi, gradients)

         do ir=1,nnr1*nlsd
            rho01(ir,1) = max(0d0, rho0(ir,1) - eps * drhoe(ir,1))
         enddo
         call vxcofrho(rho01, vxc0, vxctemp, vscr, psi, gradients)

         exc1 = 0.d0
         if (.NOT. TLSD) then
            do ir=1,nnr1
               if (rho0(ir,1) .gt. gceps) then
                  v1xc =  (vxc1(ir,1)-vxc0(ir,1)) / (2d0*eps)
                  v(ir,1) = v(ir,1) + v1xc
                  exc1 = exc1 + v1xc*drhoe(ir,1)
               endif
            enddo
         else                   ! with spin split:


            do ir=1,nnr1
               if (rho0(ir,1) .gt. gceps) then
C     beta spin:
                  v1xc = (vxc1(ir,2)-vxc0(ir,2)) / (2d0*eps)
                  v(ir,2) = v(ir,1) + v1xc
                  exc1 = exc1 + v1xc*drhoe(ir,2)
C     alpha spin:
                  v1xc = (vxc1(ir,1)-vxc0(ir,1)) / (2d0*eps)
                  v(ir,1) = v(ir,1) + v1xc
                  exc1 = exc1 + v1xc*(drhoe(ir,1)-drhoe(ir,2))
               else
                  v(ir,2) = v(ir,1)
               endif
            enddo
         endif
         exc1    = 0.5d0 * exc1*omega/dble(nr1s*nr2s*nr3s)
c     ==--------------------------------------------------------------==
      call tihalt('     V1XC_P',isub)
      return
      end
c     ==================================================================


c     ==================================================================
      SUBROUTINE vxcofrho(rho, vxc, vxctemp, vscr, psi, gradients)
c     ==--------------------------------------------------------------==
C     INPUT: rho in R-space, (alpha+beta, beta) storage (untouched)
C     OUTPUT: vxc in R-space, (alpha, beta) storage
C     Rest is scratch.
      implicit none
      include 'system.h'
      include 'spin.inc'        !nlsd
      include 'cppt.inc'
      include 'fft.inc'

      real*8   rho(nnr1,nlsd),vxc(nnr1,nlsd),vscr(*),
     &     psi,gradients(nnr1,4*nlsd)
      complex*16 vxctemp(maxfft,nlsd),vxc_g(nhg,nlsd)
      real*8 sgcx,sgcc,ssxc,vvxc
      integer  ir
      pointer (ip_vxc_g,vxc_g)
c     ==--------------------------------------------------------------==
      IF(TTAU) THEN
        CALL STOPGM('vxcofrho','META FUNCTIONALS NOT IMPLENTED')
      ENDIF
      CALL GET_ADDR(IP_vxc_g,vxc)
      call zazzero(vxctemp, maxfft*nlsd)
      call azzero(vxc,nnr1*nlsd)

      call xcener(ssxc,vvxc,rho,rho,vxctemp)
C     The non-grad-corrected potential is now in VXCtemp
C     (in R-space, in complex*16 storage)

      if (TGC) then
         call fwfft(vxctemp(1,1))
         call zgthr(nhg,vxctemp(1,1),vxc_g(1,1),nzh)
C     The non-grad-corrected potential is now in VXC 
C     (in G-space, packed storage)

         if (tlsd) then
            call fwfft(vxctemp(1,2))
            call zgthr(nhg,vxctemp(1,2),vxc_g(1,2),nzh)

            call DAXPY(nnr1,-1.d0,rho(1,2),1, rho(1,1),1)
            call graden(rho(1,1),psi,gradients(1,1),vscr)
            call graden(rho(1,2),psi,gradients(1,5),vscr)
            call GCLSD(sgcx,sgcc,rho,vxctemp,vxc_g,vscr,gradients,
     &                 .FALSE.)
            call DAXPY(nnr1,+1.d0,rho(1,2),1, rho(1,1),1)
         else
            call graden(rho(1,1),psi,gradients(1,1),vscr)
            call GCENER(sgcx,sgcc,rho,vxctemp,vxc_g,vscr,gradients,
     &           .FALSE.)
         endif
      endif                     ! if tgc = if gradient correction


C     XCENER and GCener/GCLSD return the total, gradient-corrected
C     xc-potential in R-space (complex*16 storage) in vxctmp.
      call DCOPY(nnr1,vxctemp(1,1),2,vxc,1)
      if (TLSD) call DCOPY(nnr1,vxctemp(1,2),2,vxc(1,2),1)

      return
      end
c     ==================================================================







C     ==================================================================
      SUBROUTINE vxc0_lda(vxc0,rho0)
C     ==------------------------------------------------------------==
c     calculates the 2nd functional derivative of the exchange-
C     correlation potential used in perturbation theory,
C     FOR LDA.
C     INPUT:  rho0 = the (nnr1) dimension density array containing
C     \              the gnd state density (direct space, real*8)
C     OUTPUT: vxc0 = d^2 E_xc [n0] / dn^2(r)
C     ==------------------------------------------------------------==
      implicit none
      include 'system.h'
      include 'func.inc'
      include 'cnst.inc'
C     Parameters:
      real*8 rho0(*),vxc0(*)
C     Variables for the second derivative:
      REAL*8 A0U,A1U,A2U,A3U,B1U,B2U,B3U,B4U,COEF,
     &    rho,root_rho,rs,drsdn,drsdn2,dexcdrs,dexcdrs2,dmu
      PARAMETER (A0U=.4581652932831429D0,
     *    A1U=2.217058676663745D0,
     *    A2U=0.7405551735357053D0,
     *    A3U=0.01968227878617998D0)
      PARAMETER (B1U=1.0D0,
     *    B2U=4.504130959426697D0,
     *    B3U=1.110667363742916D0,
     *    B4U=0.02359291751427506D0)
      PARAMETER (COEF=0.620350490901d0)
C     ==------------------------------------------------------------==
      INTEGER ir
C     ==------------------------------------------------------------==
      if(mfxcc.ne.9.and.mfxcc.ne.3) then
         call stopgm('v0xc_lda','ONLY LDA')
      else
         do ir=1,nnr1
            rho = rho0(ir)
            if (rho .LT. GCEPS) then
               vxc0(ir) = 0d0
            else
C..   Pade approximation to LDA functional
               root_rho=rho**(1.d0/3.d0)
               rs=coef/root_rho
               drsdn=-coef/(3.d0*root_rho**4)
               drsdn2=(4.d0*coef)/(9.D0*root_rho**7)
               dexcdrs=(A0U*(B1U + 
     -              rs*(2*B2U + 3*B3U*rs + 4*B4U*rs**2)) +
     -              rs**2*(A1U*B2U - 2*A3U*B1U*rs + 2*A1U*B3U*rs -
     -              A3U*B2U*rs**2 + 
     -              3*A1U*B4U*rs**2 + A3U*B4U*rs**4 +
     -              A2U*(-B1U + B3U*rs**2 + 2*B4U*rs**3)))/
     -              (rs**2*(B1U + rs*(B2U + rs*(B3U + B4U*rs)))**2)
               dexcdrs2= (2*rs*(B2U + 3*rs*(B3U + 2*B4U*rs))*
     -              (A0U + rs*(A1U + rs*(A2U + A3U*rs)))*
     -              (B1U + rs*(B2U + rs*(B3U + B4U*rs))) -
     -              2*rs**2*(A2U + 3*A3U*rs)*
     -              (B1U + rs*(B2U + rs*(B3U + B4U*rs)))**2 +
     -              2*rs*(A1U + rs*(2*A2U + 3*A3U*rs))*
     -              (B1U + rs*(B2U + rs*(B3U + B4U*rs)))*
     -              (B1U + rs*(2*B2U + rs*(3*B3U + 4*B4U*rs))) -
     -              2*(A0U + rs*(A1U + rs*(A2U + A3U*rs)))*
     -              (B1U + rs*(2*B2U + rs*(3*B3U + 4*B4U*rs)))**2)/
     -              (rs**3*(B1U + rs*(B2U + rs*(B3U + B4U*rs)))**3)
               dmu=2.d0*drsdn+rho*drsdn2

               vxc0(ir)=dexcdrs*dmu+rho*dexcdrs2*drsdn*drsdn

            endif
         enddo
      endif
      return
      end
C     ==--------------------------------------------------------------



c     ==================================================================
      subroutine give_scr_v1xc_p(lvofrho,tag)
c     ==--------------------------------------------------------------==
      implicit none
      include 'system.h'        !nhg
      include 'spin.inc'        !nlsd nlsx
      include 'fft.inc'        !nlsd nlsx

c     arguments
      integer   lvofrho
      character tag*30

      lvofrho  =  3*nnr1 + 2*maxfft*nlsd + max(2*nhg*nlsd,nnr1*nlsd)
     &            + nnr1*nlsd + 4*nnr1*nlsd

      tag      =  'v1xc_p      '
c     ==--------------------------------------------------------------==
      return
      end
c     ==--------------------------------------------------------------==
