CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
C                                                                C
C     Computes the real spherical harmonic of order lm           C
C     Here |m|<=L, L>=0                                          C
C     Cohen-Tannoudji convention: Y0L (0) = SQRT(2L+1/4Pi)       C
C                                                                C
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
      REAL*8 FUNCTION RSPHGEN(L,M,Vx,Vy,Vz)
      IMPLICIT NONE

      INTEGER L,M
      REAL*8 Vx,Vy,Vz

C     including the constants(PI) definition
      INCLUDE 'cnst.inc'

      REAL*8 Prefactor
      REAL*8 RESULT
      REAL*8 R_XYZ,ctheta,phi
      REAL*8 psi_of_M,plm,plgndr
      EXTERNAL plgndr
      REAL*8 D_FAC,D_FAC_RAT
      EXTERNAL D_FAC,D_FAC_RAT
      
#ifdef PARALLEL
      call stopgm('LEGENDRE_P','RSPHGEN not parallel.')
#endif

C     First calculating the norm of the vector
      R_XYZ=DSQRT(Vx**2+Vy**2+Vz**2)

C     is that a zero vector?
      IF (R_XYZ.EQ.0.D0) THEN
C         YES!
          IF (M.EQ.0) THEN
C             same value for all L, exit
C             RSPHGEN=1.D0
              RSPHGEN=0.D0
              RETURN
          ELSE
C             otherwise, zero...
              RSPHGEN=0.D0
              RETURN
          ENDIF
      ENDIF
             
C     From here onwards, R>0
C     calculating cos(theta) [z=R cos(theta)]
      ctheta=Vz/R_XYZ

C     special case if theta=0 (Merzbacher p251)
      IF (DABS(ctheta).EQ.1.D0) THEN
         IF (M.EQ.0) THEN
             RSPHGEN=1.D0
            RETURN
         ELSE
            RSPHGEN=0.D0
            RETURN
         ENDIF
      ENDIF

C     calculating psi of m 
C     m=0
      IF (M.EQ.0) THEN
         psi_of_M=1.D0
         prefactor=1.D0
         plm=plgndr(L,0,ctheta)
      ENDIF

C     m>0
      IF (M.GT.0) THEN
C        calculating phi [x=R sin(theta) cos(phi), y=R sin(theta) sin(phi)]
         phi=DATAN2(Vy,Vx)
  
         psi_of_M=DSQRT(2.D0)*DCOS(DBLE(M)*phi)
         prefactor=(-1)**M *DSQRT(D_FAC_RAT(L-M,L+M))
         plm=plgndr(L,M,ctheta)
      ENDIF 

C     m<0
      IF (M.LT.0) THEN
C        calculating phi
         phi=DATAN2(Vy,Vx)
  
         psi_of_M=DSQRT(2.D0)*DSIN(DBLE(-M)*phi)
         prefactor=(-1)**M *DSQRT(D_FAC_RAT(L+M,L-M))
         plm=plgndr(L,-M,ctheta)
      ENDIF 

C     Now we can calculate the value of the spherical harmonic...

      RESULT=prefactor*plm*psi_of_M

C     copying the result
      RSPHGEN=RESULT

C     IF the resulting value is a NAN, output debug informations
      IF (RESULT.NE.RESULT) THEN
          write(6,*) '-SSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSSS-'
          write(6,*) 'legendre: L=',L,'M=',M,
     $               'V(',Vx,',',Vy,',',Vz,')'

          IF (M.NE.0) THEN
             write(6,*) 'phi =',phi
          ENDIF
          write(6,*) 'ctheta =',ctheta
          write(6,*) 'prefac =',prefactor
          write(6,*) 'psiofM=',psi_of_M
          write(6,*) 'Result = ',RESULT

          write(6,*) 'L-M!=',D_FAC(L-M)
          write(6,*) 'L+M!=',D_FAC(L+M)
          write(6,*) 'D_FAC_RAT(L+M,L-M)=',D_FAC_RAT(L+M,L-M)
          write(6,*) 'D_FAC_RAT(L-M,L+M)=',D_FAC_RAT(L-M,L+M)
          write(6,*) '-EEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEE-'
      ENDIF

      RETURN
      END


C     calculates the factorial of a returns dble
      REAL*8 FUNCTION D_FAC(A)
      IMPLICIT NONE
      INTEGER A

      INTEGER I
      REAL*8 resl 

      resl=1.D0
      IF (A.GT.1) THEN
         DO I=1,A
            resl=resl*DBLE(I)
         ENDDO
      ENDIF

      D_FAC=resl

      RETURN
      END

c     calculates the binomial coefficient n \over k
      REAL*8 FUNCTION D_BINOM(N,K) 
      IMPLICIT NONE
      INTEGER N,K
      REAL*8 D_FAC
      EXTERNAL D_FAC

      D_BINOM=0.0D0
      IF((K.GE.0).AND.(K.LE.N)) THEN
        D_BINOM=D_FAC(N)/(D_FAC(N-K)*D_FAC(K))
      ENDIF
      RETURN
      END

CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
C                                           C
C     calculates the ratio of (a!)/(b!)     C
C     and returns the result as a dble      C
C                                           C
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
      REAL*8 FUNCTION D_FAC_RAT(A,B)
      IMPLICIT NONE
      INTEGER A,B

      INTEGER I
      REAL*8 RESL 

      RESL=1.D0
      
      IF(A.LT.0)THEN
        write(6,*) 'error in D_FAC_RAT, negative A'
      ENDIF
      IF(B.LT.0)THEN
        write(6,*) 'error in D_FAC_RAT, negative B'
      ENDIF
      
C     A>B then we only need to calculate A*(A-1)*...*(B+1)
      IF (A.GT.B) THEN
        DO I=(B+1),A
          RESL=RESL*DBLE(I)
        ENDDO
C     B<A then we need to calculate B*(B-1)*...*(A+1) and invert it
      ELSEIF (B.GT.A) THEN
         DO I=(A+1),B
           RESL=RESL*DBLE(I)
         ENDDO
         RESL=1.0D0/RESL
      ENDIF
      
      D_FAC_RAT=RESL
      RETURN
      END

CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
C                                                                C
C     Computes the associated Legendre polynomial Pml (x).       C
C     Here m and l are integers satisfying 0 <= m <= l,          C
C     while x lies in the range -1 <= x <= 1                     C
C     Note:                                                      C
C     Numerical recipe subroutine, changed to double precision   C
C                                                                C
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
      REAL*8 FUNCTION plgndr(l,m,x)
      IMPLICIT NONE

      INTEGER l,m
      REAL*8 x
      
      INTEGER i,ll
      REAL*8 fact,pll,pmm,pmmp1,somx2
#ifdef PARALLEL
      call stopgm('LEGENDRE_P','PLGNDR not parallel.')
#endif

      if (m.lt.0.or.m.gt.l.or.abs(x).gt.1.d0) THEN
         call stopgm('LEGENDRE_P','bad arguments in plgndr')
      ENDIF

C     Compute Pmm
      pmm=1.D0

      if (m.gt.0) then
          somx2=DSQRT((1.D0-x)*(1.D0+x))
          fact=1.D0
          do i=1,m
             pmm=-pmm*fact*somx2
             fact=fact+2.D0
          enddo
      endif

      if (l.eq.m) then
         plgndr=pmm
      else

C        Compute P mm+1
         pmmp1=x*DBLE(2*m+1)*pmm 

         if (l.eq.m+1) then
            plgndr=pmmp1
         else

C           Compute P ml, l >m + 1
            do ll=m+2,l
               pll=(x*DBLE(2*ll-1)*pmmp1-DBLE(ll+m-1)*pmm)
     $              /DBLE(ll-m)
               pmm=pmmp1
               pmmp1=pll
             enddo
             plgndr=pll
         endif
      endif
      RETURN
      END
